#include-once
#include <Direct2D.au3>
#include <DirectWriteConstants.au3>
#include <WinApi.au3>

; #INDEX# =======================================================================================================================
; Title .........: DirectWrite
; AutoIt Version : 3.3.7.20++
; Language ......: English
; Description ...:
; Author ........: trancexx, Eukalyptus
; Modified ......:
; Dll ...........: dwrite.dll
; ===============================================================================================================================

Global Const $__g_hDWriteDLL = DllOpen("dwrite.dll")


;#######################################################################################################################################################################
;# IDwriteBitmapRenderTarget
;#######################################################################################################################################################################
#cs    IDwriteBitmapRenderTarget
	Method ...: DrawGlyphRun (float;float;uint;struct*;struct*;uint;ptr*)
	Info .....:     Draws a run of glyphs to a bitmap target at the specified position.
	Param ....:         BaselineOriginX  <FLOAT>                    The horizontal position of the baseline origin, in DIPs, relative to the upper-left corner of the DIB.
	Param ....:         BaselineOriginY  <FLOAT>                    The vertical position of the baseline origin, in DIPs, relative to the upper-left corner of the DIB.
	Param ....:         MeasuringMode    <DWRITE_MEASURING_MODE>    The measuring method for glyphs in the run, used with the other properties to determine the rendering mode.
	Param ....:         GlyphRun         <DWRITE_GLYPH_RUN>         The structure containing the properties of the glyph run.
	Param ....:         RenderingParams  <IDWriteRenderingParams>   The object that controls rendering behavior.
	Param ....:         TextColor        <COLORREF>                 The foreground color of the text.
	Param ....:         BlackBoxRect     <RECT>                     The optional rectangle that receives the bounding box (in pixels not DIPs) of all the pixels affected by drawing the glyph run. The black box rectangle may extend beyond the dimensions of the bitmap.

	Method ...: GetCurrentTransform (ptr*)
	Info .....:     Gets the transform that maps abstract coordinates to DIPs. By default this is the identity transform. Note that this is unrelated to the world transform of the underlying device context.
	Param ....:         Transform  <DWRITE_MATRIX>  When this method returns, contains a transform matrix.

	Method ...: GetMemoryDC ()
	Info .....:     Gets a handle to the memory device context.
	Param ....:         This method has no parameters.

	Method ...: GetPixelsPerDip ()
	Info .....:     Gets the number of bitmap pixels per DIP.
	Param ....:         This method has no parameters.

	Method ...: GetSize (ptr*)
	Info .....:     Gets the dimensions of the target bitmap.
	Param ....:         Size  <SIZE>  Returns the width and height of the bitmap in pixels.

	Method ...: Resize (uint;uint)
	Info .....:     Resizes the bitmap.
	Param ....:         Width   <UINT32>  The new bitmap width, in pixels.
	Param ....:         Height  <UINT32>  The new bitmap height, in pixels.

	Method ...: SetCurrentTransform (struct*)
	Info .....:     Sets the transform that maps abstract coordinate to DIPs (device-independent pixel). This does not affect the world transform of the underlying device context.
	Param ....:         Transform  <DWRITE_MATRIX>   Specifies the new transform. This parameter can be NULL, in which case the identity transform is implied.

	Method ...: SetPixelsPerDip (float)
	Info .....:     Sets the number of bitmap pixels per DIP (device-independent pixel). A DIP is 1/96 inch, so this value is the number if pixels per inch divided by 96.
	Param ....:         PixelsPerDip  <FLOAT>  A value that specifies the number of pixels per DIP.

#ce
; _DWrite_BitmapRenderTarget_DrawGlyphRun
; _DWrite_BitmapRenderTarget_GetCurrentTransform
; _DWrite_BitmapRenderTarget_GetMemoryDC
; _DWrite_BitmapRenderTarget_GetPixelsPerDip
; _DWrite_BitmapRenderTarget_GetSize
; _DWrite_BitmapRenderTarget_Resize
; _DWrite_BitmapRenderTarget_SetCurrentTransform
; _DWrite_BitmapRenderTarget_SetPixelsPerDip





;#######################################################################################################################################################################
;# IDwriteBitmapRenderTarget1
;#######################################################################################################################################################################
#cs    IDwriteBitmapRenderTarget1
	Method ...: GetTextAntialiasMode (?)
	Info .....:     Gets the current text antialiasing mode of the bitmap render target.
	Param ....:         This method has no parameters.

	Method ...: SetTextAntialiasMode (?)
	Info .....:     Sets the current text antialiasing mode of the bitmap render target.
	Param ....:         AntialiasMode  <DWRITE_TEXT_ANTIALIAS_MODE>  A DWRITE_TEXT_ANTIALIAS_MODE-typed value that specifies the antialiasing mode.

#ce
; _DWrite_BitmapRenderTarget1_GetTextAntialiasMode
; _DWrite_BitmapRenderTarget1_SetTextAntialiasMode





;#######################################################################################################################################################################
;# IDwriteColorGlyphRunEnumerator
;#######################################################################################################################################################################
#cs    IDwriteColorGlyphRunEnumerator
	Method ...: GetCurrentRun (?)
	Info .....:     Returns the current glyph run of the enumerator.
	Param ....:         ColorGlyphRun  <DWRITE_COLOR_GLYPH_RUN>  A pointer to the current glyph run.

	Method ...: MoveNext (?)
	Info .....:     Move to the next glyph run in the enumerator.
	Param ....:         HaveRun  <BOOL>  Returns TRUE if there is a next glyph run.

#ce
; _DWrite_ColorGlyphRunEnumerator_GetCurrentRun
; _DWrite_ColorGlyphRunEnumerator_MoveNext





;#######################################################################################################################################################################
;# IDwriteFactory
;#######################################################################################################################################################################
#cs    IDwriteFactory
	Method ...: CreateCustomFontCollection (struct*;struct*;uint;ptr*)
	Info .....:     Creates a font collection using a custom font collection loader.
	Param ....:         CollectionLoader   <IDWriteFontCollectionLoader>  An application-defined font collection loader, which must have been previously registered using RegisterFontCollectionLoader.
	Param ....:         CollectionKey      <void>                         The key used by the loader to identify a collection of font files. The buffer allocated for this key should at least be the size of "collectionKeySize".
	Param ....:         CollectionKeySize  <UINT32>                       The size, in bytes, of the collection key.
	Param ....:         FontCollection     <IDWriteFontCollection>        Contains an address of a pointer to the system font collection object if the method succeeds, or NULL in case of failure.

	Method ...: CreateCustomFontFileReference (struct*;uint;struct*;ptr*)
	Info .....:     Creates a reference to an application-specific font file resource.
	Param ....:         FontFileReferenceKey      <void>                   A font file reference key that uniquely identifies the font file resource during the lifetime of "fontFileLoader".
	Param ....:         FontFileReferenceKeySize  <UINT32>                 The size of the font file reference key in bytes.
	Param ....:         FontFileLoader            <IDWriteFontFileLoader>  The font file loader that will be used by the font system to load data from the file identified by "fontFileReferenceKey".
	Param ....:         FontFile                  <IDWriteFontFile>        Contains an address of a pointer to the newly created font file object when this method succeeds, or NULL in case of failure.

	Method ...: CreateCustomRenderingParams (float;float;float;uint;uint;ptr*)
	Info .....:     Creates a rendering parameters object with the specified properties.
	Param ....:         Gamma             <FLOAT>                   The gamma level to be set for the new rendering parameters object.
	Param ....:         EnhancedContrast  <FLOAT>                   The enhanced contrast level to be set for the new rendering parameters object.
	Param ....:         ClearTypeLevel    <FLOAT>                   The ClearType level to be set for the new rendering parameters object.
	Param ....:         PixelGeometry     <DWRITE_PIXEL_GEOMETRY>   Represents the internal structure of a device pixel (that is, the physical arrangement of red, green, and blue color components) that is assumed for purposes of rendering text.
	Param ....:         RenderingMode     <DWRITE_RENDERING_MODE>   A value that represents the method (for example, ClearType natural quality) for rendering glyphs.
	Param ....:         RenderingParams   <IDWriteRenderingParams>  When this method returns, contains an address of a pointer to the newly created rendering parameters object.

	Method ...: CreateEllipsisTrimmingSign (struct*;ptr*)
	Info .....:     Creates an inline object for trimming, using an ellipsis as the omission sign.
	Param ....:         TextFormat    <IDWriteTextFormat>    A text format object, created with CreateTextFormat, used for text layout.
	Param ....:         TrimmingSign  <IDWriteInlineObject>  When this method returns, contains an address of a pointer to the omission (that is, ellipsis trimming) sign created by this method.

	Method ...: CreateFontFace (uint;uint;struct*;uint;uint;ptr*)
	Info .....:     Creates an object that represents a font face.
	Param ....:         FontFaceType             <DWRITE_FONT_FACE_TYPE>    A value that indicates the type of file format of the font face.
	Param ....:         NumberOfFiles            <UINT32>                   The number of font files, in element count, required to represent the font face.
	Param ....:         FontFiles                <IDWriteFontFile>          A font file object representing the font face. Because IDWriteFontFace maintains its own references to the input font file objects, you may release them after this call.
	Param ....:         FaceIndex                <UINT32>                   The zero-based index of a font face, in cases when the font files contain a collection of font faces. If the font files contain a single face, this value should be zero.
	Param ....:         FontFaceSimulationFlags  <DWRITE_FONT_SIMULATIONS>  A value that indicates which, if any, font face simulation flags for algorithmic means of making text bold or italic are applied to the current font face.
	Param ....:         FontFace                 <IDWriteFontFace>          When this method returns, contains an address of a pointer to the newly created font face object, or NULL in case of failure.

	Method ...: CreateFontFileReference (wstr;struct*;ptr*)
	Info .....:     Creates a font file reference object from a local font file.
	Param ....:         FilePath       <WCHAR>            An array of characters that contains the absolute file path for the font file. Subsequent operations on the constructed object may fail if the user provided "filePath" doesn't correspond to a valid file on the disk.
	Param ....:         LastWriteTime  <FILETIME>         The last modified time of the input file path. If the parameter is omitted, the function will access the font file to obtain its last write time. You should specify this value to avoid extra disk access. Subsequent operations on the constructed object may fail if the user provided "lastWriteTime" doesn't match the file on the disk.
	Param ....:         FontFile       <IDWriteFontFile>  When this method returns, contains an address of a pointer to the newly created font file reference object, or NULL in case of failure.

	Method ...: CreateGdiCompatibleTextLayout (wstr;uint;struct*;float;float;float;struct*;bool;ptr*)
	Info .....:     Takes a string, format, and associated constraints, and produces an object representing the result, formatted for a particular display resolution and measuring mode.
	Param ....:         String         <WCHAR>              An array of characters that contains the string to create a new IDWriteTextLayout object from. This array must be of length "stringLength" and can contain embedded NULL characters.
	Param ....:         StringLength   <UINT32>             The length of the string, in character count.
	Param ....:         TextFormat     <IDWriteTextFormat>  The text formatting object to apply to the string.
	Param ....:         LayoutWidth    <FLOAT>              The width of the layout box.
	Param ....:         LayoutHeight   <FLOAT>              The height of the layout box.
	Param ....:         PixelsPerDip   <FLOAT>              The number of physical pixels per DIP (device independent pixel). For example, if rendering onto a 96 DPI device "pixelsPerDip" is 1. If rendering onto a 120 DPI device "pixelsPerDip" is 1.25 (120/96).
	Param ....:         Transform      <DWRITE_MATRIX>      An optional transform applied to the glyphs and their positions. This transform is applied after the scaling specifies the font size and pixels per DIP.
	Param ....:         UseGdiNatural  <BOOL>                Instructs the text layout to use the same metrics as GDI bi-level text when set to FALSE. When set to TRUE, instructs the text layout to use the same metrics as text measured by GDI using a font created with CLEARTYPE_NATURAL_QUALITY.
	Param ....:         TextLayout     <IDWriteTextLayout>  When this method returns, contains an address to the pointer of the resultant text layout object.

	Method ...: CreateGlyphRunAnalysis (struct*;float;struct*;uint;uint;float;float;ptr*)
	Info .....:     Creates a glyph run analysis object, which encapsulates information used to render a glyph run.
	Param ....:         GlyphRun          <DWRITE_GLYPH_RUN>         A structure that contains the properties of the glyph run (font face, advances, and so on).
	Param ....:         PixelsPerDip      <FLOAT>                    Number of physical pixels per DIP (device independent pixel). For example, if rendering onto a 96 DPI bitmap then "pixelsPerDip" is 1. If rendering onto a 120 DPI bitmap then "pixelsPerDip" is 1.25.
	Param ....:         Transform         <DWRITE_MATRIX>            Optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified the "emSize" and "pixelsPerDip".
	Param ....:         RenderingMode     <DWRITE_RENDERING_MODE>    A value that specifies the rendering mode, which must be one of the raster rendering modes (that is, not default and not outline).
	Param ....:         MeasuringMode     <DWRITE_MEASURING_MODE>    Specifies the measuring mode to use with glyphs.
	Param ....:         BaselineOriginX   <FLOAT>                    The horizontal position (X-coordinate) of the baseline origin, in DIPs.
	Param ....:         BaselineOriginY   <FLOAT>                    Vertical position (Y-coordinate) of the baseline origin, in DIPs.
	Param ....:         GlyphRunAnalysis  <IDWriteGlyphRunAnalysis>  When this method returns, contains an address of a pointer to the newly created glyph run analysis object.

	Method ...: CreateMonitorRenderingParams (handle;ptr*)
	Info .....:     Creates a rendering parameters object with default settings for the specified monitor. In most cases, this is the preferred way to create a rendering parameters object.
	Param ....:         Monitor          <HMONITOR>                A handle for the specified monitor.
	Param ....:         RenderingParams  <IDWriteRenderingParams>  When this method returns, contains an address of a pointer to the rendering parameters object created by this method.

	Method ...: CreateNumberSubstitution (uint;wstr;bool;ptr*)
	Info .....:     Creates a number substitution object using a locale name, substitution method, and an indicator whether to ignore user overrides (use NLS defaults for the given culture instead).
	Param ....:         SubstitutionMethod  <DWRITE_NUMBER_SUBSTITUTION_METHOD>  A value that specifies how to apply number substitution on digits and related punctuation.
	Param ....:         LocaleName          <WCHAR>                              The name of the locale to be used in the "numberSubstitution" object.
	Param ....:         IgnoreUserOverride  <BOOL>                               A Boolean flag that indicates whether to ignore user overrides.
	Param ....:         NumberSubstitution  <IDWriteNumberSubstitution>          When this method returns, contains an address to a pointer to the number substitution object created by this method.

	Method ...: CreateRenderingParams (ptr*)
	Info .....:     Creates a rendering parameters object with default settings for the primary monitor. Different monitors may have different rendering parameters, for more information see the How to Add Support for Multiple Monitors topic.
	Param ....:         RenderingParams  <IDWriteRenderingParams>  When this method returns, contains an address of a pointer to the newly created rendering parameters object.

	Method ...: CreateTextAnalyzer (ptr*)
	Info .....:     Returns an interface for performing text analysis.
	Param ....:         TextAnalyzer  <IDWriteTextAnalyzer>  When this method returns, contains an address of a pointer to the newly created text analyzer object.

	Method ...: CreateTextFormat (wstr;struct*;uint;uint;uint;float;wstr;ptr*)
	Info .....:     Creates a text format object used for text layout.
	Param ....:         FontFamilyName  <WCHAR>                  An array of characters that contains the name of the font family
	Param ....:         FontCollection  <IDWriteFontCollection>  A pointer to a font collection object. When this is NULL, indicates the system font collection.
	Param ....:         FontWeight      <DWRITE_FONT_WEIGHT>     A value that indicates the font weight for the text object created by this method.
	Param ....:         FontStyle       <DWRITE_FONT_STYLE>      A value that indicates the font style for the text object created by this method.
	Param ....:         FontStretch     <DWRITE_FONT_STRETCH>    A value that indicates the font stretch for the text object created by this method.
	Param ....:         FontSize        <FLOAT>                  The logical size of the font in DIP ("device-independent pixel") units. A DIP equals 1/96 inch.
	Param ....:         LocaleName      <WCHAR>                  An array of characters that contains the locale name.
	Param ....:         TextFormat      <IDWriteTextFormat>      When this method returns, contains an address of a pointer to a newly created text format object, or NULL in case of failure.

	Method ...: CreateTextLayout (wstr;uint;struct*;float;float;ptr*)
	Info .....:     Takes a string, text format, and associated constraints, and produces an object that represents the fully analyzed and formatted result.
	Param ....:         String        <WCHAR>              An array of characters that contains the string to create a new IDWriteTextLayout object from. This array must be of length "stringLength" and can contain embedded NULL characters.
	Param ....:         StringLength  <UINT32>             The number of characters in the string.
	Param ....:         TextFormat    <IDWriteTextFormat>  A pointer to an object that indicates the format to apply to the string.
	Param ....:         MaxWidth      <FLOAT>              The width of the layout box.
	Param ....:         MaxHeight     <FLOAT>              The height of the layout box.
	Param ....:         TextLayout    <IDWriteTextLayout>  When this method returns, contains an address of a pointer to the resultant text layout object.

	Method ...: CreateTypography (ptr*)
	Info .....:     Creates a typography object for use in a text layout.
	Param ....:         Typography  <IDWriteTypography>  When this method returns, contains the address of a pointer to a newly created typography object, or NULL in case of failure.

	Method ...: GetGdiInterop (ptr*)
	Info .....:     Creates an object that is used for interoperability with GDI.
	Param ....:         GdiInterop  <IDWriteGdiInterop>  When this method returns, contains an address of a pointer to a GDI interop object if successful, or NULL in case of failure.

	Method ...: GetSystemFontCollection (ptr*;bool)
	Info .....:     Gets an object which represents the set of installed fonts.
	Param ....:         FontCollection   <IDWriteFontCollection>  When this method returns, contains the address of a pointer to the system font collection object, or NULL in case of failure.
	Param ....:         CheckForUpdates  <BOOL>                   If this parameter is nonzero, the function performs an immediate check for changes to the set of installed fonts. If this parameter is FALSE, the function will still detect changes if the font cache service is running, but there may be some latency. For example, an application might specify TRUE if it has itself just installed a font and wants to be sure the font collection contains that font.

	Method ...: RegisterFontCollectionLoader (struct*)
	Info .....:     Registers a custom font collection loader with the factory object.
	Param ....:         FontCollectionLoader  <IDWriteFontCollectionLoader>  Pointer to a IDWriteFontCollectionLoader object to be registered.

	Method ...: RegisterFontFileLoader (struct*)
	Info .....:     Registers a font file loader with DirectWrite.
	Param ....:         FontFileLoader  <IDWriteFontFileLoader>  Pointer to a IDWriteFontFileLoader object for a particular file resource type.

	Method ...: UnregisterFontCollectionLoader (struct*)
	Info .....:     Unregisters a custom font collection loader that was previously registered using RegisterFontCollectionLoader.
	Param ....:         FontCollectionLoader  <IDWriteFontCollectionLoader>  Pointer to a IDWriteFontCollectionLoader object to be unregistered.

	Method ...: UnregisterFontFileLoader (struct*)
	Info .....:     Unregisters a font file loader that was previously registered with the DirectWrite font system using RegisterFontFileLoader.
	Param ....:         FontFileLoader  <IDWriteFontFileLoader>  Pointer to the file loader that was previously registered with the DirectWrite font system using RegisterFontFileLoader.

#ce

; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_Create
; Description ...:  Creates an IDWriteFactory object.
; Syntax ........: _DWrite_Factory_Create($oIFactory)
; Parameters ....:
; Return values .: Success - An IDWriteFactory object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_Create()
	Local $tIID_IDWriteFactory = _WinAPI_GUIDFromString($sIID_IDWriteFactory)

	Local $aResult = DllCall($__g_hDWriteDLL, "int", "DWriteCreateFactory", "uint", 0, "struct*", $tIID_IDWriteFactory, "ptr*", 0)
	If @error Or $aResult[0] Then Return SetError(1, 1, False)

	Local $oObj = ObjCreateInterface($aResult[3], $sIID_IDWriteFactory, $tagIDWriteFactory)
	If Not IsObj($oObj) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)

	Return $oObj
EndFunc   ;==>_IDWriteFactory_Create

; _DWrite_Factory_CreateCustomFontCollection
; _DWrite_Factory_CreateCustomFontFileReference
; _DWrite_Factory_CreateCustomRenderingParams
; _DWrite_Factory_CreateEllipsisTrimmingSign


; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_CreateFontFace
; Description ...:  Creates an object that represents a font face.
; Syntax ........: _DWrite_Factory_CreateFontFace($oIFactory, $oFontFiles[, $iFontFaceType = $DWRITE_FONT_FACE_TYPE_TRUETYPE[, $iNumberOfFiles = 1[, $iFaceIndex = 0[, $iFontFaceSimulationFlags = 0]]]])
; Parameters ....: $oIFactory                - This IDWriteFactory object
;                  $oFontFiles               - An IDWriteFontFile object. A font file object representing the font face. Because IDWriteFontFace
;                                              maintains its own references to the input font file objects, you may release them after this call.
;                  $iFontFaceType            - [optional] A $DWRITE_FONT_FACE_TYPE enumeraion. A value that indicates the type of file format
;                                                         of the font face.
;                  $iNumberOfFiles           - [optional] The number of font files, in element count, required to represent the font face.
;                  $iFaceIndex               - [optional] The zero-based index of a font face, in cases when the font files contain a collection
;                                                         of font faces. If the font files contain a single face, this value should be zero.
;                  $iFontFaceSimulationFlags - [optional] A $DWRITE_FONT_SIMULATIONS enumeraion. A value that indicates which, if any, font face
;                                                         simulation flags for algorithmic means of making text bold or italic are applied to
;                                                         the current font face.
; Return values .: Success - An IDWriteFontFace object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_CreateFontFace($oIFactory, $pFontFiles, $iFontFaceType = $DWRITE_FONT_FACE_TYPE_TRUETYPE, $iNumberOfFiles = 1, $iFaceIndex = 0, $iFontFaceSimulationFlags = 0)
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)
	If IsObj($pFontFiles) Then
		$pFontFiles = $pFontFiles()
	ElseIf Not $pFontFiles Then
		Return SetError($DWRITEERR_NOOBJ, 1, False)
	EndIf

	Local $tFontFiles = DllStructCreate("ptr Ptr;")
	$tFontFiles.Ptr = $pFontFiles

	Local $pFontFace
	Local $iHResult = $oIFactory.CreateFontFace($iFontFaceType, $iNumberOfFiles, $tFontFiles, $iFaceIndex, $iFontFaceSimulationFlags, $pFontFace)
	If $iHResult Or Not $pFontFace Then Return SetError($iHResult, 2, False)

	Local $oFontFace = ObjCreateInterface($pFontFace, $sIID_IDWriteFontFace, $tagIDWriteFontFace)
	If Not IsObj($oFontFace) Then Return SetError($DWRITEERR_OBJFAIL, 3, False)
	Return $oFontFace
EndFunc


; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_CreateFontFileReference
; Description ...:  Creates a font file reference object from a local font file.
; Syntax ........: _DWrite_Factory_CreateFontFileReference($oIFactory, $sFilePath[, $tLastWriteTime = Null])
; Parameters ....: $oIFactory      - This IDWriteFactory object
;                  $sFilePath      - A string that contains the absolute file path for the font file. Subsequent operations on the constructed
;                                    object may fail if the user provided "filePath" doesn't correspond to a valid file on the disk.
;                  $tLastWriteTime - [optional] The last modified time of the input file path. If the parameter is omitted, the function will
;                                               access the font file to obtain its last write time. You should specify this value to avoid extra
;                                               disk access. Subsequent operations on the constructed object may fail if the user provided "lastWriteTime"
;                                               doesn't match the file on the disk.
; Return values .: Success - An IDWriteFontFile object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_CreateFontFileReference($oIFactory, $sFilePath, $tLastWriteTime = Null)
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pFontFile
	Local $iHResult = $oIFactory.CreateFontFileReference($sFilePath, $tLastWriteTime, $pFontFile)
	If $iHResult Or Not $pFontFile Then Return SetError($iHResult, 1, False)

	Local $oFontFile = ObjCreateInterface($pFontFile, $sIID_IDWriteFontFile, $tagIDWriteFontFile)
	If Not IsObj($oFontFile) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oFontFile
EndFunc

; _DWrite_Factory_CreateGdiCompatibleTextLayout
; _DWrite_Factory_CreateGlyphRunAnalysis
; _DWrite_Factory_CreateMonitorRenderingParams
; _DWrite_Factory_CreateNumberSubstitution
; _DWrite_Factory_CreateRenderingParams
; _DWrite_Factory_CreateTextAnalyzer

; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_CreateTextFormat
; Description ...:  Creates a text format object used for text layout.
; Syntax ........: _DWrite_Factory_CreateTextFormat($oIFactory[, $sFontFamilyName = "Arial"[, $fFontSize = 48[, $iFontWeight = 400[, $iFontStyle = 0[, $iFontStretch = 5[, $oFontCollection = Null[, $sLocaleName = "en-us"]]]]]]])
; Parameters ....: $oIFactory       - This IDWriteFactory object
;                  $sFontFamilyName - [optional] A string that contains the name of the font family
;                  $fFontSize       - [optional] The logical size of the font in DIP ("device-independent pixel") units. A DIP equals 1/96 inch.
;                  $iFontWeight     - [optional] A $DWRITE_FONT_WEIGHT enumeraion. A value that indicates the font weight for the text object
;                                                created by this method.
;                  $iFontStyle      - [optional] A $DWRITE_FONT_STYLE enumeraion. A value that indicates the font style for the text object created
;                                                by this method.
;                  $iFontStretch    - [optional] A $DWRITE_FONT_STRETCH enumeraion. A value that indicates the font stretch for the text object
;                                                created by this method.
;                  $oFontCollection - [optional] An IDWriteFontCollection object. A pointer to a font collection object. When this is NULL, indicates
;                                                the system font collection.
;                  $sLocaleName     - [optional] A string that contains the locale name.
; Return values .: Success - An IDWriteTextFormat object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_CreateTextFormat($oIFactory, $sFontFamilyName = "Arial", $fFontSize = 48, $iFontWeight = 400, $iFontStyle = 0, $iFontStretch = 5, $oFontCollection = Null, $sLocaleName = "en-us")
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)
	If $oFontCollection <> Null And Not IsObj($oFontCollection) Then Return SetError($DWRITEERR_NOOBJ, 1, False)

	Local $pTextFormat
	Local $iHResult = $oIFactory.CreateTextFormat($sFontFamilyName, $oFontCollection, $iFontWeight, $iFontStyle, $iFontStretch, $fFontSize, $sLocaleName, $pTextFormat)
	If $iHResult Or Not $pTextFormat Then Return SetError($iHResult, 2, False)

	Local $oTextFormat = ObjCreateInterface($pTextFormat, $sIID_IDWriteTextFormat, $tagIDWriteTextFormat)
	If Not IsObj($oTextFormat) Then Return SetError($DWRITEERR_OBJFAIL, 3, False)
	Return $oTextFormat
EndFunc


; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_CreateTextLayout
; Description ...:  Takes a string, text format, and associated constraints, and produces an object that represents the fully analyzed and formatted result.
; Syntax ........: _DWrite_Factory_CreateTextLayout($oIFactory, $sString, $oTextFormat, $fMaxWidth, $fMaxHeight)
; Parameters ....: $oIFactory   - This IDWriteFactory object
;                  $sString     - A string that contains the string to create a new IDWriteTextLayout object from.
;                  $oTextFormat - An IDWriteTextFormat object. A pointer to an object that indicates the format to apply to the string.
;                  $fMaxWidth   - The width of the layout box.
;                  $fMaxHeight  - The height of the layout box.
; Return values .: Success - An IDWriteTextLayout object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_CreateTextLayout($oIFactory, $sString, $oTextFormat, $fMaxWidth, $fMaxHeight)
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)
	If Not IsObj($oTextFormat) Then Return SetError($DWRITEERR_NOOBJ, 1, False)

	Local $pTextLayout
	Local $iHResult = $oIFactory.CreateTextLayout($sString, StringLen($sString), $oTextFormat, $fMaxWidth, $fMaxHeight, $pTextLayout)
	If $iHResult Or Not $pTextLayout Then Return SetError($iHResult, 2, False)

	Local $oTextLayout = ObjCreateInterface($pTextLayout, $sIID_IDWriteTextLayout, $tagIDWriteTextLayout)
	If Not IsObj($oTextLayout) Then Return SetError($DWRITEERR_OBJFAIL, 3, False)
	Return $oTextLayout
EndFunc


; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_CreateTypography
; Description ...:  Creates a typography object for use in a text layout.
; Syntax ........: _DWrite_Factory_CreateTypography($oIFactory)
; Parameters ....: $oIFactory - This IDWriteFactory object
; Return values .: Success - An IDWriteTypography object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_CreateTypography($oIFactory)
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pTypography
	Local $iHResult = $oIFactory.CreateTypography($pTypography)
	If $iHResult Or Not $pTypography Then Return SetError($iHResult, 1, False)

	Local $oTypography = ObjCreateInterface($pTypography, $sIID_IDWriteTypography, $tagIDWriteTypography)
	If Not IsObj($oTypography) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oTypography
EndFunc

; _DWrite_Factory_GetGdiInterop



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Factory_GetSystemFontCollection
; Description ...:  Gets an object which represents the set of installed fonts.
; Syntax ........: _DWrite_Factory_GetSystemFontCollection($oIFactory[, $bCheckForUpdates = False])
; Parameters ....: $oIFactory        - This IDWriteFactory object
;                  $bCheckForUpdates - [optional] If this parameter is nonzero, the function performs an immediate check for changes to the set
;                                                 of installed fonts. If this parameter is FALSE, the function will still detect changes if the
;                                                 font cache service is running, but there may be some latency. For example, an application might
;                                                 specify TRUE if it has itself just installed a font and wants to be sure the font collection
;                                                 contains that font.
; Return values .: Success - An IDWriteFontCollection object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Factory_GetSystemFontCollection($oIFactory, $bCheckForUpdates = False)
	If Not IsObj($oIFactory) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pFontCollection
	Local $iHResult = $oIFactory.GetSystemFontCollection($pFontCollection, $bCheckForUpdates)
	If $iHResult Or Not $pFontCollection Then Return SetError($iHResult, 1, False)

	Local $oFontCollection = ObjCreateInterface($pFontCollection, $sIID_IDWriteFontCollection, $tagIDWriteFontCollection)
	If Not IsObj($oFontCollection) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oFontCollection
EndFunc


; _DWrite_Factory_RegisterFontCollectionLoader
; _DWrite_Factory_RegisterFontFileLoader
; _DWrite_Factory_UnregisterFontCollectionLoader
; _DWrite_Factory_UnregisterFontFileLoader





;#######################################################################################################################################################################
;# IDwriteFactory1
;#######################################################################################################################################################################
#cs    IDwriteFactory1
	Method ...: CreateCustomRenderingParams (?)
	Info .....:     Creates a rendering parameters object with the specified properties.
	Param ....:         Gamma                      <FLOAT>                    The gamma level to be set for the new rendering parameters object.
	Param ....:         EnhancedContrast           <FLOAT>                    The enhanced contrast level to be set for the new rendering parameters object.
	Param ....:         EnhancedContrastGrayscale  <FLOAT>                    The amount of contrast enhancement to use for grayscale antialiasing, zero or greater.
	Param ....:         ClearTypeLevel             <FLOAT>                    The ClearType level to be set for the new rendering parameters object.
	Param ....:         PixelGeometry              <DWRITE_PIXEL_GEOMETRY>    Represents the internal structure of a device pixel (that is, the physical arrangement of red, green, and blue color components) that is assumed for purposes of rendering text.
	Param ....:         RenderingMode              <DWRITE_RENDERING_MODE>    A value that represents the method (for example, ClearType natural quality) for rendering glyphs.
	Param ....:         RenderingParams            <IDWriteRenderingParams1>  When this method returns, contains an address of a pointer to the newly created rendering parameters object.

	Method ...: GetEudcFontCollection (?)
	Info .....:     Gets a font collection representing the set of EUDC (end-user defined characters) fonts.
	Param ....:         FontCollection   <IDWriteFontCollection>  The font collection to fill.
	Param ....:         CheckForUpdates  <checkForUpdates>        Whether to check for updates.

#ce
; _DWrite_Factory1_CreateCustomRenderingParams
; _DWrite_Factory1_GetEudcFontCollection





;#######################################################################################################################################################################
;# IDwriteFactory2
;#######################################################################################################################################################################
#cs    IDwriteFactory2
	Method ...: CreateFontFallbackBuilder (?)
	Info .....:     Creates a font fallback builder object.
	Param ....:         FontFallbackBuilder  <IDWriteFontFallbackBuilder>  Contains an address of a pointer to the newly created font fallback builder object.

	Method ...: GetSystemFontFallback (?)
	Info .....:     Creates a font fallback object from the system font fallback list.
	Param ....:         FontFallback  <IDWriteFontFallback>  Contains an address of a pointer to the newly created font fallback object.

	Method ...: TranslateColorGlyphRun (?)
	Info .....:     This method is called on a glyph run to translate it in to multiple color glyph runs.
	Param ....:         BaselineOriginX         <FLOAT>                           The horizontal baseline origin of the original glyph run.
	Param ....:         BaselineOriginY         <FLOAT>                           The vertical baseline origin of the original glyph run.
	Param ....:         GlyphRun                <DWRITE_GLYPH_RUN>                Original glyph run containing monochrome glyph IDs.
	Param ....:         GlyphRunDescription     <DWRITE_GLYPH_RUN_DESCRIPTION>    Optional glyph run description.
	Param ....:         MeasuringMode           <DWRITE_MEASURING_MODE>           Measuring mode used to compute glyph positions if the run contains color glyphs.
	Param ....:         WorldToDeviceTransform  <DWRITE_MATRIX>                   World transform multiplied by any DPI scaling. This is needed to compute glyph positions if the run contains color glyphs and the measuring mode is not DWRITE_MEASURING_MODE_NATURAL. If this parameter is NULL, and identity transform is assumed.
	Param ....:         ColorPaletteIndex       <UINT32>                          Zero-based index of the color palette to use. Valid indices are less than the number of palettes in the font, as returned by IDWriteFontFace2::GetColorPaletteCount.
	Param ....:         ColorLayers             <IDWriteColorGlyphRunEnumerator>  If the original glyph run contains color glyphs, this parameter receives a pointer to an IDWriteColorGlyphRunEnumerator interface. The client uses the returned interface to get information about glyph runs and associated colors to render instead of the original glyph run. If the original glyph run does not contain color glyphs, this method returns DWRITE_E_NOCOLOR and the output pointer is NULL.

#ce
; _DWrite_Factory2_CreateFontFallbackBuilder
; _DWrite_Factory2_GetSystemFontFallback
; _DWrite_Factory2_TranslateColorGlyphRun





;#######################################################################################################################################################################
;# IDwriteFont
;#######################################################################################################################################################################
#cs    IDwriteFont
	Method ...: CreateFontFace (ptr*)
	Info .....:     Creates a font face object for the font.
	Param ....:         FontFace  <IDWriteFontFace>  When this method returns, contains an address of a pointer to the newly created font face object.

	Method ...: GetFaceNames (ptr*)
	Info .....:     Gets a localized strings collection containing the face names for the font (such as Regular or Bold), indexed by locale name.
	Param ....:         Names  <IDWriteLocalizedStrings>  When this method returns, contains an address to a pointer to the newly created localized strings object.

	Method ...: GetFontFamily (ptr*)
	Info .....:     Gets the font family to which the specified font belongs.
	Param ....:         FontFamily  <IDWriteFontFamily>  When this method returns, contains an address of a pointer to the font family object to which the specified font belongs.

	Method ...: GetInformationalStrings (uint;struct*;bool*)
	Info .....:     Gets a localized strings collection containing the specified informational strings, indexed by locale name.
	Param ....:         InformationalStringID  <DWRITE_INFORMATIONAL_STRING_ID>  A value that identifies the informational string to get. For example, DWRITE_INFORMATIONAL_STRING_DESCRIPTION specifies a string that contains a description of the font.
	Param ....:         InformationalStrings   <IDWriteLocalizedStrings>         When this method returns, contains an address of a pointer to the newly created localized strings object.
	Param ....:         Exists                 <BOOL>                            When this method returns, TRUE if the font contains the specified string ID; otherwise, FALSE.

	Method ...: GetMetrics (struct*)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a font face and are used by applications for layout calculations.
	Param ....:         FontMetrics  <DWRITE_FONT_METRICS>  When this method returns, contains a structure that has font metrics for the current font face. The metrics returned by this function are in font design units.

	Method ...: GetSimulations ()
	Info .....:     Gets a value that indicates what simulations are applied to the specified font.
	Param ....:         This method has no parameters.

	Method ...: GetStretch ()
	Info .....:     Gets the stretch, or width, of the specified font.
	Param ....:         This method has no parameters.

	Method ...: GetStyle ()
	Info .....:     Gets the style, or slope, of the specified font.
	Param ....:         This method has no parameters.

	Method ...: GetWeight ()
	Info .....:     Gets the weight, or stroke thickness, of the specified font.
	Param ....:         This method has no parameters.

	Method ...: HasCharacter (uint;bool*)
	Info .....:     Determines whether the font supports a specified character.
	Param ....:         UnicodeValue  <UINT32>  A Unicode (UCS-4) character value for the method to inspect.
	Param ....:         Exists        <BOOL>    When this method returns, TRUE if the font supports the specified character; otherwise, FALSE.

	Method ...: IsSymbolFont ()
	Info .....:     Determines whether the font is a symbol font.
	Param ....:         This method has no parameters.

#ce

; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Font_CreateFontFace
; Description ...:  Creates a font face object for the font.
; Syntax ........: _DWrite_Font_CreateFontFace($oIFont)
; Parameters ....: $oIFont - This IDWriteFont object
; Return values .: Success - An IDWriteFontFace object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Font_CreateFontFace($oIFont)
	If Not IsObj($oIFont) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pFontFace
	Local $iHResult = $oIFont.CreateFontFace($pFontFace)
	If $iHResult Or Not $pFontFace Then Return SetError($iHResult, 1, False)

	Local $oFontFace = ObjCreateInterface($pFontFace, $sIID_IDWriteFontFace, $tagIDWriteFontFace)
	If Not IsObj($oFontFace) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oFontFace
EndFunc




; _DWrite_Font_GetFaceNames
; _DWrite_Font_GetFontFamily
; _DWrite_Font_GetInformationalStrings
; _DWrite_Font_GetMetrics
; _DWrite_Font_GetSimulations
; _DWrite_Font_GetStretch
; _DWrite_Font_GetStyle
; _DWrite_Font_GetWeight
; _DWrite_Font_HasCharacter
; _DWrite_Font_IsSymbolFont





;#######################################################################################################################################################################
;# IDwriteFont1
;#######################################################################################################################################################################
#cs    IDwriteFont1
	Method ...: GetMetrics (?)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a font face and are used by applications for layout calculations.
	Param ....:         FontMetrics  <DWRITE_FONT_METRICS1>   A filled DWRITE_FONT_METRICS1 structure that has font metrics for the current font face. The metrics returned by this method are in font design units.

	Method ...: GetPanose (?)
	Info .....:     Gets the PANOSE values from the font and is used for font selection and matching.
	Param ....:         Panose  <DWRITE_PANOSE>  A pointer to the DWRITE_PANOSE structure to fill in.

	Method ...: GetUnicodeRanges (?)
	Info .....:     Retrieves the list of character ranges supported by a font.
	Param ....:         MaxRangeCount     <UINT32>                The maximum number of character ranges passed in from the client.
	Param ....:         UnicodeRanges     <DWRITE_UNICODE_RANGE>  An array of DWRITE_UNICODE_RANGE structures that are filled with the character ranges.
	Param ....:         ActualRangeCount  <UINT32>                A pointer to the actual number of character ranges, regardless of the maximum count.

	Method ...: IsMonospacedFont (?)
	Info .....:     Determines if the font is monospaced, that is, the characters are the same fixed-pitch width (non-proportional).
	Param ....:         This method has no parameters.

#ce
; _DWrite_Font1_GetMetrics
; _DWrite_Font1_GetPanose
; _DWrite_Font1_GetUnicodeRanges
; _DWrite_Font1_IsMonospacedFont





;#######################################################################################################################################################################
;# IDwriteFont2
;#######################################################################################################################################################################
#cs    IDwriteFont2
	Method ...: IsColorFont (?)
	Info .....:     Enables determining if a color rendering path is potentially necessary.
	Param ....:         This method has no parameters.

#ce
; _DWrite_Font2_IsColorFont





;#######################################################################################################################################################################
;# IDwriteFontCollection
;#######################################################################################################################################################################
#cs    IDwriteFontCollection
	Method ...: FindFamilyName (wstr;uint;bool*)
	Info .....:     Finds the font family with the specified family name.
	Param ....:         FamilyName  <WCHAR>   An array of characters, which is null-terminated, containing the name of the font family. The name is not case-sensitive but must otherwise exactly match a family name in the collection.
	Param ....:         Index       <UINT32>  When this method returns, contains the zero-based index of the matching font family if the family name was found; otherwise, UINT_MAX.
	Param ....:         Exists      <BOOL>    When this method returns, TRUE if the family name exists; otherwise, FALSE.

	Method ...: GetFontFamily ()
	Info .....:     Creates a font family object given a zero-based font family index.
	Param ....:         Index       <UINT32>             Zero-based index of the font family.
	Param ....:         FontFamily  <IDWriteFontFamily>  When this method returns, contains the address of a pointer to the newly created font family object.

	Method ...: GetFontFamilyCount ()
	Info .....:     Gets the number of font families in the collection.
	Param ....:         This method has no parameters.

	Method ...: GetFontFromFontFace (struct*;ptr*)
	Info .....:     Gets the font object that corresponds to the same physical font as the specified font face object. The specified physical font must belong to the font collection.
	Param ....:         FontFace  <IDWriteFontFace>  A font face object that specifies the physical font.
	Param ....:         Font      <IDWriteFont>      When this method returns, contains the address of a pointer to the newly created font object if successful; otherwise, NULL.

#ce
; _DWrite_FontCollection_FindFamilyName



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_FontCollection_GetFontFamily
; Description ...:  Creates a font family object given a zero-based font family index.
; Syntax ........: _DWrite_FontCollection_GetFontFamily($oIFontCollection, $iIndex)
; Parameters ....: $oIFontCollection - This IDWriteFontCollection object
;                  $iIndex           - Zero-based index of the font family.
; Return values .: Success - An IDWriteFontFamily object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_FontCollection_GetFontFamily($oIFontCollection, $iIndex)
	If Not IsObj($oIFontCollection) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pFontFamily
	Local $iHResult = $oIFontCollection.GetFontFamily($iIndex, $pFontFamily)
	If $iHResult Or Not $pFontFamily Then Return SetError($iHResult, 1, False)

	Local $oFontFamily = ObjCreateInterface($pFontFamily, $sIID_IDWriteFontFamily, $tagIDWriteFontFamily)
	If Not IsObj($oFontFamily) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oFontFamily
EndFunc



; _DWrite_FontCollection_GetFontFamilyCount
; _DWrite_FontCollection_GetFontFromFontFace





;#######################################################################################################################################################################
;# IDwriteFontCollectionLoader
;#######################################################################################################################################################################
#cs    IDwriteFontCollectionLoader
	Method ...: CreateEnumeratorFromKey (struct*;struct*;uint;ptr*)
	Info .....:     Creates a font file enumerator object that encapsulates a collection of font files. The font system calls back to this interface to create a font collection.
	Param ....:         Factory             <IDWriteFactory>             Pointer to the IDWriteFactory object that was used to create the current font collection.
	Param ....:         CollectionKey       <void>                       A font collection key that uniquely identifies the collection of font files within the scope of the font collection loader being used. The buffer allocated for this key must be at least the size, in bytes, specified by "collectionKeySize".
	Param ....:         CollectionKeySize   <UINT32>                     The size of the font collection key, in bytes.
	Param ....:         FontFileEnumerator  <IDWriteFontFileEnumerator>  When this method returns, contains the address of a pointer to the newly created font file enumerator.

#ce
; _DWrite_FontCollectionLoader_CreateEnumeratorFromKey





;#######################################################################################################################################################################
;# IDwriteFontFace
;#######################################################################################################################################################################
#cs    IDwriteFontFace
	Method ...: GetDesignGlyphMetrics (struct*;uint;ptr*;bool)
	Info .....:     Obtains ideal (resolution-independent) glyph metrics in font design units.
	Param ....:         GlyphIndices  <UINT16>                 An array of glyph indices for which to compute metrics. The array must contain at least as many elements as specified by "glyphCount".
	Param ....:         GlyphCount    <UINT32>                The number of elements in the "glyphIndices" array.
	Param ....:         GlyphMetrics  <DWRITE_GLYPH_METRICS>  When this method returns, contains an array of DWRITE_GLYPH_METRICS structures. "glyphMetrics" must be initialized with an empty buffer that contains at least as many elements as "glyphCount". The metrics returned by this function are in font design units.
	Param ....:         IsSideways    <BOOL>                  Indicates whether the font is being used in a sideways run. This can affect the glyph metrics if the font has oblique simulation because sideways oblique simulation differs from non-sideways oblique simulation

	Method ...: GetFiles (uint*;ptr*)
	Info .....:     Obtains the font files representing a font face.
	Param ....:         NumberOfFiles  <UINT32>           If "fontFiles" is NULL, receives the number of files representing the font face. Otherwise, the number of font files being requested should be passed. See the Remarks section below for more information.
	Param ....:         FontFiles      <IDWriteFontFile>  When this method returns, contains a pointer to a user-provided array that stores pointers to font files representing the font face. This parameter can be NULL if the user wants only the number of files representing the font face. This API increments reference count of the font file pointers returned according to COM conventions, and the client should release them when finished.

	Method ...: GetGdiCompatibleGlyphMetrics (float;float;struct*;bool;struct*;uint;struct*;bool)
	Info .....:     Obtains glyph metrics in font design units with the return values compatible with what GDI would produce.
	Param ....:         EmSize         <FLOAT>                 The ogical size of the font in DIP units.
	Param ....:         PixelsPerDip   <FLOAT>                 The number of physical pixels per DIP.
	Param ....:         Transform      <DWRITE_MATRIX>         An optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified by the font size and "pixelsPerDip".
	Param ....:         UseGdiNatural  <BOOL>                  When set to FALSE, the metrics are the same as the metrics of GDI aliased text. When set to TRUE, the metrics are the same as the metrics of text measured by GDI using a font created with CLEARTYPE_NATURAL_QUALITY.
	Param ....:         GlyphIndices   <UINT16>                An array of glyph indices for which to compute the metrics.
	Param ....:         GlyphCount     <UINT32>                The number of elements in the "glyphIndices" array.
	Param ....:         GlyphMetrics   <DWRITE_GLYPH_METRICS>  An array of DWRITE_GLYPH_METRICS structures filled by this function. The metrics are in font design units.
	Param ....:         IsSideways     <BOOL>                  A BOOL value that indicates whether the font is being used in a sideways run. This can affect the glyph metrics if the font has oblique simulation because sideways oblique simulation differs from non-sideways oblique simulation.

	Method ...: GetGdiCompatibleMetrics (float;float;struct*;struct*)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a fontface and are used by applications for layout calculations.
	Param ....:         EmSize           <FLOAT>                The logical size of the font in DIP units.
	Param ....:         PixelsPerDip     <FLOAT>                The number of physical pixels per DIP.
	Param ....:         Transform        <DWRITE_MATRIX>        An optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified by the font size and "pixelsPerDip".
	Param ....:         FontFaceMetrics  <DWRITE_FONT_METRICS>  A pointer to a DWRITE_FONT_METRICS structure to fill in. The metrics returned by this function are in font design units.

	Method ...: GetGlyphCount ()
	Info .....:     Obtains the number of glyphs in the font face.
	Param ....:         This method has no parameters.

	Method ...: GetGlyphIndices (struct*;uint;struct*)
	Info .....:     Returns the nominal mapping of UCS4 Unicode code points to glyph indices as defined by the font 'CMAP' table.
	Param ....:         CodePoints      <UINT32>  An array of USC4 code points from which to obtain nominal glyph indices. The array must be allocated and be able to contain the number of elements specified by "codePointCount".
	Param ....:         CodePointCount  <UINT32>  The number of elements in the "codePoints" array.
	Param ....:         GlyphIndices    <UINT16>  When this method returns, contains a pointer to an array of nominal glyph indices filled by this function.

	Method ...: GetGlyphRunOutline (float;struct*;struct*;struct*;uint;bool;bool;struct*)
	Info .....:     Computes the outline of a run of glyphs by calling back to the outline sink interface.
	Param ....:         EmSize         <FLOAT>                The logical size of the font in DIP units. A DIP ("device-independent pixel") equals 1/96 inch.
	Param ....:         GlyphIndices   <UINT16>               An array of glyph indices. The glyphs are in logical order and the advance direction depends on the "isRightToLeft" parameter. The array must be allocated and be able to contain the number of elements specified by "glyphCount".
	Param ....:         GlyphAdvances  <FLOAT>                An optional array of glyph advances in DIPs. The advance of a glyph is the amount to advance the position (in the direction of the baseline) after drawing the glyph. "glyphAdvances" contains the number of elements specified by "glyphCount".
	Param ....:         GlyphOffsets   <DWRITE_GLYPH_OFFSET>  An optional array of glyph offsets, each of which specifies the offset along the baseline and offset perpendicular to the baseline of a glyph relative to the current pen position. "glyphOffsets" contains the number of elements specified by "glyphCount".
	Param ....:         GlyphCount     <UINT32>               The number of glyphs in the run.
	Param ....:         IsSideways     <BOOL>                 A client can render a vertical run by setting "isSideways" to TRUE and rotating the resulting geometry 90 degrees to the right using a transform. The "isSideways" and "isRightToLeft" parameters cannot both be true.
	Param ....:         IsRightToLeft  <BOOL>                 The visual order of the glyphs. If this parameter is FALSE, then glyph advances are from left to right. If TRUE, the advance direction is right to left. By default, the advance direction is left to right.
	Param ....:         GeometrySink   <IDWriteGeometrySink>  A pointer to the interface that is called back to perform outline drawing operations.

	Method ...: GetIndex ()
	Info .....:     Obtains the index of a font face in the context of its font files.
	Param ....:         This method has no parameters.

	Method ...: GetMetrics (ptr*)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a font face and are used by applications for layout calculations.
	Param ....:         FontFaceMetrics  <DWRITE_FONT_METRICS>  When this method returns, a DWRITE_FONT_METRICS structure that holds metrics (such as ascent, descent, or cap height) for the current font face element. The metrics returned by this function are in font design units.

	Method ...: GetRecommendedRenderingMode (float;float;uint;struct*;uint*)
	Info .....:     Determines the recommended rendering mode for the font, using the specified size and rendering parameters.
	Param ....:         EmSize           <FLOAT>                   The logical size of the font in DIP units. A DIP ("device-independent pixel") equals 1/96 inch.
	Param ....:         PixelsPerDip     <FLOAT>                   The number of physical pixels per DIP. For example, if the DPI of the rendering surface is 96, this value is 1.0f. If the DPI is 120, this value is 120.0f/96.
	Param ....:         MeasuringMode    <DWRITE_MEASURING_MODE>   The measuring method that will be used for glyphs in the font. Renderer implementations may choose different rendering modes for different measuring methods, for example:
	Param ....:         RenderingParams  <IDWriteRenderingParams>  A pointer to an object that contains rendering settings such as gamma level, enhanced contrast, and ClearType level. This parameter is necessary in case the rendering parameters object overrides the rendering mode.
	Param ....:         RenderingMode    <DWRITE_RENDERING_MODE>   When this method returns, contains a value that indicates the recommended rendering mode to use.

	Method ...: GetSimulations ()
	Info .....:     Obtains the algorithmic style simulation flags of a font face.
	Param ....:         This method has no parameters.

	Method ...: GetType ()
	Info .....:     Obtains the file format type of a font face.
	Param ....:         This method has no parameters.

	Method ...: IsSymbolFont ()
	Info .....:     Determines whether the font is a symbol font.
	Param ....:         This method has no parameters.

	Method ...: ReleaseFontTable (struct*)
	Info .....:     Releases the table obtained earlier from TryGetFontTable.
	Param ....:         TableContext  <void>  A pointer to the opaque context from TryGetFontTable.

	Method ...: TryGetFontTable (uint;struct*;uint*;ptr*;bool*)
	Info .....:     Finds the specified OpenType font table if it exists and returns a pointer to it. The function accesses the underlying font data through the IDWriteFontFileStream interface implemented by the font file loader.
	Param ....:         OpenTypeTableTag  <UINT32>  The four-character tag of a OpenType font table to find. Use the DWRITE_MAKE_OPENTYPE_TAG macro to create it as an UINT32. Unlike GDI, it does not support the special TTCF and null tags to access the whole font.
	Param ....:         TableData         <void>    When this method returns, contains the address of a pointer to the base of the table in memory. The pointer is valid only as long as the font face used to get the font table still exists; (not any other font face, even if it actually refers to the same physical font). This parameter is passed uninitialized.
	Param ....:         TableSize         <UINT32>  When this method returns, contains a pointer to the size, in bytes, of the font table.
	Param ....:         TableContext      <void>    When this method returns, the address of a pointer to the opaque context, which must be freed by calling ReleaseFontTable. The context actually comes from the lower-level IDWriteFontFileStream, which may be implemented by the application or DWrite itself. It is possible for a NULL"tableContext" to be returned, especially if the implementation performs direct memory mapping on the whole file. Nevertheless, always release it later, and do not use it as a test for function success. The same table can be queried multiple times, but because each returned context can be different, you must release each context separately.
	Param ....:         Exists            <BOOL>    When this method returns, TRUE if the font table exists; otherwise, FALSE.

#ce
; _DWrite_FontFace_GetDesignGlyphMetrics
; _DWrite_FontFace_GetFiles
; _DWrite_FontFace_GetGdiCompatibleGlyphMetrics
; _DWrite_FontFace_GetGdiCompatibleMetrics
; _DWrite_FontFace_GetGlyphCount



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_FontFace_GetGlyphIndices
; Description ...:  Returns the nominal mapping of UCS4 Unicode code points to glyph indices as defined by the font 'CMAP' table.
; Syntax ........: _DWrite_FontFace_GetGlyphIndices($oIFontFace, $sString)
; Parameters ....: $oIFontFace - This IDWriteFontFace object
;                  $sString    - A string from which to obtain nominal glyph indices.
; Return values .: Success - An short struct array of nominal glyph indices filled by this function.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......: Note that this mapping is primarily provided for line layout engines built on top of the physical font API. Because of OpenType
;                  glyph substitution and line layout character substitution, the nominal conversion does not always correspond to how a Unicode
;                  string will map to glyph indices when rendering using a particular font face. Also, note that Unicode variant selectors provide
;                  for alternate mappings for character to glyph. This call will always return the default variant.
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_FontFace_GetGlyphIndices($oIFontFace, $sString)
	If Not IsObj($oIFontFace) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $iStrLen = StringLen($sString)
	Local $tCodePoints = DllStructCreate("byte UTF32[" & $iStrLen * 4 & "];")
	$tCodePoints.UTF32 = StringToBinary(BinaryToString(StringToBinary($sString, 2), 1), 2)

	Local $tGlyphIndices = DllStructCreate("ushort Index[" & $iStrLen & "];")
	Local $iHResult = $oIFontFace.GetGlyphIndices($tCodePoints, $iStrLen, $tGlyphIndices)
	If $iHResult Then Return SetError($iHResult, 1, False)

	Return $tGlyphIndices
EndFunc



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_FontFace_GetGlyphRunOutline
; Description ...:  Computes the outline of a run of glyphs by calling back to the outline sink interface.
; Syntax ........: _DWrite_FontFace_GetGlyphRunOutline($oIFontFace, $oGeometrySink, $fSize, $tGlyphIndices, $iGlyphCount[, $tGlyphAdvances = Null[, $tGlyphOffsets = Null[, $bIsSideways = False[, $bIsRightToLeft = False]]]])
; Parameters ....: $oIFontFace     - This IDWriteFontFace object
;                  $oGeometrySink  - An IDWriteGeometrySink object. An IDWriteGeometrySink interface that is called back to perform outline drawing operations.
;                  $fSize          - The logical size of the font in DIP units. A DIP ("device-independent pixel") equals 1/96 inch.
;                  $tGlyphIndices  - A $tagUINT16 structure. An array of glyph indices. The glyphs are in logical order and the advance direction
;                                    depends on the "isRightToLeft" parameter. The array must be allocated and be able to contain the number
;                                    of elements specified by "glyphCount".
;                  $iGlyphCount    - The number of glyphs in the run.
;                  $tGlyphAdvances - [optional] A $tagFLOAT structure. An optional array of glyph advances in DIPs. The advance of a glyph is
;                                               the amount to advance the position (in the direction of the baseline) after drawing the glyph.
;                                               "glyphAdvances" contains the number of elements specified by "glyphCount".
;                  $tGlyphOffsets  - [optional] A $tagDWRITE_GLYPH_OFFSET structure. An optional array of glyph offsets, each of which specifies
;                                               the offset along the baseline and offset perpendicular to the baseline of a glyph relative to
;                                               the current pen position. "glyphOffsets" contains the number of elements specified by "glyphCount".
;                  $bIsSideways    - [optional] A client can render a vertical run by setting "isSideways" to TRUE and rotating the resulting
;                                               geometry 90 degrees to the right using a transform. The "isSideways" and "isRightToLeft" parameters
;                                               cannot both be true.
;                  $bIsRightToLeft - [optional] The visual order of the glyphs. If this parameter is FALSE, then glyph advances are from left
;                                               to right. If TRUE, the advance direction is right to left. By default, the advance direction
;                                               is left to right.
; Return values .: Success - True
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_FontFace_GetGlyphRunOutline($oIFontFace, $oGeometrySink, $fSize, $tGlyphIndices, $iGlyphCount, $tGlyphAdvances = Null, $tGlyphOffsets = Null, $bIsSideways = False, $bIsRightToLeft = False)
	If Not IsObj($oIFontFace) Then Return SetError($DWRITEERR_NOOBJ, 0, False)
	If Not IsObj($oGeometrySink) Then Return SetError($DWRITEERR_NOOBJ, 1, False)

	Local $iHResult = $oIFontFace.GetGlyphRunOutline($fSize, $tGlyphIndices, $tGlyphAdvances, $tGlyphOffsets, $iGlyphCount, $bIsSideways, $bIsRightToLeft, $oGeometrySink)
	If $iHResult Then Return SetError($iHResult, 2, False)

	Return True
EndFunc



; _DWrite_FontFace_GetIndex
; _DWrite_FontFace_GetMetrics
; _DWrite_FontFace_GetRecommendedRenderingMode
; _DWrite_FontFace_GetSimulations
; _DWrite_FontFace_GetType
; _DWrite_FontFace_IsSymbolFont
; _DWrite_FontFace_ReleaseFontTable
; _DWrite_FontFace_TryGetFontTable





;#######################################################################################################################################################################
;# IDwriteFontFace1
;#######################################################################################################################################################################
#cs    IDwriteFontFace1
	Method ...: GetCaretMetrics (?)
	Info .....:     Gets caret metrics for the font in design units.
	Param ....:         CaretMetrics  <DWRITE_CARET_METRICS>  A pointer to the DWRITE_CARET_METRICS structure that is filled.

	Method ...: GetDesignGlyphAdvances (?)
	Info .....:     Retrieves the advances in design units for a sequences of glyphs.
	Param ....:         GlyphCount     <UINT32>      The number of glyphs to retrieve advances for.
	Param ....:         GlyphIndices   <UINT16>      An array of glyph id's to retrieve advances for.
	Param ....:         GlyphAdvances  <INT32>       The returned advances in font design units for each glyph.
	Param ....:         IsSideways     <isSideways>  Retrieve the glyph's vertical advance height rather than horizontal advance widths.

	Method ...: GetGdiCompatibleGlyphAdvances (?)
	Info .....:     Returns the pixel-aligned advances for a sequences of glyphs.
	Param ....:         EmSize         <FLOAT>          Logical size of the font in DIP units. A DIP ("device-independent pixel") equals 1/96 inch.
	Param ....:         PixelsPerDip   <FLOAT>          Number of physical pixels per DIP. For example, if the DPI of the rendering surface is 96 this value is 1.0f. If the DPI is 120, this value is 120.0f/96.
	Param ....:         Transform      <DWRITE_MATRIX>  Optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified by the font size and pixelsPerDip.
	Param ....:         UseGdiNatural  <BOOL>           When FALSE, the metrics are the same as GDI aliased text (DWRITE_MEASURING_MODE_GDI_CLASSIC). When TRUE, the metrics are the same as those measured by GDI using a font using CLEARTYPE_NATURAL_QUALITY (DWRITE_MEASURING_MODE_GDI_NATURAL).
	Param ....:         IsSideways     <BOOL>           Retrieve the glyph's vertical advances rather than horizontal advances.
	Param ....:         GlyphCount     <UINT32>         Total glyphs to retrieve adjustments for.
	Param ....:         GlyphIndices   <UINT16>         An array of glyph id's to retrieve advances.
	Param ....:         GlyphAdvances  <INT32>          The returned advances in font design units for each glyph.

	Method ...: GetGdiCompatibleMetrics (?)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a fontface and are used by applications for layout calculations.
	Param ....:         EmSize        <FLOAT>                 The logical size of the font in DIP units.
	Param ....:         PixelsPerDip  <FLOAT>                 The number of physical pixels per DIP.
	Param ....:         Transform     <DWRITE_MATRIX>         An optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified by the font size and "pixelsPerDip".
	Param ....:         FontMetrics   <DWRITE_FONT_METRICS1>  A pointer to a DWRITE_FONT_METRICS1 structure to fill in. The metrics returned by this function are in font design units.

	Method ...: GetKerningPairAdjustments (?)
	Info .....:     Retrieves the kerning pair adjustments from the font's kern table.
	Param ....:         GlyphCount               <UINT32>  Number of glyphs to retrieve adjustments for.
	Param ....:         GlyphIndices             <UINT16>  An array of glyph id's to retrieve adjustments for.
	Param ....:         GlyphAdvanceAdjustments  <INT32>   The advances, returned in font design units, for each glyph. The last glyph adjustment is zero.

	Method ...: GetMetrics (?)
	Info .....:     Obtains design units and common metrics for the font face. These metrics are applicable to all the glyphs within a font face and are used by applications for layout calculations.
	Param ....:         FontMetrics  <DWRITE_FONT_METRICS1>  A filled DWRITE_FONT_METRICS1 structure that holds metrics for the current font face element. The metrics returned by this method are in font design units.

	Method ...: GetRecommendedRenderingMode (?)
	Info .....:     Determines the recommended rendering mode for the font, using the specified size and rendering parameters.
	Param ....:         FontEmSize        <FLOAT>                     The logical size of the font in DIP units. A DIP ("device-independent pixel") equals 1/96 inch.
	Param ....:         DpiX              <FLOAT>                     The number of physical pixels per DIP in a horizontal position. For example, if the DPI of the rendering surface is 96, this value is 1.0f. If the DPI is 120, this value is 120.0f/96.
	Param ....:         DpiY              <FLOAT>                     The number of physical pixels per DIP in a vertical position. For example, if the DPI of the rendering surface is 96, this value is 1.0f. If the DPI is 120, this value is 120.0f/96.
	Param ....:         Transform         <DWRITE_MATRIX>             Specifies the world transform.
	Param ....:         IsSideways        <BOOL>                      Whether the glyphs in the run are sideways or not.
	Param ....:         OutlineThreshold  <DWRITE_OUTLINE_THRESHOLD>  A DWRITE_OUTLINE_THRESHOLD-typed value that specifies the quality of the graphics system's outline rendering, affects the size threshold above which outline rendering is used.
	Param ....:         MeasuringMode     <DWRITE_MEASURING_MODE>     The measuring method that will be used for glyphs in the font. Renderer implementations may choose different rendering modes for different measuring methods, for example:
	Param ....:         RenderingMode     <DWRITE_RENDERING_MODE>     When this method returns, contains a value that indicates the recommended rendering mode to use.

	Method ...: GetUnicodeRanges (?)
	Info .....:     Retrieves a list of character ranges supported by a font.
	Param ....:         MaxRangeCount     <UINT32>                Maximum number of character ranges passed in from the client.
	Param ....:         UnicodeRanges     <DWRITE_UNICODE_RANGE>  An array of DWRITE_UNICODE_RANGE structures that are filled with the character ranges.
	Param ....:         ActualRangeCount  <UINT32>                A pointer to the actual number of character ranges, regardless of the maximum count.

	Method ...: GetVerticalGlyphVariants (?)
	Info .....:     Retrieves the vertical forms of the nominal glyphs retrieved from GetGlyphIndices.
	Param ....:         GlyphCount            <UINT32>  The number of glyphs to retrieve.
	Param ....:         NominalGlyphIndices   <UINT16>  Original glyph indices from cmap.
	Param ....:         VerticalGlyphIndices  <UINT16>  The vertical form of glyph indices.

	Method ...: HasKerningPairs (?)
	Info .....:     Determines whether the font supports pair-kerning.
	Param ....:         This method has no parameters.

	Method ...: HasVerticalGlyphVariants (?)
	Info .....:     Determines whether the font has any vertical glyph variants.
	Param ....:         This method has no parameters.

	Method ...: IsMonospacedFont (?)
	Info .....:     Determines whether the font of a text range is monospaced, that is, the font characters are the same fixed-pitch width.
	Param ....:         This method has no parameters.

#ce
; _DWrite_FontFace1_GetCaretMetrics
; _DWrite_FontFace1_GetDesignGlyphAdvances
; _DWrite_FontFace1_GetGdiCompatibleGlyphAdvances
; _DWrite_FontFace1_GetGdiCompatibleMetrics
; _DWrite_FontFace1_GetKerningPairAdjustments
; _DWrite_FontFace1_GetMetrics
; _DWrite_FontFace1_GetRecommendedRenderingMode
; _DWrite_FontFace1_GetUnicodeRanges
; _DWrite_FontFace1_GetVerticalGlyphVariants
; _DWrite_FontFace1_HasKerningPairs
; _DWrite_FontFace1_HasVerticalGlyphVariants
; _DWrite_FontFace1_IsMonospacedFont





;#######################################################################################################################################################################
;# IDwriteFontFace2
;#######################################################################################################################################################################
#cs    IDwriteFontFace2
	Method ...: GetColorPaletteCount (?)
	Info .....:     Gets the number of color palettes defined by the font.
	Param ....:         This method has no parameters.

	Method ...: GetPaletteEntries (?)
	Info .....:     Gets color values from the font's color palette.
	Param ....:         ColorPaletteIndex  <UINT32>          Zero-based index of the color palette. If the font does not have a palette with the specified index, the method returns DWRITE_E_NOCOLOR.
	Param ....:         FirstEntryIndex    <UINT32>          Zero-based index of the first palette entry to read.
	Param ....:         EntryCount         <UINT32>          Number of palette entries to read.
	Param ....:         PaletteEntries     <DWRITE_COLOR_F>  Array that receives the color values.

	Method ...: GetPaletteEntryCount (?)
	Info .....:     Get the number of entries in each color palette.
	Param ....:         This method has no parameters.

	Method ...: IsColorFont (?)
	Info .....:     Allows you to determine if a color rendering path is potentially necessary.
	Param ....:         This method has no parameters.

#ce
; _DWrite_FontFace2_GetColorPaletteCount
; _DWrite_FontFace2_GetPaletteEntries
; _DWrite_FontFace2_GetPaletteEntryCount
; _DWrite_FontFace2_IsColorFont





;#######################################################################################################################################################################
;# IDwriteFontFallback
;#######################################################################################################################################################################
#cs    IDwriteFontFallback
	Method ...: MapCharacters (?)
	Info .....:     Determines an appropriate font to use to render the beginning range of text.
	Param ....:         Source              <IDWriteTextAnalysisSource>  The text source implementation holds the text and locale.
	Param ....:         TextPosition        <UINT32>                     Starting position to analyze.
	Param ....:         TextLength          <UINT32>                     Length of the text to analyze.
	Param ....:         BaseFontCollection  <IDWriteFontCollection>      Default font collection to use.
	Param ....:         BaseFamilyName      <wchar_t>                    Family name of the base font. If you pass null, no matching will be done against the family.
	Param ....:         BaseWeight          <DWRITE_FONT_WEIGHT>         The desired weight.
	Param ....:         BaseStyle           <DWRITE_FONT_STYLE>          The desired style.
	Param ....:         BaseStretch         <DWRITE_FONT_STRETCH>        The desired stretch.
	Param ....:         MappedLength        <UINT32>                     Length of text mapped to the mapped font. This will always be less than or equal to the text length and greater than zero (if the text length is non-zero) so the caller advances at least one character.
	Param ....:         MappedFont          <IDWriteFont>                The font that should be used to render the first "mappedLength" characters of the text. If it returns NULL, that means that no font can render the text, and "mappedLength" is the number of characters to skip (rendered with a missing glyph).
	Param ....:         Scale               <FLOAT>                      Scale factor to multiply the em size of the returned font by.

#ce
; _DWrite_FontFallback_MapCharacters





;#######################################################################################################################################################################
;# IDwriteFontFallbackBuilder
;#######################################################################################################################################################################
#cs    IDwriteFontFallbackBuilder
	Method ...: AddMapping (?)
	Info .....:     Appends a single mapping to the list. Call this once for each additional mapping.
	Param ....:         Ranges                  <DWRITE_UNICODE_RANGE>  Unicode ranges that apply to this mapping.
	Param ....:         RangesCount             <UINT32>                Number of Unicode ranges.
	Param ....:         TargetFamilyNames       <WCHAR>                 List of target family name strings.
	Param ....:         TargetFamilyNamesCount  <UINT32>                Number of target family names.
	Param ....:         FontCollection          <fontCollection>        Optional explicit font collection for this mapping.
	Param ....:         LocaleName              <localeName>            Locale of the context.
	Param ....:         BaseFamilyName          <baseFamilyName>        Base family name to match against, if applicable.
	Param ....:         Scale                   <scale>                 Scale factor to multiply the result target font by.

	Method ...: AddMappings (?)
	Info .....:     Add all the mappings from an existing font fallback object.
	Param ....:         FontFallback  <IDWriteFontFallback>  An existing font fallback object.

	Method ...: CreateFontFallback (?)
	Info .....:     Creates the finalized fallback object from the mappings added.
	Param ....:         FontFallback  <IDWriteFontFallback>  Contains an address of a pointer to the created fallback list.

#ce
; _DWrite_FontFallbackBuilder_AddMapping
; _DWrite_FontFallbackBuilder_AddMappings
; _DWrite_FontFallbackBuilder_CreateFontFallback





;#######################################################################################################################################################################
;# IDwriteFontFamily
;#######################################################################################################################################################################
#cs    IDwriteFontFamily
	Method ...: GetFamilyNames (ptr*)
	Info .....:     Creates a localized strings object that contains the family names for the font family, indexed by locale name.
	Param ....:         Names  <IDWriteLocalizedStrings>  The address of a pointer to the newly created IDWriteLocalizedStrings object.

	Method ...: GetFirstMatchingFont (uint;uint;uint;ptr*)
	Info .....:     Gets the font that best matches the specified properties.
	Param ....:         Weight        <DWRITE_FONT_WEIGHT>   A value that is used to match a requested font weight.
	Param ....:         Stretch       <DWRITE_FONT_STRETCH>  A value that is used to match a requested font stretch.
	Param ....:         Style         <DWRITE_FONT_STYLE>    A value that is used to match a requested font style.
	Param ....:         MatchingFont  <IDWriteFont>          When this method returns, contains the address of a pointer to the newly created IDWriteFont object.

	Method ...: GetMatchingFonts (uint;uint;uint;ptr*)
	Info .....:     Gets a list of fonts in the font family ranked in order of how well they match the specified properties.
	Param ....:         Weight         <DWRITE_FONT_WEIGHT>   A value that is used to match a requested font weight.
	Param ....:         Stretch        <DWRITE_FONT_STRETCH>  A value that is used to match a requested font stretch.
	Param ....:         Style          <DWRITE_FONT_STYLE>    A value that is used to match a requested font style.
	Param ....:         MatchingFonts  <IDWriteFontList>      An address of a pointer to the newly created IDWriteFontList object.

#ce
; _DWrite_FontFamily_GetFamilyNames



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_FontFamily_GetFirstMatchingFont
; Description ...:  Gets the font that best matches the specified properties.
; Syntax ........: _DWrite_FontFamily_GetFirstMatchingFont($oIFontFamily[, $iWeight = $DWRITE_FONT_WEIGHT_NORMAL[, $iStretch = $DWRITE_FONT_STRETCH_NORMAL[, $iStyle = $DWRITE_FONT_STYLE_NORMAL]]])
; Parameters ....: $oIFontFamily - This IDWriteFontFamily object
;                  $iWeight      - [optional] A $DWRITE_FONT_WEIGHT enumeraion. A value that is used to match a requested font weight.
;                  $iStretch     - [optional] A $DWRITE_FONT_STRETCH enumeraion. A value that is used to match a requested font stretch.
;                  $iStyle       - [optional] A $DWRITE_FONT_STYLE enumeraion. A value that is used to match a requested font style.
; Return values .: Success - An IDWriteFont object.
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_FontFamily_GetFirstMatchingFont($oIFontFamily, $iWeight = $DWRITE_FONT_WEIGHT_NORMAL, $iStretch = $DWRITE_FONT_STRETCH_NORMAL, $iStyle = $DWRITE_FONT_STYLE_NORMAL)
	If Not IsObj($oIFontFamily) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $pMatchingFont
	Local $iHResult = $oIFontFamily.GetFirstMatchingFont($iWeight, $iStretch, $iStyle, $pMatchingFont)
	If $iHResult Or Not $pMatchingFont Then Return SetError($iHResult, 1, False)

	Local $oMatchingFont = ObjCreateInterface($pMatchingFont, $sIID_IDWriteFont, $tagIDWriteFont)
	If Not IsObj($oMatchingFont) Then Return SetError($DWRITEERR_OBJFAIL, 2, False)
	Return $oMatchingFont
EndFunc


; _DWrite_FontFamily_GetMatchingFonts





;#######################################################################################################################################################################
;# IDwriteFontFile
;#######################################################################################################################################################################
#cs    IDwriteFontFile
	Method ...: Analyze (bool*;uint*;uint*;uint*)
	Info .....:     Analyzes a file and returns whether it represents a font, and whether the font type is supported by the font system.
	Param ....:         IsSupportedFontType  <BOOL>                    TRUE if the font type is supported by the font system; otherwise, FALSE.
	Param ....:         FontFileType         <DWRITE_FONT_FILE_TYPE>  When this method returns, contains a value that indicates the type of the font file. Note that even if " isSupportedFontType" is FALSE, the "fontFileType" value may be different from DWRITE_FONT_FILE_TYPE_UNKNOWN.
	Param ....:         FontFaceType         <DWRITE_FONT_FACE_TYPE>  When this method returns, contains a value that indicates the type of the font face. If "fontFileType" is not equal to DWRITE_FONT_FILE_TYPE_UNKNOWN, then that can be constructed from the font file.
	Param ....:         NumberOfFaces        <UINT32>                 When this method returns, contains the number of font faces contained in the font file.

	Method ...: GetLoader (ptr*)
	Info .....:     Obtains the file loader associated with a font file object.
	Param ....:         FontFileLoader  <IDWriteFontFileLoader>  When this method returns, contains the address of a pointer to the font file loader associated with the font file object.

	Method ...: GetReferenceKey (ptr*;uint*)
	Info .....:     Obtains the pointer to the reference key of a font file. The returned pointer is valid until the font file object is released.
	Param ....:         FontFileReferenceKey      <void>    When this method returns, contains an address of a pointer to the font file reference key. Note that the pointer value is only valid until the font file object it is obtained from is released. This parameter is passed uninitialized.
	Param ....:         FontFileReferenceKeySize  <UINT32>  When this method returns, contains the size of the font file reference key in bytes. This parameter is passed uninitialized.

#ce
; _DWrite_FontFile_Analyze
; _DWrite_FontFile_GetLoader
; _DWrite_FontFile_GetReferenceKey





;#######################################################################################################################################################################
;# IDwriteFontFile
;#######################################################################################################################################################################
#cs    IDwriteFontFile
	Method ...: Analyze (bool*;uint*;uint*;uint*)
	Info .....:     Analyzes a file and returns whether it represents a font, and whether the font type is supported by the font system.
	Param ....:         IsSupportedFontType  <BOOL>                    TRUE if the font type is supported by the font system; otherwise, FALSE.
	Param ....:         FontFileType         <DWRITE_FONT_FILE_TYPE>  When this method returns, contains a value that indicates the type of the font file. Note that even if " isSupportedFontType" is FALSE, the "fontFileType" value may be different from DWRITE_FONT_FILE_TYPE_UNKNOWN.
	Param ....:         FontFaceType         <DWRITE_FONT_FACE_TYPE>  When this method returns, contains a value that indicates the type of the font face. If "fontFileType" is not equal to DWRITE_FONT_FILE_TYPE_UNKNOWN, then that can be constructed from the font file.
	Param ....:         NumberOfFaces        <UINT32>                 When this method returns, contains the number of font faces contained in the font file.

	Method ...: GetLoader (ptr*)
	Info .....:     Obtains the file loader associated with a font file object.
	Param ....:         FontFileLoader  <IDWriteFontFileLoader>  When this method returns, contains the address of a pointer to the font file loader associated with the font file object.

	Method ...: GetReferenceKey (ptr*;uint*)
	Info .....:     Obtains the pointer to the reference key of a font file. The returned pointer is valid until the font file object is released.
	Param ....:         FontFileReferenceKey      <void>    When this method returns, contains an address of a pointer to the font file reference key. Note that the pointer value is only valid until the font file object it is obtained from is released. This parameter is passed uninitialized.
	Param ....:         FontFileReferenceKeySize  <UINT32>  When this method returns, contains the size of the font file reference key in bytes. This parameter is passed uninitialized.

#ce
; _DWrite_FontFile_Analyze
; _DWrite_FontFile_GetLoader
; _DWrite_FontFile_GetReferenceKey





;#######################################################################################################################################################################
;# IDwriteFontFileEnumerator
;#######################################################################################################################################################################
#cs    IDwriteFontFileEnumerator
	Method ...: GetCurrentFontFile (ptr*)
	Info .....:     Gets a reference to the current font file.
	Param ....:         FontFile  <IDWriteFontFile>  When this method returns, the address of a pointer to the newly created IDWriteFontFile object.

	Method ...: MoveNext (bool*)
	Info .....:     Advances to the next font file in the collection. When it is first created, the enumerator is positioned before the first element of the collection and the first call to MoveNext advances to the first file.
	Param ....:         HasCurrentFile  <BOOL>  When the method returns, contains the value TRUE if the enumerator advances to a file; otherwise, FALSE if the enumerator advances past the last file in the collection.

#ce
; _DWrite_FontFileEnumerator_GetCurrentFontFile
; _DWrite_FontFileEnumerator_MoveNext





;#######################################################################################################################################################################
;# IDwriteFontFileLoader
;#######################################################################################################################################################################
#cs    IDwriteFontFileLoader
	Method ...: CreateStreamFromKey (struct*;uint;ptr*)
	Info .....:     Creates a font file stream object that encapsulates an open file resource.
	Param ....:         FontFileReferenceKey      <void>                   A pointer to a font file reference key that uniquely identifies the font file resource within the scope of the font loader being used. The buffer allocated for this key must at least be the size, in bytes, specified by " fontFileReferenceKeySize".
	Param ....:         FontFileReferenceKeySize  <UINT32>                 The size of font file reference key, in bytes.
	Param ....:         FontFileStream            <IDWriteFontFileStream>  When this method returns, contains the address of a pointer to the newly created IDWriteFontFileStream object.

#ce
; _DWrite_FontFileLoader_CreateStreamFromKey





;#######################################################################################################################################################################
;# IDwriteFontFileStream
;#######################################################################################################################################################################
#cs    IDwriteFontFileStream
	Method ...: GetFileSize (uint64*)
	Info .....:     Obtains the total size of a file.
	Param ....:         FileSize  <UINT64>  When this method returns, contains the total size of the file.

	Method ...: GetLastWriteTime (uint64*)
	Info .....:     Obtains the last modified time of the file.
	Param ....:         LastWriteTime  <UINT64>  When this method returns, contains the last modified time of the file in the format that represents the number of 100-nanosecond intervals since January 1, 1601 (UTC).

	Method ...: ReadFileFragment (ptr*;uint64;uint64;ptr*)
	Info .....:     Reads a fragment from a font file.
	Param ....:         FragmentStart    <void>    When this method returns, contains an address of a pointer to the start of the font file fragment. This parameter is passed uninitialized.
	Param ....:         FileOffset       <UINT64>  The offset of the fragment, in bytes, from the beginning of the font file.
	Param ....:         FragmentSize     <UINT64>  The size of the file fragment, in bytes.
	Param ....:         FragmentContext  <void>    When this method returns, contains the address of a pointer to a pointer to the client-defined context to be passed to ReleaseFileFragment.

	Method ...: ReleaseFileFragment (struct*)
	Info .....:     Releases a fragment from a file.
	Param ....:         FragmentContext  <void>  A pointer to the client-defined context of a font fragment returned from ReadFileFragment.

#ce
; _DWrite_FontFileStream_GetFileSize
; _DWrite_FontFileStream_GetLastWriteTime
; _DWrite_FontFileStream_ReadFileFragment
; _DWrite_FontFileStream_ReleaseFileFragment





;#######################################################################################################################################################################
;# IDwriteFontList
;#######################################################################################################################################################################
#cs    IDwriteFontList
	Method ...: GetFont (ptr*)
	Info .....:     Gets a font given its zero-based index.
	Param ....:         Index  <UINT32>       Zero-based index of the font in the font list.
	Param ....:         Font   <IDWriteFont>  When this method returns, contains the address of a pointer to the newly created IDWriteFont object.

	Method ...: GetFontCollection (ptr*)
	Info .....:     Gets the font collection that contains the fonts in the font list.
	Param ....:         FontCollection  <IDWriteFontCollection>  When this method returns, contains the address of a pointer to the current IDWriteFontCollection object.

	Method ...: GetFontCount ()
	Info .....:     Gets the number of fonts in the font list.
	Param ....:         This method has no parameters.

#ce
; _DWrite_FontList_GetFont
; _DWrite_FontList_GetFontCollection
; _DWrite_FontList_GetFontCount





;#######################################################################################################################################################################
;# IDwriteGdiInterop
;#######################################################################################################################################################################
#cs    IDwriteGdiInterop
	Method ...: ConvertFontFaceToLOGFONT (struct*;ptr*)
	Info .....:     Initializes a LOGFONT structure based on the GDI-compatible properties of the specified font.
	Param ....:         Font     <IDWriteFontFace>  An IDWriteFontFace object to be converted into a GDI-compatible LOGFONT structure.
	Param ....:         LogFont  <LOGFONTW>         When this method returns, contains a pointer to a structure that receives a GDI-compatible font description.

	Method ...: ConvertFontToLOGFONT (struct*;struct*;bool*;)
	Info .....:     Initializes a LOGFONT structure based on the GDI-compatible properties of the specified font.
	Param ....:         Font          <IDWriteFont>  An IDWriteFont object to be converted into a GDI-compatible LOGFONT structure.
	Param ....:         LogFont       <LOGFONTW>     When this method returns, contains a structure that receives a GDI-compatible font description.
	Param ....:         IsSystemFont  <BOOL>         When this method returns, contains TRUE if the specified font object is part of the system font collection; otherwise, FALSE.

	Method ...: CreateBitmapRenderTarget (handle;uint;uint;ptr*)
	Info .....:     Creates an object that encapsulates a bitmap and memory DC (device context) which can be used for rendering glyphs.
	Param ....:         Hdc           <HDC>                        A handle to the optional device context used to create a compatible memory DC (device context).
	Param ....:         Width         <UINT32>                     The width of the bitmap render target.
	Param ....:         Height        <UINT32>                     The height of the bitmap render target.
	Param ....:         RenderTarget  <IDWriteBitmapRenderTarget>  When this method returns, contains an address of a pointer to the newly created IDWriteBitmapRenderTarget object.

	Method ...: CreateFontFaceFromHdc (handle;ptr*)
	Info .....:     Creates an IDWriteFontFace object that corresponds to the currently selected HFONT of the specified HDC.
	Param ....:         Hdc       <HDC>              A handle to a device context into which a font has been selected. It is assumed that the client has already performed font mapping and that the font selected into the device context is the actual font to be used for rendering glyphs.
	Param ....:         FontFace  <IDWriteFontFace>  Contains an address of a pointer to the newly created font face object, or NULL in case of failure. The font face returned is guaranteed to reference the same physical typeface that would be used for drawing glyphs (but not necessarily characters) using ExtTextOut.

	Method ...: CreateFontFromLOGFONT (struct*;ptr*)
	Info .....:     Creates a font object that matches the properties specified by the LOGFONT structure.
	Param ....:         LogFont  <LOGFONTW>     A structure containing a GDI-compatible font description.
	Param ....:         Font     <IDWriteFont>  When this method returns, contains an address of a pointer to a newly created IDWriteFont object if successful; otherwise, NULL.

#ce
; _DWrite_GdiInterop_ConvertFontFaceToLOGFONT
; _DWrite_GdiInterop_ConvertFontToLOGFONT
; _DWrite_GdiInterop_CreateBitmapRenderTarget
; _DWrite_GdiInterop_CreateFontFaceFromHdc
; _DWrite_GdiInterop_CreateFontFromLOGFONT





;#######################################################################################################################################################################
;# IDwriteGdiInterop
;#######################################################################################################################################################################
#cs    IDwriteGdiInterop
#ce





;#######################################################################################################################################################################
;# IDwriteGdiInterop
;#######################################################################################################################################################################
#cs    IDwriteGdiInterop
#ce





;#######################################################################################################################################################################
;# IDwriteGlyphRunAnalysis
;#######################################################################################################################################################################
#cs    IDwriteGlyphRunAnalysis
	Method ...: CreateAlphaTexture (uint;struct*;struct*;uint)
	Info .....:     Creates an alpha texture of the specified type for glyphs within a specified bounding rectangle.
	Param ....:         TextureType    <DWRITE_TEXTURE_TYPE>  A value that specifies the type of texture requested. This can be DWRITE_TEXTURE_BILEVEL_1x1 or DWRITE_TEXTURE_CLEARTYPE_3x1. If a bi-level texture is requested, the texture contains only bi-level glyphs. Otherwise, the texture contains only antialiased glyphs.
	Param ....:         TextureBounds  <RECT>                 The bounding rectangle of the texture, which can be different than the bounding rectangle returned by GetAlphaTextureBounds.
	Param ....:         AlphaValues    <BYTE>                 When this method returns, contains the array of alpha values from the texture. The buffer allocated for this array must be at least the size of "bufferSize".
	Param ....:         BufferSize     <UINT32>               The size of the "alphaValues" array, in bytes. The minimum size depends on the dimensions of the rectangle and the type of texture requested.

	Method ...: GetAlphaBlendParams (struct*;float*;float*;float*)
	Info .....:     Gets alpha blending properties required for ClearType blending.
	Param ....:         RenderingParams        <IDWriteRenderingParams>  An object that specifies the ClearType level and enhanced contrast, gamma, pixel geometry, and rendering mode. In most cases, the values returned by the output parameters of this method are based on the properties of this object, unless a GDI-compatible rendering mode was specified.
	Param ....:         BlendGamma             <FLOAT>                   When this method returns, contains the gamma value to use for gamma correction.
	Param ....:         BlendEnhancedContrast  <FLOAT>                   When this method returns, contains the enhanced contrast value to be used for blending.
	Param ....:         BlendClearTypeLevel    <FLOAT>                   When this method returns, contains the ClearType level used in the alpha blending.

	Method ...: GetAlphaTextureBounds (uint;ptr*)
	Info .....:     Gets the bounding rectangle of the physical pixels affected by the glyph run.
	Param ....:         TextureType    <DWRITE_TEXTURE_TYPE>  Specifies the type of texture requested. If a bi-level texture is requested, the bounding rectangle includes only bi-level glyphs. Otherwise, the bounding rectangle includes only antialiased glyphs.
	Param ....:         TextureBounds  <RECT>                 When this method returns, contains the bounding rectangle of the physical pixels affected by the glyph run, or an empty rectangle if there are no glyphs of the specified texture type.

#ce
; _DWrite_GlyphRunAnalysis_CreateAlphaTexture
; _DWrite_GlyphRunAnalysis_GetAlphaBlendParams
; _DWrite_GlyphRunAnalysis_GetAlphaTextureBounds





;#######################################################################################################################################################################
;# IDwriteInlineObject
;#######################################################################################################################################################################
#cs    IDwriteInlineObject
	Method ...: Draw (struct*;struct*;float;float;bool;bool;struct*)
	Info .....:     The application implemented rendering callback ( IDWriteTextRenderer::DrawInlineObject) can use this to draw the inline object without needing to cast or query the object type. The text layout does not call this method directly.
	Param ....:         ClientDrawingContext  <void>                 The drawing context passed to IDWriteTextLayout::Draw. This parameter may be NULL.
	Param ....:         Renderer              <IDWriteTextRenderer>  The same renderer passed to IDWriteTextLayout::Draw as the object's containing parent. This is useful if the inline object is recursive such as a nested layout.
	Param ....:         OriginX               <FLOAT>                The x-coordinate at the upper-left corner of the inline object.
	Param ....:         OriginY               <FLOAT>                The y-coordinate at the upper-left corner of the inline object.
	Param ....:         IsSideways            <BOOL>                 A Boolean flag that indicates whether the object's baseline runs alongside the baseline axis of the line.
	Param ....:         IsRightToLeft         <BOOL>                 A Boolean flag that indicates whether the object is in a right-to-left context and should be drawn flipped.
	Param ....:         ClientDrawingEffect   <IUnknown>             The drawing effect set in IDWriteTextLayout::SetDrawingEffect. Usually this effect is a foreground brush that is used in glyph drawing.

	Method ...: GetBreakConditions (uint*;uint*)
	Info .....:     Layout uses this to determine the line-breaking behavior of the inline object among the text.
	Param ....:         BreakConditionBefore  <DWRITE_BREAK_CONDITION>  When this method returns, contains a value which indicates the line-breaking condition between the object and the content immediately preceding it.
	Param ....:         BreakConditionAfter   <DWRITE_BREAK_CONDITION>  When this method returns, contains a value which indicates the line-breaking condition between the object and the content immediately following it.

	Method ...: GetMetrics (struct*)
	Info .....:     IDWriteTextLayout calls this callback function to get the measurement of the inline object.
	Param ....:         Metrics  <DWRITE_INLINE_OBJECT_METRICS>  When this method returns, contains a structure describing the geometric measurement of anapplication-defined inline object. These metrics are in relation to the baseline of the adjacent text.

	Method ...: GetOverhangMetrics (struct*)
	Info .....:     IDWriteTextLayout calls this callback function to get the visible extents (in DIPs) of the inline object. In the case of a simple bitmap, with no padding and no overhang, all the overhangs will simply be zeroes.
	Param ....:         Overhangs  <DWRITE_OVERHANG_METRICS>  Overshoot of visible extents (in DIPs) outside the object.

#ce
; _DWrite_InlineObject_Draw
; _DWrite_InlineObject_GetBreakConditions
; _DWrite_InlineObject_GetMetrics
; _DWrite_InlineObject_GetOverhangMetrics





;#######################################################################################################################################################################
;# IDwriteLocalFontFileLoader
;#######################################################################################################################################################################
#cs    IDwriteLocalFontFileLoader
	Method ...: GetFilePathFromKey (struct*;uint;wstr;uint)
	Info .....:     Obtains the absolute font file path from the font file reference key.
	Param ....:         FontFileReferenceKey      <void>    The font file reference key that uniquely identifies the local font file within the scope of the font loader being used.
	Param ....:         FontFileReferenceKeySize  <UINT32>  The size of font file reference key in bytes.
	Param ....:         FilePath                  <WCHAR>   The character array that receives the local file path.
	Param ....:         FilePathSize              <UINT32>  The length of the file path character array.

	Method ...: GetFilePathLengthFromKey (struct*;uint;uint*)
	Info .....:     Obtains the length of the absolute file path from the font file reference key.
	Param ....:         FontFileReferenceKey      <void>    Font file reference key that uniquely identifies the local font file within the scope of the font loader being used.
	Param ....:         FontFileReferenceKeySize  <UINT32>  Size of font file reference key in bytes.
	Param ....:         FilePathLength            <UINT32>  Length of the file path string, not including the terminated NULL character.

	Method ...: GetLastWriteTimeFromKey (struct*;uint;struct*)
	Info .....:     Obtains the last write time of the file from the font file reference key.
	Param ....:         FontFileReferenceKey      <void>      The font file reference key that uniquely identifies the local font file within the scope of the font loader being used.
	Param ....:         FontFileReferenceKeySize  <UINT32>    The size of font file reference key in bytes.
	Param ....:         LastWriteTime             <FILETIME>  The time of the last font file modification.

#ce
; _DWrite_LocalFontFileLoader_GetFilePathFromKey
; _DWrite_LocalFontFileLoader_GetFilePathLengthFromKey
; _DWrite_LocalFontFileLoader_GetLastWriteTimeFromKey





;#######################################################################################################################################################################
;# IDwriteFontFileLoader
;#######################################################################################################################################################################
#cs    IDwriteFontFileLoader
	Method ...: CreateStreamFromKey (struct*;uint;ptr*)
	Info .....:     Creates a font file stream object that encapsulates an open file resource.
	Param ....:         FontFileReferenceKey      <void>                   A pointer to a font file reference key that uniquely identifies the font file resource within the scope of the font loader being used. The buffer allocated for this key must at least be the size, in bytes, specified by " fontFileReferenceKeySize".
	Param ....:         FontFileReferenceKeySize  <UINT32>                 The size of font file reference key, in bytes.
	Param ....:         FontFileStream            <IDWriteFontFileStream>  When this method returns, contains the address of a pointer to the newly created IDWriteFontFileStream object.

#ce
; _DWrite_FontFileLoader_CreateStreamFromKey





;#######################################################################################################################################################################
;# IDwriteLocalizedStrings
;#######################################################################################################################################################################
#cs    IDwriteLocalizedStrings
	Method ...: FindLocaleName (wstr;uint*;bool*)
	Info .....:     Gets the zero-based index of the locale name/string pair with the specified locale name.
	Param ....:         LocaleName  <WCHAR>   A null-terminated array of characters containing the locale name to look for.
	Param ....:         Index       <UINT32>  The zero-based index of the locale name/string pair. This method initializes "index" to UINT_MAX.
	Param ....:         Exists      <BOOL>    When this method returns, contains TRUE if the locale name exists; otherwise, FALSE. This method initializes "exists" to FALSE.

	Method ...: GetCount ()
	Info .....:     Gets the number of language/string pairs.
	Param ....:         This method has no parameters.

	Method ...: GetLocaleName (uint;uint*)
	Info .....:     Copies the locale name with the specified index to the specified array.
	Param ....:         Index       <UINT32>  Zero-based index of the locale name to be retrieved.
	Param ....:         LocaleName  <WCHAR>   When this method returns, contains a character array, which is null-terminated, that receives the locale name from the language/string pair. The buffer allocated for this array must be at least the size of "size", in element count.
	Param ....:         Size        <UINT32>  The size of the array in characters. The size must include space for the terminating null character.

	Method ...: GetLocaleNameLength (uint;uint*)
	Info .....:     Gets the length in characters (not including the null terminator) of the locale name with the specified index.
	Param ....:         Index   <UINT32>  Zero-based index of the locale name to be retrieved.
	Param ....:         Length  <UINT32>  When this method returns, contains the length in characters of the locale name, not including the null terminator.

	Method ...: GetString (uint;uint*)
	Info .....:     Copies the string with the specified index to the specified array.
	Param ....:         Index         <UINT32>  The zero-based index of the language/string pair to be examined.
	Param ....:         StringBuffer  <WCHAR>   The null terminated array of characters that receives the string from the language/string pair. The buffer allocated for this array should be at least the size of "size". GetStringLength can be used to get the size of the array before using this method.
	Param ....:         Size          <UINT32>  The size of the array in characters. The size must include space for the terminating null character. GetStringLength can be used to get the size of the array before using this method.

	Method ...: GetStringLength (uint;uint*)
	Info .....:     Gets the length in characters (not including the null terminator) of the string with the specified index.
	Param ....:         Index   <UINT32>  A zero-based index of the language/string pair.
	Param ....:         Length  <UINT32>  The length in characters of the string, not including the null terminator, from the language/string pair.

#ce
; _DWrite_LocalizedStrings_FindLocaleName
; _DWrite_LocalizedStrings_GetCount
; _DWrite_LocalizedStrings_GetLocaleName
; _DWrite_LocalizedStrings_GetLocaleNameLength
; _DWrite_LocalizedStrings_GetString
; _DWrite_LocalizedStrings_GetStringLength





;#######################################################################################################################################################################
;# IDwriteLocalizedStrings
;#######################################################################################################################################################################
#cs    IDwriteLocalizedStrings
#ce





;#######################################################################################################################################################################
;# IDwritePixelSnapping
;#######################################################################################################################################################################
#cs    IDwritePixelSnapping
	Method ...: GetCurrentTransform (struct*;struct*)
	Info .....:     Gets a transform that maps abstract coordinates to DIPs.
	Param ....:         ClientDrawingContext  <void>           The drawing context passed to IDWriteTextLayout::Draw.
	Param ....:         Transform             <DWRITE_MATRIX>  When this method returns, contains a structure which has transform information for pixel snapping.

	Method ...: GetPixelsPerDip (struct*;float*)
	Info .....:     Gets the number of physical pixels per DIP.
	Param ....:         ClientDrawingContext  <void>   The drawing context passed to IDWriteTextLayout::Draw.
	Param ....:         PixelsPerDip          <FLOAT>  When this method returns, contains the number of physical pixels per DIP.

	Method ...: IsPixelSnappingEnabled (struct*;bool*)
	Info .....:     Determines whether pixel snapping is disabled. The recommended default is FALSE, unless doing animation that requires subpixel vertical placement.
	Param ....:         ClientDrawingContext  <void>  The context passed to IDWriteTextLayout::Draw.
	Param ....:         IsDisabled            <BOOL>  Receives TRUE if pixel snapping is disabled; otherwise, FALSE.

#ce
; _DWrite_PixelSnapping_GetCurrentTransform
; _DWrite_PixelSnapping_GetPixelsPerDip
; _DWrite_PixelSnapping_IsPixelSnappingEnabled





;#######################################################################################################################################################################
;# IDwriteRenderingParams
;#######################################################################################################################################################################
#cs    IDwriteRenderingParams
	Method ...: GetClearTypeLevel ()
	Info .....:     Gets the ClearType level of the rendering parameters object.
	Param ....:         This method has no parameters.

	Method ...: GetEnhancedContrast ()
	Info .....:     Gets the enhanced contrast property of the rendering parameters object. Valid values are greater than or equal to zero.
	Param ....:         This method has no parameters.

	Method ...: GetGamma ()
	Info .....:     Gets the gamma value used for gamma correction. Valid values must be greater than zero and cannot exceed 256.
	Param ....:         This method has no parameters.

	Method ...: GetPixelGeometry ()
	Info .....:     Gets the pixel geometry of the rendering parameters object.
	Param ....:         This method has no parameters.

	Method ...: GetRenderingMode ()
	Info .....:     Gets the rendering mode of the rendering parameters object.
	Param ....:         This method has no parameters.

#ce
; _DWrite_RenderingParams_GetClearTypeLevel
; _DWrite_RenderingParams_GetEnhancedContrast
; _DWrite_RenderingParams_GetGamma
; _DWrite_RenderingParams_GetPixelGeometry
; _DWrite_RenderingParams_GetRenderingMode




;#######################################################################################################################################################################
;# IDwriteRenderingParams1
;#######################################################################################################################################################################
#cs    IDwriteRenderingParams1
	Method ...: GetGrayscaleEnhancedContrast (?)
	Info .....:     Gets the amount of contrast enhancement to use for grayscale antialiasing.
	Param ....:         This method has no parameters.

#ce
; _DWrite_RenderingParams1_GetGrayscaleEnhancedContrast





;#######################################################################################################################################################################
;# IDwriteTextAnalysisSink
;#######################################################################################################################################################################
#cs    IDwriteTextAnalysisSink
	Method ...: SetBidiLevel (?)
	Info .....:     Sets a bidirectional level on the range, which is called once per run change (either explicit or resolved implicit).
	Param ....:         TextPosition   <UINT32>  The starting position from which to report.
	Param ....:         TextLength     <UINT32>  The number of UTF16 units of the reported range.
	Param ....:         ExplicitLevel  <UINT8>   The explicit level from the paragraph reading direction and any embedded control codes RLE/RLO/LRE/LRO/PDF, which is determined before any additional rules.
	Param ....:         ResolvedLevel  <UINT8>   The final implicit level considering the explicit level and characters' natural directionality, after all Bidi rules have been applied.

	Method ...: SetLineBreakpoints (?)
	Info .....:     Sets line-break opportunities for each character, starting from the specified position.
	Param ....:         TextPosition     <UINT32>                  The starting text position from which to report.
	Param ....:         TextLength       <UINT32>                  The number of UTF16 units of the reported range.
	Param ....:         LineBreakpoints  <DWRITE_LINE_BREAKPOINT>  A pointer to a structure that contains breaking conditions set for each character from the starting position to the end of the specified range.

	Method ...: SetNumberSubstitution (?)
	Info .....:     Sets the number substitution on the text range affected by the text analysis.
	Param ....:         TextPosition        <UINT32>                     The starting position from which to report.
	Param ....:         TextLength          <UINT32>                     The number of UTF16 units of the reported range.
	Param ....:         NumberSubstitution  <IDWriteNumberSubstitution>  An object that holds the appropriate digits and numeric punctuation for a given locale. Use IDWriteFactory::CreateNumberSubstitution to create this object.

	Method ...: SetScriptAnalysis (?)
	Info .....:     Reports script analysis for the specified text range.
	Param ....:         TextPosition    <UINT32>                  The starting position from which to report.
	Param ....:         TextLength      <UINT32>                  The number of UTF16 units of the reported range.
	Param ....:         ScriptAnalysis  <DWRITE_SCRIPT_ANALYSIS>  A pointer to a structure that contains a zero-based index representation of a writing system script and a value indicating whether additional shaping of text is required.

#ce
; _DWrite_TextAnalysisSink_SetBidiLevel
; _DWrite_TextAnalysisSink_SetLineBreakpoints
; _DWrite_TextAnalysisSink_SetNumberSubstitution
; _DWrite_TextAnalysisSink_SetScriptAnalysis





;#######################################################################################################################################################################
;# IDwriteTextAnalysisSink1
;#######################################################################################################################################################################
#cs    IDwriteTextAnalysisSink1
	Method ...: SetGlyphOrientation (?)
	Info .....:     The text analyzer calls back to this to report the actual orientation of each character for shaping and drawing.
	Param ....:         TextPosition           <UINT32>                          The starting position to report from.
	Param ....:         TextLength             <UINT32>                          Number of UTF-16 units of the reported range.
	Param ....:         GlyphOrientationAngle  <DWRITE_GLYPH_ORIENTATION_ANGLE>  A DWRITE_GLYPH_ORIENTATION_ANGLE-typed value that specifies the angle of the glyphs within the text range (pass to IDWriteTextAnalyzer1::GetGlyphOrientationTransform to get the world relative transform).
	Param ....:         AdjustedBidiLevel      <UINT8>                           The adjusted bidi level to be used by the client layout for reordering runs. This will differ from the resolved bidi level retrieved from the source for cases such as Arabic stacked top-to-bottom, where the glyphs are still shaped as RTL, but the runs are TTB along with any CJK or Latin.
	Param ....:         IsSideways             <BOOL>                            Whether the glyphs are rotated on their side, which is the default case for CJK and the case stacked Latin
	Param ....:         IsRightToLeft          <BOOL>                            Whether the script should be shaped as right-to-left. For Arabic stacked top-to-bottom, even when the adjusted bidi level is coerced to an even level, this will still be true.

#ce
; _DWrite_TextAnalysisSink1_SetGlyphOrientation





;#######################################################################################################################################################################
;# IDwriteTextAnalysisSource
;#######################################################################################################################################################################
#cs    IDwriteTextAnalysisSource
	Method ...: GetLocaleName (?)
	Info .....:     Gets the locale name on the range affected by the text analysis.
	Param ....:         TextPosition  <UINT32>  The text position to examine.
	Param ....:         TextLength    <UINT32>  Contains the length of the text being affected by the text analysis up to the next differing locale.
	Param ....:         LocaleName    <WCHAR>   Contains an address of a pointer to an array of characters which receives the locale name from the text affected by the text analysis. The array of characters is null-terminated.

	Method ...: GetNumberSubstitution (?)
	Info .....:     Gets the number substitution from the text range affected by the text analysis.
	Param ....:         TextPosition        <UINT32>                     The starting position from which to report.
	Param ....:         TextLength          <UINT32>                     Contains the length of the text, in characters, remaining in the text range up to the next differing number substitution.
	Param ....:         NumberSubstitution  <IDWriteNumberSubstitution>  Contains an address of a pointer to an object, which was created with IDWriteFactory::CreateNumberSubstitution, that holds the appropriate digits and numeric punctuation for a given locale.

	Method ...: GetParagraphReadingDirection (?)
	Info .....:     Gets the paragraph reading direction.
	Param ....:         This method has no parameters.

	Method ...: GetTextAtPosition (?)
	Info .....:     Gets a block of text starting at the specified text position.
	Param ....:         TextPosition  <UINT32>  The first position of the piece to obtain. All positions are in UTF16 code units, not whole characters, which matters when supplementary characters are used.
	Param ....:         TextString    <WCHAR>   When this method returns, contains an address of the block of text as an array of characters to be retrieved from the text analysis.
	Param ....:         TextLength    <UINT32>  When this method returns, contains the number of UTF16 units of the retrieved chunk. The returned length is not the length of the block, but the length remaining in the block, from the specified position until its end. For example, querying for a position that is 75 positions into a 100-position block would return 25.

	Method ...: GetTextBeforePosition (?)
	Info .....:     Gets a block of text immediately preceding the specified position.
	Param ....:         TextPosition  <UINT32>  The position immediately after the last position of the block of text to obtain.
	Param ....:         TextString    <WCHAR>   When this method returns, contains an address of a pointer to the block of text, as an array of characters from the specified range. The text range will be from "textPosition" to the front of the block.
	Param ....:         TextLength    <UINT32>  Number of UTF16 units of the retrieved block. The length returned is from the specified position to the front of the block.

#ce
; _DWrite_TextAnalysisSource_GetLocaleName
; _DWrite_TextAnalysisSource_GetNumberSubstitution
; _DWrite_TextAnalysisSource_GetParagraphReadingDirection
; _DWrite_TextAnalysisSource_GetTextAtPosition
; _DWrite_TextAnalysisSource_GetTextBeforePosition





;#######################################################################################################################################################################
;# IDwriteTextAnalysisSource1
;#######################################################################################################################################################################
#cs    IDwriteTextAnalysisSource1
	Method ...: GetVerticalGlyphOrientation (?)
	Info .....:     Used by the text analyzer to obtain the desired glyph orientation and resolved bidi level.
	Param ....:         TextPosition      <UINT32>                             The text position.
	Param ....:         TextLength        <UINT32>                             A pointer to the text length.
	Param ....:         GlyphOrientation  <DWRITE_VERTICAL_GLYPH_ORIENTATION>  A DWRITE_VERTICAL_GLYPH_ORIENTATION-typed value that specifies the desired kind of glyph orientation for the text.
	Param ....:         BidiLevel         <UINT8>                              A pointer to the resolved bidi level.

#ce
; _DWrite_TextAnalysisSource1_GetVerticalGlyphOrientation





;#######################################################################################################################################################################
;# IDwriteTextAnalyzer
;#######################################################################################################################################################################
#cs    IDwriteTextAnalyzer
	Method ...: AnalyzeBidi (struct*;uint;uint;struct*)
	Info .....:     Analyzes a text range for script directionality, reading attributes from the source and reporting levels to the sink callback SetBidiLevel.
	Param ....:         AnalysisSource  <IDWriteTextAnalysisSource>  A pointer to a source object to analyze.
	Param ....:         TextPosition    <UINT32>                     The starting text position within the source object.
	Param ....:         TextLength      <UINT32>                     The text length to analyze.
	Param ....:         AnalysisSink    <IDWriteTextAnalysisSink>    A pointer to the sink callback object that receives the text analysis.

	Method ...: AnalyzeLineBreakpoints (struct*;uint;uint;struct*)
	Info .....:     Analyzes a text range for potential breakpoint opportunities, reading attributes from the source and reporting breakpoint opportunities to the sink callback SetLineBreakpoints.
	Param ....:         AnalysisSource  <IDWriteTextAnalysisSource>  A pointer to the source object to analyze.
	Param ....:         TextPosition    <UINT32>                     The starting text position within the source object.
	Param ....:         TextLength      <UINT32>                     The text length to analyze.
	Param ....:         AnalysisSink    <IDWriteTextAnalysisSink>    A pointer to the sink callback object that receives the text analysis.

	Method ...: AnalyzeNumberSubstitution (struct*;uint;uint;struct*)
	Info .....:     Analyzes a text range for spans where number substitution is applicable, reading attributes from the source and reporting substitutable ranges to the sink callback SetNumberSubstitution.
	Param ....:         AnalysisSource  <IDWriteTextAnalysisSource>  The source object to analyze.
	Param ....:         TextPosition    <UINT32>                     The starting position within the source object.
	Param ....:         TextLength      <UINT32>                     The length to analyze.
	Param ....:         AnalysisSink    <IDWriteTextAnalysisSink>    A pointer to the sink callback object that receives the text analysis.

	Method ...: AnalyzeScript (struct*;uint;uint;struct*)
	Info .....:     Analyzes a text range for script boundaries, reading text attributes from the source and reporting the Unicode script ID to the sink callback SetScript.
	Param ....:         AnalysisSource  <IDWriteTextAnalysisSource>  A pointer to the source object to analyze.
	Param ....:         TextPosition    <UINT32>                     The starting text position within the source object.
	Param ....:         TextLength      <UINT32>                     The text length to analyze.
	Param ....:         AnalysisSink    <IDWriteTextAnalysisSink>    A pointer to the sink callback object that receives the text analysis.

	Method ...: GetGdiCompatibleGlyphPlacements (wstr;struct*;struct*;uint;struct*;struct*;uint;struct*;float;float;struct*;bool;bool;bool;struct*;wstr;struct*;struct*;uint;struct*;struct*)
	Info .....:     Place glyphs output from the GetGlyphs method according to the font and the writing system's rendering rules.
	Param ....:         TextString           <WCHAR>                            An array of characters containing the original string from which the glyphs came.
	Param ....:         ClusterMap           <UINT16>                           A pointer to the mapping from character ranges to glyph ranges. This is returned by GetGlyphs.
	Param ....:         TextProps            <DWRITE_SHAPING_TEXT_PROPERTIES>   A pointer to an array of structures that contains shaping properties for each character. This structure is returned by GetGlyphs.
	Param ....:         TextLength           <UINT32>                           The text length of "textString".
	Param ....:         GlyphIndices         <UINT16>                           An array of glyph indices returned by GetGlyphs.
	Param ....:         GlyphProps           <DWRITE_SHAPING_GLYPH_PROPERTIES>  A pointer to an array of structures that contain shaping properties for each glyph returned by GetGlyphs.
	Param ....:         GlyphCount           <UINT32>                           The number of glyphs returned from GetGlyphs.
	Param ....:         FontFace             <IDWriteFontFace>                  A pointer to the font face that is the source for the output glyphs.
	Param ....:         FontEmSize           <FLOAT>                            The logical font size in DIPs.
	Param ....:         PixelsPerDip         <FLOAT>                            The number of physical pixels per DIP.
	Param ....:         Transform            <DWRITE_MATRIX>                    An optional transform applied to the glyphs and their positions. This transform is applied after the scaling specified by the font size and "pixelsPerDip".
	Param ....:         UseGdiNatural        <BOOL>                             When set to FALSE, the metrics are the same as the metrics of GDI aliased text. When set to TRUE, the metrics are the same as the metrics of text measured by GDI using a font created with CLEARTYPE_NATURAL_QUALITY.
	Param ....:         IsSideways           <BOOL>                             A Boolean flag set to TRUE if the text is intended to be drawn vertically.
	Param ....:         IsRightToLeft        <BOOL>                             A Boolean flag set to TRUE for right-to-left text.
	Param ....:         ScriptAnalysis       <DWRITE_SCRIPT_ANALYSIS>           A pointer to a Script analysis result from an AnalyzeScript call.
	Param ....:         LocaleName           <WCHAR>                            An array of characters containing the locale to use when selecting glyphs. For example, the same character may map to different glyphs for ja-jp versus zh-chs. If this is NULL, then the default mapping based on the script is used.
	Param ....:         Features             <DWRITE_TYPOGRAPHIC_FEATURES>      An array of pointers to the sets of typographic features to use in each feature range.
	Param ....:         FeatureRangeLengths  <UINT32>                           The length of each feature range, in characters. The sum of all lengths should be equal to "textLength".
	Param ....:         FeatureRanges        <UINT32>                           The number of feature ranges.
	Param ....:         GlyphAdvances        <FLOAT>                            When this method returns, contains the advance width of each glyph.
	Param ....:         GlyphOffsets         <DWRITE_GLYPH_OFFSET>              When this method returns, contains the offset of the origin of each glyph.

	Method ...: GetGlyphPlacements (wstr;struct*;struct*;uint;struct*;struct*;uint;struct*;float;bool;bool;struct*;wstr;struct*;struct*;uint;struct*;struct*)
	Info .....:     Places glyphs output from the GetGlyphs method according to the font and the writing system's rendering rules.
	Param ....:         TextString           <WCHAR>                            An array of characters containing the original string from which the glyphs came.
	Param ....:         ClusterMap           <UINT16>                           A pointer to the mapping from character ranges to glyph ranges. This is returned by GetGlyphs.
	Param ....:         TextProps            <DWRITE_SHAPING_TEXT_PROPERTIES>   A pointer to an array of structures that contains shaping properties for each character. This structure is returned by GetGlyphs.
	Param ....:         TextLength           <UINT32>                           The text length of "textString".
	Param ....:         GlyphIndices         <UINT16>                           An array of glyph indices returned by GetGlyphs.
	Param ....:         GlyphProps           <DWRITE_SHAPING_GLYPH_PROPERTIES>  A pointer to an array of structures that contain shaping properties for each glyph returned by GetGlyphs.
	Param ....:         GlyphCount           <UINT32>                           The number of glyphs returned from GetGlyphs.
	Param ....:         FontFace             <IDWriteFontFace>                  A pointer to the font face that is the source for the output glyphs.
	Param ....:         FontEmSize           <FLOAT>                            The logical font size in DIPs.
	Param ....:         IsSideways           <BOOL>                             A Boolean flag set to TRUE if the text is intended to be drawn vertically.
	Param ....:         IsRightToLeft        <BOOL>                             A Boolean flag set to TRUE for right-to-left text.
	Param ....:         ScriptAnalysis       <DWRITE_SCRIPT_ANALYSIS>           A pointer to a Script analysis result from an AnalyzeScript call.
	Param ....:         LocaleName           <WCHAR>                            An array of characters containing the locale to use when selecting glyphs. For example, the same character may map to different glyphs for ja-jp versus zh-chs. If this is NULL, the default mapping based on the script is used.
	Param ....:         Features             <DWRITE_TYPOGRAPHIC_FEATURES>      An array of pointers to the sets of typographic features to use in each feature range.
	Param ....:         FeatureRangeLengths  <UINT32>                           The length of each feature range, in characters. The sum of all lengths should be equal to "textLength".
	Param ....:         FeatureRanges        <UINT32>                           The number of feature ranges.
	Param ....:         GlyphAdvances        <FLOAT>                            When this method returns, contains the advance width of each glyph.
	Param ....:         GlyphOffsets         <DWRITE_GLYPH_OFFSET>              When this method returns, contains the offset of the origin of each glyph.

	Method ...: GetGlyphs (wstr;uint;struct*;bool;bool;struct*;wstr;struct*;struct*;uint;uint;struct*;struct*;struct*;struct*;uint*)
	Info .....:     Parses the input text string and maps it to the set of glyphs and associated glyph data according to the font and the writing system's rendering rules.
	Param ....:         TextString           <WCHAR>                            An array of characters to convert to glyphs.
	Param ....:         TextLength           <UINT32>                           The length of "textString".
	Param ....:         FontFace             <IDWriteFontFace>                  The font face that is the source of the output glyphs.
	Param ....:         IsSideways           <BOOL>                             A Boolean flag set to TRUE if the text is intended to be drawn vertically.
	Param ....:         IsRightToLeft        <BOOL>                             A Boolean flag set to TRUE for right-to-left text.
	Param ....:         ScriptAnalysis       <DWRITE_SCRIPT_ANALYSIS>           A pointer to a Script analysis result from an AnalyzeScript call.
	Param ....:         LocaleName           <WCHAR>                            The locale to use when selecting glyphs. For example the same character may map to different glyphs for ja-jp versus zh-chs. If this is NULL, then the default mapping based on the script is used.
	Param ....:         NumberSubstitution   <IDWriteNumberSubstitution>        A pointer to an optional number substitution which selects the appropriate glyphs for digits and related numeric characters, depending on the results obtained from AnalyzeNumberSubstitution. Passing NULL indicates that no substitution is needed and that the digits should receive nominal glyphs.
	Param ....:         Features             <DWRITE_TYPOGRAPHIC_FEATURES>      An array of pointers to the sets of typographic features to use in each feature range.
	Param ....:         FeatureRangeLengths  <UINT32>                           The length of each feature range, in characters. The sum of all lengths should be equal to "textLength".
	Param ....:         FeatureRanges        <UINT32>                           The number of feature ranges.
	Param ....:         MaxGlyphCount        <UINT32>                           The maximum number of glyphs that can be returned.
	Param ....:         ClusterMap           <UINT16>                           When this method returns, contains the mapping from character ranges to glyph ranges.
	Param ....:         TextProps            <DWRITE_SHAPING_TEXT_PROPERTIES>   When this method returns, contains a pointer to an array of structures that contains shaping properties for each character.
	Param ....:         GlyphIndices         <UINT16>                           The output glyph indices.
	Param ....:         GlyphProps           <DWRITE_SHAPING_GLYPH_PROPERTIES>  When this method returns, contains a pointer to an array of structures that contain shaping properties for each output glyph.
	Param ....:         ActualGlyphCount     <UINT32>                           When this method returns, contains the actual number of glyphs returned if the call succeeds.

#ce
; _DWrite_TextAnalyzer_AnalyzeBidi
; _DWrite_TextAnalyzer_AnalyzeLineBreakpoints
; _DWrite_TextAnalyzer_AnalyzeNumberSubstitution
; _DWrite_TextAnalyzer_AnalyzeScript
; _DWrite_TextAnalyzer_GetGdiCompatibleGlyphPlacements
; _DWrite_TextAnalyzer_GetGlyphPlacements
; _DWrite_TextAnalyzer_GetGlyphs





;#######################################################################################################################################################################
;# IDwriteTextAnalyzer1
;#######################################################################################################################################################################
#cs    IDwriteTextAnalyzer1
	Method ...: AnalyzeVerticalGlyphOrientation (?)
	Info .....:     Analyzes a text range for script orientation, reading text and attributes from the source and reporting results to the sink callback SetGlyphOrientation.
	Param ....:         AnalysisSource  <IDWriteTextAnalysisSource1>  Source object to analyze.
	Param ....:         TextPosition    <UINT32>                      Starting position within the source object.
	Param ....:         TextLength      <UINT32>                      Length to analyze.
	Param ....:         AnalysisSink    <IDWriteTextAnalysisSink1>    Length to analyze.

	Method ...: ApplyCharacterSpacing (?)
	Info .....:     Applies spacing between characters, properly adjusting glyph clusters and diacritics.
	Param ....:         LeadingSpacing         <FLOAT>                            The spacing before each character, in reading order.
	Param ....:         TrailingSpacing        <FLOAT>                            The spacing after each character, in reading order.
	Param ....:         MinimumAdvanceWidth    <FLOAT>                            The minimum advance of each character, to prevent characters from becoming too thin or zero-width. This must be zero or greater.
	Param ....:         TextLength             <UINT32>                           The length of the clustermap and original text.
	Param ....:         GlyphCount             <UINT32>                           The number of glyphs.
	Param ....:         ClusterMap             <UINT16>                           Mapping from character ranges to glyph ranges.
	Param ....:         GlyphAdvances          <FLOAT>                            The advance width of each glyph.
	Param ....:         GlyphOffsets           <DWRITE_GLYPH_OFFSET>              The offset of the origin of each glyph.
	Param ....:         GlyphProperties        <DWRITE_SHAPING_GLYPH_PROPERTIES>  Properties of each glyph, from GetGlyphs.
	Param ....:         ModifiedGlyphAdvances  <FLOAT>                            The new advance width of each glyph.
	Param ....:         ModifiedGlyphOffsets   <DWRITE_GLYPH_OFFSET>              The new offset of the origin of each glyph.

	Method ...: GetBaseline (?)
	Info .....:     Retrieves the given baseline from the font.
	Param ....:         FontFace             <IDWriteFontFace>         The font face to read.
	Param ....:         Baseline             <DWRITE_BASELINE>         A DWRITE_BASELINE-typed value that specifies the baseline of interest.
	Param ....:         IsVertical           <BOOL>                    Whether the baseline is vertical or horizontal.
	Param ....:         IsSimulationAllowed  <BOOL>                    Simulate the baseline if it is missing in the font.
	Param ....:         ScriptAnalysis       <DWRITE_SCRIPT_ANALYSIS>  Script analysis result from AnalyzeScript.
	Param ....:         LocaleName           <WCHAR>                   The language of the run.
	Param ....:         BaselineCoordinate   <INT32>                   The baseline coordinate value in design units.
	Param ....:         Exists               <BOOL>                    Whether the returned baseline exists in the font.

	Method ...: GetGlyphOrientationTransform (?)
	Info .....:     Returns 2x3 transform matrix for the respective angle to draw the glyph run.
	Param ....:         GlyphOrientationAngle  <DWRITE_GLYPH_ORIENTATION_ANGLE>  A DWRITE_GLYPH_ORIENTATION_ANGLE-typed value that specifies the angle that was reported into IDWriteTextAnalysisSink1::SetGlyphOrientation.
	Param ....:         IsSideways             <BOOL>                            Whether the run's glyphs are sideways or not.
	Param ....:         Transform              <DWRITE_MATRIX>                   Returned transform.

	Method ...: GetJustificationOpportunities (?)
	Info .....:     Retrieves justification opportunity information for each of the glyphs given the text and shaping glyph properties.
	Param ....:         FontFace                    <IDWriteFontFace>                   May be NULL.
	Param ....:         FontEmSize                  <FLOAT>                             Font em size used for the glyph run.
	Param ....:         ScriptAnalysis              <DWRITE_SCRIPT_ANALYSIS>            Script of the text from the itemizer.
	Param ....:         TextLength                  <UINT32>                            Length of the text.
	Param ....:         GlyphCount                  <UINT32>                            Number of glyphs.
	Param ....:         TextString                  <WCHAR>                             Characters used to produce the glyphs.
	Param ....:         ClusterMap                  <UINT16>                            Clustermap produced from shaping.
	Param ....:         GlyphProperties             <DWRITE_SHAPING_GLYPH_PROPERTIES>   Glyph properties produced from shaping.
	Param ....:         JustificationOpportunities  <DWRITE_JUSTIFICATION_OPPORTUNITY>  A pointer to a DWRITE_JUSTIFICATION_OPPORTUNITY structure that receives info for the allowed justification expansion/compression for each glyph.

	Method ...: GetJustifiedGlyphs (?)
	Info .....:     Fills in new glyphs for complex scripts where justification increased the advances of glyphs, such as Arabic with kashida.
	Param ....:         FontFace                <IDWriteFontFace>                  May be NULL.
	Param ....:         FontEmSize              <FLOAT>                            Font em size used for the glyph run.
	Param ....:         ScriptAnalysis          <DWRITE_SCRIPT_ANALYSIS>           Script of the text from the itemizer.
	Param ....:         TextLength              <UINT32>                           Length of the text.
	Param ....:         GlyphCount              <UINT32>                           Number of glyphs.
	Param ....:         MaxGlyphCount           <UINT32>                           Maximum number of output glyphs allocated by caller.
	Param ....:         ClusterMap              <UINT16>                           Clustermap produced from shaping.
	Param ....:         GlyphIndices            <UINT16>                           Original glyphs produced from shaping.
	Param ....:         GlyphAdvances           <FLOAT>                            Original glyph advances produced from shaping.
	Param ....:         JustifiedGlyphAdvances  <FLOAT>                            Justified glyph advances from IDWriteTextAnalyzer1::JustifyGlyphAdvances.
	Param ....:         JustifiedGlyphOffsets   <DWRITE_GLYPH_OFFSET>              Justified glyph offsets from IDWriteTextAnalyzer1::JustifyGlyphAdvances.
	Param ....:         GlyphProperties         <DWRITE_SHAPING_GLYPH_PROPERTIES>  Properties of each glyph, from IDWriteTextAnalyzer::GetGlyphs.
	Param ....:         ActualGlyphCount        <UINT32>                           The new glyph count written to the modified arrays, or the needed glyph count if the size is not large enough.
	Param ....:         ModifiedClusterMap      <UINT16>                           Updated clustermap.
	Param ....:         ModifiedGlyphIndices    <UINT16>                           Updated glyphs with new glyphs inserted where needed.
	Param ....:         ModifiedGlyphAdvances   <FLOAT>                            Updated glyph advances.
	Param ....:         ModifiedGlyphOffsets    <DWRITE_GLYPH_OFFSET>              Updated glyph offsets.

	Method ...: GetScriptProperties (?)
	Info .....:     Retrieves the properties for a given script.
	Param ....:         ScriptAnalysis    <DWRITE_SCRIPT_ANALYSIS>    The script for a run of text returned from IDWriteTextAnalyzer::AnalyzeScript.
	Param ....:         ScriptProperties  <DWRITE_SCRIPT_PROPERTIES>  A pointer to a DWRITE_SCRIPT_PROPERTIES structure that describes info for the script.

	Method ...: GetTextComplexity (?)
	Info .....:     Determines the complexity of text, and whether you need to call IDWriteTextAnalyzer::GetGlyphs for full script shaping.
	Param ....:         TextString      <WCHAR>            The text to check for complexity. This string may be UTF-16, but any supplementary characters will be considered complex.
	Param ....:         TextLength      <UINT32>           Length of the text to check.
	Param ....:         FontFace        <IDWriteFontFace>  The font face to read.
	Param ....:         IsTextSimple    <BOOL>             If true, the text is simple, and the "glyphIndices" array will already have the nominal glyphs for you. Otherwise, you need to call IDWriteTextAnalyzer::GetGlyphs to properly shape complex scripts and OpenType features.
	Param ....:         TextLengthRead  <UINT32>           The length read of the text run with the same complexity, simple or complex. You may call again from that point onward.
	Param ....:         GlyphIndices    <UINT16>           Optional glyph indices for the text. If the function returned that the text was simple, you already have the glyphs you need. Otherwise the glyph indices are not meaningful, and you need to call IDWriteTextAnalyzer::GetGlyphs for shaping instead.

	Method ...: JustifyGlyphAdvances (?)
	Info .....:     Justifies an array of glyph advances to fit the line width.
	Param ....:         LineWidth                   <FLOAT>                             The line width.
	Param ....:         GlyphCount                  <UINT32>                            The glyph count.
	Param ....:         JustificationOpportunities  <DWRITE_JUSTIFICATION_OPPORTUNITY>  A pointer to a DWRITE_JUSTIFICATION_OPPORTUNITY structure that contains info for the allowed justification expansion/compression for each glyph. Get this info from IDWriteTextAnalyzer1::GetJustificationOpportunities.
	Param ....:         GlyphAdvances               <FLOAT>                             An array of glyph advances.
	Param ....:         GlyphOffsets                <DWRITE_GLYPH_OFFSET>               An array of glyph offsets.
	Param ....:         JustifiedGlyphAdvances      <FLOAT>                             The returned array of justified glyph advances.
	Param ....:         JustifiedGlyphOffsets       <DWRITE_GLYPH_OFFSET>               The returned array of justified glyph offsets.

#ce
; _DWrite_TextAnalyzer1_AnalyzeVerticalGlyphOrientation
; _DWrite_TextAnalyzer1_ApplyCharacterSpacing
; _DWrite_TextAnalyzer1_GetBaseline
; _DWrite_TextAnalyzer1_GetGlyphOrientationTransform
; _DWrite_TextAnalyzer1_GetJustificationOpportunities
; _DWrite_TextAnalyzer1_GetJustifiedGlyphs
; _DWrite_TextAnalyzer1_GetScriptProperties
; _DWrite_TextAnalyzer1_GetTextComplexity
; _DWrite_TextAnalyzer1_JustifyGlyphAdvances





;#######################################################################################################################################################################
;# IDwriteTextAnalyzer2
;#######################################################################################################################################################################
#cs    IDwriteTextAnalyzer2
	Method ...: CheckTypographicFeature (?)
	Info .....:     Checks if a typographic feature is available for a glyph or a set of glyphs.
	Param ....:         ScriptAnalysis  <DWRITE_SCRIPT_ANALYSIS>   The script analysis for the script or font to check.
	Param ....:         LocaleName      <WCHAR>                    The locale name to check.
	Param ....:         Tag             <DWRITE_FONT_FEATURE_TAG>  The font feature tag to check.
	Param ....:         GlyphCount      <UINT32>                   The number of glyphs to check.
	Param ....:         GlyphIndices    <UINT16>                   An array of glyph indices to check.
	Param ....:         FeatureApplies  <UINT8>                    An array of integers that indicate whether or not the font feature applies to each glyph specified.

	Method ...: GetGlyphOrientationTransform (?)
	Info .....:     Returns 2x3 transform matrix for the respective angle to draw the glyph run.
	Param ....:         GlyphOrientationAngle  <DWRITE_GLYPH_ORIENTATION_ANGLE>  A DWRITE_GLYPH_ORIENTATION_ANGLE-typed value that specifies the angle that was reported into IDWriteTextAnalysisSink1::SetGlyphOrientation.
	Param ....:         IsSideways             <BOOL>                            Whether the run's glyphs are sideways or not.
	Param ....:         OriginX                <FLOAT>                           The X value of the baseline origin.
	Param ....:         OriginY                <FLOAT>                           The Y value of the baseline origin.
	Param ....:         Transform              <DWRITE_MATRIX>                   Returned transform.

	Method ...: GetTypographicFeatures (?)
	Info .....:     Returns a complete list of OpenType features available for a script or font. If a feature is partially supported, then this method indicates that it is supported.
	Param ....:         ScriptAnalysis  <DWRITE_SCRIPT_ANALYSIS>   The script analysis for the script or font to check.
	Param ....:         LocaleName      <WCHAR>                    The locale name to check.
	Param ....:         MaxTagCount     <UINT32>                   The maximum number of tags to return.
	Param ....:         ActualTagCount  <UINT32>                   The actual number of tags returned.
	Param ....:         Tags            <DWRITE_FONT_FEATURE_TAG>  An array of OpenType font feature tags.

#ce
; _DWrite_TextAnalyzer2_CheckTypographicFeature
; _DWrite_TextAnalyzer2_GetGlyphOrientationTransform
; _DWrite_TextAnalyzer2_GetTypographicFeatures





;#######################################################################################################################################################################
;# IDwriteTextFormat
;#######################################################################################################################################################################
#cs    IDwriteTextFormat
	Method ...: GetFlowDirection ()
	Info .....:     Gets the direction that text lines flow.
	Param ....:         This method has no parameters.

	Method ...: GetFontCollection (ptr*)
	Info .....:     Gets the current font collection.
	Param ....:         FontCollection  <IDWriteFontCollection>  When this method returns, contains an address of a pointer to the font collection being used for the current text.

	Method ...: GetFontFamilyName (wstr;uint)
	Info .....:     Gets a copy of the font family name.
	Param ....:         FontFamilyName  <WCHAR>   When this method returns, contains a pointer to a character array, which is null-terminated, that receives the current font family name. The buffer allocated for this array should be at least the size, in elements, of "nameSize".
	Param ....:         NameSize        <UINT32>  The size of the "fontFamilyName" character array, in character count, including the terminated NULL character. To find the size of "fontFamilyName", use GetFontFamilyNameLength.

	Method ...: GetFontFamilyNameLength (wstr;uint)
	Info .....:     Gets the length of the font family name.
	Param ....:         This method has no parameters.

	Method ...: GetFontSize ()
	Info .....:     Gets the font size in DIP unites.
	Param ....:         This method has no parameters.

	Method ...: GetFontStretch ()
	Info .....:     Gets the font stretch of the text.
	Param ....:         This method has no parameters.

	Method ...: GetFontStyle ()
	Info .....:     Gets the font style of the text.
	Param ....:         This method has no parameters.

	Method ...: GetFontWeight ()
	Info .....:     Gets the font weight of the text.
	Param ....:         This method has no parameters.

	Method ...: GetIncrementalTabStop ()
	Info .....:     Gets the incremental tab stop position.
	Param ....:         This method has no parameters.

	Method ...: GetLineSpacing (uint*;float*float*)
	Info .....:     Gets the line spacing adjustment set for a multiline text paragraph.
	Param ....:         LineSpacingMethod  <DWRITE_LINE_SPACING_METHOD>  A value that indicates how line height is determined.
	Param ....:         LineSpacing        <FLOAT>                       When this method returns, contains the line height, or distance between one baseline to another.
	Param ....:         Baseline           <FLOAT>                       When this method returns, contains the distance from top of line to baseline. A reasonable ratio to "lineSpacing" is 80 percent.

	Method ...: GetLocaleName ()
	Info .....:     Gets a copy of the locale name.
	Param ....:         LocaleName  <WCHAR>   Contains a character array that receives the current locale name.
	Param ....:         NameSize    <UINT32>  The size of the character array, in character count, including the terminated NULL character. Use GetLocaleNameLength to get the size of the locale name character array.

	Method ...: GetLocaleNameLength ()
	Info .....:     Gets the length of the locale name.
	Param ....:         This method has no parameters.

	Method ...: GetParagraphAlignment ()
	Info .....:     Gets the alignment option of a paragraph which is relative to the top and bottom edges of a layout box.
	Param ....:         This method has no parameters.

	Method ...: GetReadingDirection ()
	Info .....:     Gets the current reading direction for text in a paragraph.
	Param ....:         This method has no parameters.

	Method ...: GetTextAlignment ()
	Info .....:     Gets the alignment option of text relative to the layout box's leading and trailing edge.
	Param ....:         This method has no parameters.

	Method ...: GetTrimming (struct*;ptr*)
	Info .....:     Gets the trimming options for text that overflows the layout box.
	Param ....:         TrimmingOptions  <DWRITE_TRIMMING>      When this method returns, it contains a pointer to a DWRITE_TRIMMING structure that holds the text trimming options for the overflowing text.
	Param ....:         TrimmingSign     <IDWriteInlineObject>  When this method returns, contains an address of a pointer to a trimming omission sign. This parameter may be NULL.

	Method ...: GetWordWrapping ()
	Info .....:     Gets the word wrapping option.
	Param ....:         This method has no parameters.

	Method ...: SetFlowDirection (uint)
	Info .....:     Sets the paragraph flow direction.
	Param ....:         FlowDirection  <DWRITE_FLOW_DIRECTION>  The paragraph flow direction; see DWRITE_FLOW_DIRECTION for more information.

	Method ...: SetIncrementalTabStop (float)
	Info .....:     Sets a fixed distance between two adjacent tab stops.
	Param ....:         IncrementalTabStop  <FLOAT>  The fixed distance between two adjacent tab stops.

	Method ...: SetLineSpacing (uint;float;float)
	Info .....:     Sets the line spacing.
	Param ....:         LineSpacingMethod  <DWRITE_LINE_SPACING_METHOD>  Specifies how line height is being determined; see DWRITE_LINE_SPACING_METHOD for more information.
	Param ....:         LineSpacing        <FLOAT>                       The line height, or distance between one baseline to another.
	Param ....:         Baseline           <FLOAT>                       The distance from top of line to baseline. A reasonable ratio to "lineSpacing" is 80 percent.

	Method ...: SetParagraphAlignment (uint)
	Info .....:     Sets the alignment option of a paragraph relative to the layout box's top and bottom edge.
	Param ....:         ParagraphAlignment  <DWRITE_PARAGRAPH_ALIGNMENT>  The paragraph alignment option being set for a paragraph; see DWRITE_PARAGRAPH_ALIGNMENT for more information.

	Method ...: SetReadingDirection (uint)
	Info .....:     Sets the paragraph reading direction.
	Param ....:         ReadingDirection  <DWRITE_READING_DIRECTION>  The text reading direction (for example, DWRITE_READING_DIRECTION_RIGHT_TO_LEFT for languages, such as Arabic, that read from right to left) for a paragraph.

	Method ...: SetTextAlignment (uint)
	Info .....:     Sets the alignment of text in a paragraph, relative to the leading and trailing edge of a layout box for a IDWriteTextFormat interface.
	Param ....:         TextAlignment  <DWRITE_TEXT_ALIGNMENT>   The text alignment option being set for the paragraph of type DWRITE_TEXT_ALIGNMENT. For more information, see Remarks.

	Method ...: SetTrimming (struct*;ptr*)
	Info .....:     Sets trimming options for text overflowing the layout width.
	Param ....:         TrimmingOptions  <DWRITE_TRIMMING>      Text trimming options.
	Param ....:         TrimmingSign     <IDWriteInlineObject>  Application-defined omission sign. This parameter may be NULL. See IDWriteInlineObject for more information.

	Method ...: SetWordWrapping (uint)
	Info .....:     Sets the word wrapping option.
	Param ....:         WordWrapping  <DWRITE_WORD_WRAPPING>  The word wrapping option being set for a paragraph; see DWRITE_WORD_WRAPPING for more information.

#ce
; _DWrite_TextFormat_GetFlowDirection
; _DWrite_TextFormat_GetFontCollection
; _DWrite_TextFormat_GetFontFamilyName
; _DWrite_TextFormat_GetFontFamilyNameLength
; _DWrite_TextFormat_GetFontSize
; _DWrite_TextFormat_GetFontStretch
; _DWrite_TextFormat_GetFontStyle
; _DWrite_TextFormat_GetFontWeight
; _DWrite_TextFormat_GetIncrementalTabStop
; _DWrite_TextFormat_GetLineSpacing
; _DWrite_TextFormat_GetLocaleName
; _DWrite_TextFormat_GetLocaleNameLength
; _DWrite_TextFormat_GetParagraphAlignment
; _DWrite_TextFormat_GetReadingDirection
; _DWrite_TextFormat_GetTextAlignment
; _DWrite_TextFormat_GetTrimming
; _DWrite_TextFormat_GetWordWrapping
; _DWrite_TextFormat_SetFlowDirection
; _DWrite_TextFormat_SetIncrementalTabStop
; _DWrite_TextFormat_SetLineSpacing
; _DWrite_TextFormat_SetParagraphAlignment
; _DWrite_TextFormat_SetReadingDirection
; _DWrite_TextFormat_SetTextAlignment
; _DWrite_TextFormat_SetTrimming
; _DWrite_TextFormat_SetWordWrapping





;#######################################################################################################################################################################
;# IDwriteTextFormat
;#######################################################################################################################################################################
#cs    IDwriteTextFormat
	Method ...: GetFlowDirection ()
	Info .....:     Gets the direction that text lines flow.
	Param ....:         This method has no parameters.

	Method ...: GetFontCollection (ptr*)
	Info .....:     Gets the current font collection.
	Param ....:         FontCollection  <IDWriteFontCollection>  When this method returns, contains an address of a pointer to the font collection being used for the current text.

	Method ...: GetFontFamilyName (wstr;uint)
	Info .....:     Gets a copy of the font family name.
	Param ....:         FontFamilyName  <WCHAR>   When this method returns, contains a pointer to a character array, which is null-terminated, that receives the current font family name. The buffer allocated for this array should be at least the size, in elements, of "nameSize".
	Param ....:         NameSize        <UINT32>  The size of the "fontFamilyName" character array, in character count, including the terminated NULL character. To find the size of "fontFamilyName", use GetFontFamilyNameLength.

	Method ...: GetFontFamilyNameLength (wstr;uint)
	Info .....:     Gets the length of the font family name.
	Param ....:         This method has no parameters.

	Method ...: GetFontSize ()
	Info .....:     Gets the font size in DIP unites.
	Param ....:         This method has no parameters.

	Method ...: GetFontStretch ()
	Info .....:     Gets the font stretch of the text.
	Param ....:         This method has no parameters.

	Method ...: GetFontStyle ()
	Info .....:     Gets the font style of the text.
	Param ....:         This method has no parameters.

	Method ...: GetFontWeight ()
	Info .....:     Gets the font weight of the text.
	Param ....:         This method has no parameters.

	Method ...: GetIncrementalTabStop ()
	Info .....:     Gets the incremental tab stop position.
	Param ....:         This method has no parameters.

	Method ...: GetLineSpacing (uint*;float*float*)
	Info .....:     Gets the line spacing adjustment set for a multiline text paragraph.
	Param ....:         LineSpacingMethod  <DWRITE_LINE_SPACING_METHOD>  A value that indicates how line height is determined.
	Param ....:         LineSpacing        <FLOAT>                       When this method returns, contains the line height, or distance between one baseline to another.
	Param ....:         Baseline           <FLOAT>                       When this method returns, contains the distance from top of line to baseline. A reasonable ratio to "lineSpacing" is 80 percent.

	Method ...: GetLocaleName ()
	Info .....:     Gets a copy of the locale name.
	Param ....:         LocaleName  <WCHAR>   Contains a character array that receives the current locale name.
	Param ....:         NameSize    <UINT32>  The size of the character array, in character count, including the terminated NULL character. Use GetLocaleNameLength to get the size of the locale name character array.

	Method ...: GetLocaleNameLength ()
	Info .....:     Gets the length of the locale name.
	Param ....:         This method has no parameters.

	Method ...: GetParagraphAlignment ()
	Info .....:     Gets the alignment option of a paragraph which is relative to the top and bottom edges of a layout box.
	Param ....:         This method has no parameters.

	Method ...: GetReadingDirection ()
	Info .....:     Gets the current reading direction for text in a paragraph.
	Param ....:         This method has no parameters.

	Method ...: GetTextAlignment ()
	Info .....:     Gets the alignment option of text relative to the layout box's leading and trailing edge.
	Param ....:         This method has no parameters.

	Method ...: GetTrimming (struct*;ptr*)
	Info .....:     Gets the trimming options for text that overflows the layout box.
	Param ....:         TrimmingOptions  <DWRITE_TRIMMING>      When this method returns, it contains a pointer to a DWRITE_TRIMMING structure that holds the text trimming options for the overflowing text.
	Param ....:         TrimmingSign     <IDWriteInlineObject>  When this method returns, contains an address of a pointer to a trimming omission sign. This parameter may be NULL.

	Method ...: GetWordWrapping ()
	Info .....:     Gets the word wrapping option.
	Param ....:         This method has no parameters.

	Method ...: SetFlowDirection (uint)
	Info .....:     Sets the paragraph flow direction.
	Param ....:         FlowDirection  <DWRITE_FLOW_DIRECTION>  The paragraph flow direction; see DWRITE_FLOW_DIRECTION for more information.

	Method ...: SetIncrementalTabStop (float)
	Info .....:     Sets a fixed distance between two adjacent tab stops.
	Param ....:         IncrementalTabStop  <FLOAT>  The fixed distance between two adjacent tab stops.

	Method ...: SetLineSpacing (uint;float;float)
	Info .....:     Sets the line spacing.
	Param ....:         LineSpacingMethod  <DWRITE_LINE_SPACING_METHOD>  Specifies how line height is being determined; see DWRITE_LINE_SPACING_METHOD for more information.
	Param ....:         LineSpacing        <FLOAT>                       The line height, or distance between one baseline to another.
	Param ....:         Baseline           <FLOAT>                       The distance from top of line to baseline. A reasonable ratio to "lineSpacing" is 80 percent.

	Method ...: SetParagraphAlignment (uint)
	Info .....:     Sets the alignment option of a paragraph relative to the layout box's top and bottom edge.
	Param ....:         ParagraphAlignment  <DWRITE_PARAGRAPH_ALIGNMENT>  The paragraph alignment option being set for a paragraph; see DWRITE_PARAGRAPH_ALIGNMENT for more information.

	Method ...: SetReadingDirection (uint)
	Info .....:     Sets the paragraph reading direction.
	Param ....:         ReadingDirection  <DWRITE_READING_DIRECTION>  The text reading direction (for example, DWRITE_READING_DIRECTION_RIGHT_TO_LEFT for languages, such as Arabic, that read from right to left) for a paragraph.

	Method ...: SetTextAlignment (uint)
	Info .....:     Sets the alignment of text in a paragraph, relative to the leading and trailing edge of a layout box for a IDWriteTextFormat interface.
	Param ....:         TextAlignment  <DWRITE_TEXT_ALIGNMENT>   The text alignment option being set for the paragraph of type DWRITE_TEXT_ALIGNMENT. For more information, see Remarks.

	Method ...: SetTrimming (struct*;ptr*)
	Info .....:     Sets trimming options for text overflowing the layout width.
	Param ....:         TrimmingOptions  <DWRITE_TRIMMING>      Text trimming options.
	Param ....:         TrimmingSign     <IDWriteInlineObject>  Application-defined omission sign. This parameter may be NULL. See IDWriteInlineObject for more information.

	Method ...: SetWordWrapping (uint)
	Info .....:     Sets the word wrapping option.
	Param ....:         WordWrapping  <DWRITE_WORD_WRAPPING>  The word wrapping option being set for a paragraph; see DWRITE_WORD_WRAPPING for more information.

#ce
; _DWrite_TextFormat_GetFlowDirection
; _DWrite_TextFormat_GetFontCollection
; _DWrite_TextFormat_GetFontFamilyName
; _DWrite_TextFormat_GetFontFamilyNameLength
; _DWrite_TextFormat_GetFontSize
; _DWrite_TextFormat_GetFontStretch
; _DWrite_TextFormat_GetFontStyle
; _DWrite_TextFormat_GetFontWeight
; _DWrite_TextFormat_GetIncrementalTabStop
; _DWrite_TextFormat_GetLineSpacing
; _DWrite_TextFormat_GetLocaleName
; _DWrite_TextFormat_GetLocaleNameLength
; _DWrite_TextFormat_GetParagraphAlignment
; _DWrite_TextFormat_GetReadingDirection
; _DWrite_TextFormat_GetTextAlignment
; _DWrite_TextFormat_GetTrimming
; _DWrite_TextFormat_GetWordWrapping
; _DWrite_TextFormat_SetFlowDirection
; _DWrite_TextFormat_SetIncrementalTabStop
; _DWrite_TextFormat_SetLineSpacing
; _DWrite_TextFormat_SetParagraphAlignment
; _DWrite_TextFormat_SetReadingDirection
; _DWrite_TextFormat_SetTextAlignment
; _DWrite_TextFormat_SetTrimming
; _DWrite_TextFormat_SetWordWrapping





;#######################################################################################################################################################################
;# IDwriteTextFormat1
;#######################################################################################################################################################################
#cs    IDwriteTextFormat1
	Method ...: GetFontFallback (?)
	Info .....:     Gets the current fallback. If none was ever set since creating the layout, it will be nullptr.
	Param ....:         FontFallback  <IDWriteFontFallback>  Contains an address of a pointer to the the current font fallback object.

	Method ...: SetFontFallback (?)
	Info .....:     Applies the custom font fallback onto the layout. If none is set, it uses the default system fallback list.
	Param ....:         FontFallback  <IDWriteFontFallback>  The font fallback to apply to the layout.

	Method ...: SetOpticalAlignment (?)
	Info .....:     Sets the optical margin alignment for the text format.
	Param ....:         OpticalAlignment  <DWRITE_OPTICAL_ALIGNMENT>  The optical alignment to set.

	Method ...: SetVerticalGlyphOrientation (?)
	Info .....:     Sets the orientation of a text format.
	Param ....:         GlyphOrientation  <DWRITE_VERTICAL_GLYPH_ORIENTATION>  The orientation to apply to the text format.

	Method ...: SetWrapOnLastLine (?)
	Info .....:     Sets the wrapping mode of the last line.
	Param ....:         IsLastLineWrappingEnabled  <BOOL>  The last line is wrapped by default.

#ce
; _DWrite_TextFormat1_GetFontFallback
; _DWrite_TextFormat1_SetFontFallback
; _DWrite_TextFormat1_SetOpticalAlignment
; _DWrite_TextFormat1_SetVerticalGlyphOrientation
; _DWrite_TextFormat1_SetWrapOnLastLine





;#######################################################################################################################################################################
;# IDwriteTextLayout
;#######################################################################################################################################################################
#cs    IDwriteTextLayout
	Method ...: DetermineMinWidth (float*)
	Info .....:     Determines the minimum possible width the layout can be set to without emergency breaking between the characters of whole words occurring.
	Param ....:         MinWidth  <FLOAT>  Minimum width.

	Method ...: Draw (struct*;struct*;float;float)
	Info .....:     Draws text using the specified client drawing context.
	Param ....:         ClientDrawingContext  <void>                 An application-defined drawing context.
	Param ....:         Renderer              <IDWriteTextRenderer>  Pointer to the set of callback functions used to draw parts of a text string.
	Param ....:         OriginX               <FLOAT>                The x-coordinate of the layout's left side.
	Param ....:         OriginY               <FLOAT>                The y-coordinate of the layout's top side.

	Method ...: GetClusterMetrics (struct*;uint;uint*)
	Info .....:     Retrieves logical properties and measurements of each glyph cluster.
	Param ....:         ClusterMetrics      <DWRITE_CLUSTER_METRICS>  When this method returns, contains metrics, such as line-break or total advance width, for a glyph cluster.
	Param ....:         MaxClusterCount     <UINT32>                  The maximum size of the "clusterMetrics" array.
	Param ....:         ActualClusterCount  <UINT32>                  When this method returns, contains the actual size of the "clusterMetrics" array that is needed.

	Method ...: GetDrawingEffect (uint;ptr*;uint*;uint*)
	Info .....:     Gets the application-defined drawing effect at the specified text position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text whose drawing effect is to be retrieved.
	Param ....:         DrawingEffect    <IUnknown>           When this method returns, contains an address of a pointer to the current application-defined drawing effect. Usually this effect is a foreground brush that is used in glyph drawing.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  Contains the range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the drawing effect.

	Method ...: GetFontCollection (uint;ptr*;uint*;uint*)
	Info .....:     Gets the font collection associated with the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>                 The position of the text to inspect.
	Param ....:         FontCollection   <IDWriteFontCollection>  Contains an address of a pointer to the current font collection.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>      The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the underline.

	Method ...: GetFontFamilyName (uint;uint*;uint*;uint*)
	Info .....:     Copies the font family name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to examine.
	Param ....:         FontFamilyName   <WCHAR>              When this method returns, contains an array of characters that receives the current font family name. You must allocate storage for this parameter.
	Param ....:         NameSize         <UINT32>             The size of the character array in character count including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font family name.

	Method ...: GetFontFamilyNameLength (uint;uint*;uint*;uint*)
	Info .....:     Get the length of the font family name at the current position.
	Param ....:         CurrentPosition  <UINT32>             The current text position.
	Param ....:         NameLength       <UINT32>             When this method returns, contains the size of the character array containing the font family name, in character count, not including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font family.

	Method ...: GetFontSize (uint;float*;uint*;uint*)
	Info .....:     Gets the font em height of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         FontSize         <FLOAT>              When this method returns, contains the size of the font in ems of the text at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font size.

	Method ...: GetFontStretch (uint;uint*;uint*;uint*)
	Info .....:     Gets the font stretch of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>               The position of the text to inspect.
	Param ....:         FontStretch      <DWRITE_FONT_STRETCH>  When this method returns, contains a value which indicates the type of font stretch (also known as width) being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>    The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font stretch.

	Method ...: GetFontStyle (uint;uint*;uint*;uint*)
	Info .....:     Gets the font style (also known as slope) of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         FontStyle        <DWRITE_FONT_STYLE>  When this method returns, contains a value which indicates the type of font style (also known as slope or incline) being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font style.

	Method ...: GetFontWeight (uint;uint*;uint*;uint*)
	Info .....:     Gets the font weight of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>              The position of the text to inspect.
	Param ....:         FontWeight       <DWRITE_FONT_WEIGHT>  When this method returns, contains a value which indicates the type of font weight being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>   The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font weight.

	Method ...: GetInlineObject (uint;ptr*;uint*;uint*)
	Info .....:     Gets the inline object at the specified position.
	Param ....:         CurrentPosition  <UINT32>               The specified text position.
	Param ....:         InlineObject     <IDWriteInlineObject>  Contains the application-defined inline object.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>    The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the inline object.

	Method ...: GetLineMetrics (struct*;uint;uint*)
	Info .....:     Retrieves the information about each individual text line of the text string.
	Param ....:         LineMetrics      <DWRITE_LINE_METRICS>  When this method returns, contains a pointer to an array of structures containing various calculated length values of individual text lines.
	Param ....:         MaxLineCount     <UINT32>               The maximum size of the "lineMetrics" array.
	Param ....:         ActualLineCount  <UINT32>               When this method returns, contains the actual size of the "lineMetrics" array that is needed.

	Method ...: GetLocaleName (uint;uint*;uint*;uint*)
	Info .....:     Gets the locale name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         LocaleName       <WCHAR>              When this method returns, contains the character array receiving the current locale name.
	Param ....:         NameSize         <UINT32>             Size of the character array, in character count, including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the locale name.

	Method ...: GetLocaleNameLength (uint;uint*;uint*;uint*)
	Info .....:     Gets the length of the locale name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         NameLength       <UINT32>             Size of the character array, in character count, not including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the locale name.

	Method ...: GetMaxHeight ()
	Info .....:     Gets the layout maximum height.
	Param ....:         This method has no parameters.

	Method ...: GetMaxWidth ()
	Info .....:     Gets the layout maximum width.
	Param ....:         This method has no parameters.

	Method ...: GetMetrics (struct*)
	Info .....:     Retrieves overall metrics for the formatted string.
	Param ....:         TextMetrics  <DWRITE_TEXT_METRICS>  When this method returns, contains the measured distances of text and associated content after being formatted.

	Method ...: GetOverhangMetrics (struct*)
	Info .....:     Returns the overhangs (in DIPs) of the layout and all objects contained in it, including text glyphs and inline objects.
	Param ....:         Overhangs  <DWRITE_OVERHANG_METRICS>  Overshoots of visible extents (in DIPs) outside the layout.

	Method ...: GetStrikethrough (uint;bool*;uint*;uint*)
	Info .....:     Get the strikethrough presence of the text at the specified position.
	Param ....:         CurrentPosition   <UINT32>             The position of the text to inspect.
	Param ....:         HasStrikethrough  <BOOL>               A Boolean flag that indicates whether strikethrough is present at the position indicated by "currentPosition".
	Param ....:         TextRange         <DWRITE_TEXT_RANGE>  Contains the range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to strikethrough.

	Method ...: GetTypography (uint;ptr*;uint*;uint*)
	Info .....:     Gets the typography setting of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         Typography       <IDWriteTypography>  When this method returns, contains an address of a pointer to the current typography setting.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the typography.

	Method ...: GetUnderline (uint;bool*;uint*;uint*)
	Info .....:     Gets the underline presence of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The current text position.
	Param ....:         HasUnderline     <BOOL>               A Boolean flag that indicates whether underline is present at the position indicated by "currentPosition".
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the underline.

	Method ...: HitTestPoint (float;float;bool*;bool*;struct*)
	Info .....:     The application calls this function passing in a specific pixel location relative to the top-left location of the layout box and obtains the information about the correspondent hit-test metrics of the text string where the hit-test has occurred. When the specified pixel location is outside the text string, the function sets the output value "*isInside" to FALSE.
	Param ....:         PointX          <FLOAT>                    The pixel location X to hit-test, relative to the top-left location of the layout box.
	Param ....:         PointY          <FLOAT>                    The pixel location Y to hit-test, relative to the top-left location of the layout box.
	Param ....:         IsTrailingHit   <BOOL>                     An output flag that indicates whether the hit-test location is at the leading or the trailing side of the character. When the output "*isInside" value is set to FALSE, this value is set according to the output "hitTestMetrics-&gt;textPosition" value to represent the edge closest to the hit-test location.
	Param ....:         IsInside        <BOOL>                     An output flag that indicates whether the hit-test location is inside the text string. When FALSE, the position nearest the text's edge is returned.
	Param ....:         HitTestMetrics  <DWRITE_HIT_TEST_METRICS>  The output geometry fully enclosing the hit-test location. When the output "*isInside" value is set to FALSE, this structure represents the geometry enclosing the edge closest to the hit-test location.

	Method ...: HitTestTextPosition (uint;bool;float*;float*;struct*)
	Info .....:     The application calls this function to get the pixel location relative to the top-left of the layout box given the text position and the logical side of the position. This function is normally used as part of caret positioning of text where the caret is drawn at the location corresponding to the current text editing position. It may also be used as a way to programmatically obtain the geometry of a particular text position in UI automation.
	Param ....:         TextPosition    <UINT32>                   The text position used to get the pixel location.
	Param ....:         IsTrailingHit   <BOOL>                     A Boolean flag that indicates whether the pixel location is of the leading or the trailing side of the specified text position.
	Param ....:         PointX          <FLOAT>                    When this method returns, contains the output pixel location X, relative to the top-left location of the layout box.
	Param ....:         PointY          <FLOAT>                    When this method returns, contains the output pixel location Y, relative to the top-left location of the layout box.
	Param ....:         HitTestMetrics  <DWRITE_HIT_TEST_METRICS>  When this method returns, contains the output geometry fully enclosing the specified text position.

	Method ...: HitTestTextRange (uint;uint;float;float;struct*;uint;uint*)
	Info .....:     The application calls this function to get a set of hit-test metrics corresponding to a range of text positions. One of the main usages is to implement highlight selection of the text string. The function returns E_NOT_SUFFICIENT_BUFFER, which is equivalent to HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER), when the buffer size of hitTestMetrics is too small to hold all the regions calculated by the function. In this situation, the function sets the output value *actualHitTestMetricsCount to the number of geometries calculated. The application is responsible for allocating a new buffer of greater size and calling the function again. A good value to use as an initial value for maxHitTestMetricsCount may be calculated from the following equation: maxHitTestMetricsCount = lineCount * maxBidiReorderingDepth where lineCount is obtained from the value of the output argument *actualLineCount (from the function IDWriteTextLayout::GetLineLengths), and the maxBidiReorderingDepth value from the DWRITE_TEXT_METRICS structure of the output argument *textMetrics (from the function IDWriteFactory:: CreateTextLayout).
	Param ....:         TextPosition               <UINT32>                   The first text position of the specified range.
	Param ....:         TextLength                 <UINT32>                   The number of positions of the specified range.
	Param ....:         OriginX                    <FLOAT>                    The origin pixel location X at the left of the layout box. This offset is added to the hit-test metrics returned.
	Param ....:         OriginY                    <FLOAT>                    The origin pixel location Y at the top of the layout box. This offset is added to the hit-test metrics returned.
	Param ....:         HitTestMetrics             <DWRITE_HIT_TEST_METRICS>  When this method returns, contains a pointer to a buffer of the output geometry fully enclosing the specified position range. The buffer must be at least as large as "maxHitTestMetricsCount".
	Param ....:         MaxHitTestMetricsCount     <UINT32>                   Maximum number of boxes "hitTestMetrics" could hold in its buffer memory.
	Param ....:         ActualHitTestMetricsCount  <UINT32>                   Actual number of geometries "hitTestMetrics" holds in its buffer memory.

	Method ...: SetDrawingEffect (struct*;uint;uint)
	Info .....:     Sets the application-defined drawing effect.
	Param ....:         DrawingEffect  <IUnknown>           Application-defined drawing effects that apply to the range. This data object will be passed back to the application's drawing callbacks for final rendering.
	Param ....:         TextRange      <DWRITE_TEXT_RANGE>  The text range to which this change applies.

	Method ...: SetFontCollection (struct*;struct*)
	Info .....:     Sets the font collection.
	Param ....:         FontCollection  <IDWriteFontCollection>  The font collection to set.
	Param ....:         TextRange       <DWRITE_TEXT_RANGE>      Text range to which this change applies.

	Method ...: SetFontFamilyName (wstr;uint;uint)
	Info .....:     Sets null-terminated font family name for text within a specified text range.
	Param ....:         FontFamilyName  <WCHAR>              The font family name that applies to the entire text string within the range specified by "textRange".
	Param ....:         TextRange       <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetFontSize (float;uint;uint)
	Info .....:     Sets the font size in DIP units for text within a specified text range.
	Param ....:         FontSize   <FLOAT>              The font size in DIP units to be set for text in the range specified by "textRange".
	Param ....:         TextRange  <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetFontStretch (uint;uint;uint)
	Info .....:     Sets the font stretch for text within a specified text range.
	Param ....:         FontStretch  <DWRITE_FONT_STRETCH>  A value which indicates the type of font stretch for text within the range specified by "textRange".
	Param ....:         TextRange    <DWRITE_TEXT_RANGE>    Text range to which this change applies.

	Method ...: SetFontStyle (uint;uint;uint)
	Info .....:     Sets the font style for text within a text range specified by a DWRITE_TEXT_RANGE structure.
	Param ....:         FontStyle  <DWRITE_FONT_STYLE>  The font style to be set for text within a range specified by "textRange".
	Param ....:         TextRange  <DWRITE_TEXT_RANGE>  The text range to which this change applies.

	Method ...: SetInlineObject (struct*;uint;uint)
	Info .....:     Sets the inline object.
	Param ....:         InlineObject  <IDWriteInlineObject>  An application-defined inline object.
	Param ....:         TextRange     <DWRITE_TEXT_RANGE>    Text range to which this change applies.

	Method ...: SetLocaleName (wstr;uint;uint)
	Info .....:     Sets the locale name for text within a specified text range.
	Param ....:         LocaleName  <WCHAR>              A null-terminated locale name string.
	Param ....:         TextRange   <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetMaxHeight (float)
	Info .....:     Sets the layout maximum height.
	Param ....:         MaxHeight  <FLOAT>  A value that indicates the maximum height of the layout box.

	Method ...: SetMaxWidth (float)
	Info .....:     Sets the layout maximum width.
	Param ....:         MaxWidth  <FLOAT>  A value that indicates the maximum width of the layout box.

	Method ...: SetStrikethrough (bool;uint;uint)
	Info .....:     Sets strikethrough for text within a specified text range.
	Param ....:         HasStrikethrough  <BOOL>               A Boolean flag that indicates whether strikethrough takes place in the range specified by "textRange".
	Param ....:         TextRange         <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetTypography (struct*;struct)
	Info .....:     Sets font typography features for text within a specified text range.
	Param ....:         Typography  <IDWriteTypography>  Pointer to font typography settings.
	Param ....:         TextRange   <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetUnderline (bool;uint;uint)
	Info .....:     Sets underlining for text within a specified text range.
	Param ....:         HasUnderline  <BOOL>               A Boolean flag that indicates whether underline takes place within a specified text range.
	Param ....:         TextRange     <DWRITE_TEXT_RANGE>  Text range to which this change applies.

#ce
; _DWrite_TextLayout_DetermineMinWidth
; _DWrite_TextLayout_Draw
; _DWrite_TextLayout_GetClusterMetrics
; _DWrite_TextLayout_GetDrawingEffect
; _DWrite_TextLayout_GetFontCollection
; _DWrite_TextLayout_GetFontFamilyName
; _DWrite_TextLayout_GetFontFamilyNameLength
; _DWrite_TextLayout_GetFontSize
; _DWrite_TextLayout_GetFontStretch
; _DWrite_TextLayout_GetFontStyle
; _DWrite_TextLayout_GetFontWeight
; _DWrite_TextLayout_GetInlineObject
; _DWrite_TextLayout_GetLineMetrics
; _DWrite_TextLayout_GetLocaleName
; _DWrite_TextLayout_GetLocaleNameLength
; _DWrite_TextLayout_GetMaxHeight
; _DWrite_TextLayout_GetMaxWidth
; _DWrite_TextLayout_GetMetrics
; _DWrite_TextLayout_GetOverhangMetrics
; _DWrite_TextLayout_GetStrikethrough
; _DWrite_TextLayout_GetTypography
; _DWrite_TextLayout_GetUnderline
; _DWrite_TextLayout_HitTestPoint
; _DWrite_TextLayout_HitTestTextPosition
; _DWrite_TextLayout_HitTestTextRange
; _DWrite_TextLayout_SetDrawingEffect
; _DWrite_TextLayout_SetFontCollection
; _DWrite_TextLayout_SetFontFamilyName
; _DWrite_TextLayout_SetFontSize
; _DWrite_TextLayout_SetFontStretch
; _DWrite_TextLayout_SetFontStyle
; _DWrite_TextLayout_SetInlineObject
; _DWrite_TextLayout_SetLocaleName
; _DWrite_TextLayout_SetMaxHeight
; _DWrite_TextLayout_SetMaxWidth
; _DWrite_TextLayout_SetStrikethrough



; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_TextLayout_SetTypography
; Description ...:  Sets font typography features for text within a specified text range.
; Syntax ........: _DWrite_TextLayout_SetTypography($oITextLayout, $oTypography[, $iStartPosition = 0[, $iLength = -1]])
; Parameters ....: $oITextLayout   - This IDWriteTextLayout object
;                  $oTypography    - An IDWriteTypography object. Pointer to font typography settings.
;                  $iStartPosition - [optional] The start position of the text range.
;                  $iLength        - [optional] The number positions in the text range.
; Return values .: Success - True
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_TextLayout_SetTypography($oITextLayout, $oTypography, $iStartPosition = 0, $iLength = -1)
	If Not IsObj($oITextLayout) Then Return SetError($DWRITEERR_NOOBJ, 0, False)
	If Not IsObj($oTypography) Then Return SetError($DWRITEERR_NOOBJ, 1, False)

	Local $iHResult = $oITextLayout.SetTypography($oTypography, _DWRITE_TEXT_RANGE($iStartPosition, $iLength))
	If $iHResult Then Return SetError($iHResult, 2, False)

	Return True
EndFunc



; _DWrite_TextLayout_SetUnderline





;#######################################################################################################################################################################
;# IDwriteTextLayout
;#######################################################################################################################################################################
#cs    IDwriteTextLayout
	Method ...: DetermineMinWidth (float*)
	Info .....:     Determines the minimum possible width the layout can be set to without emergency breaking between the characters of whole words occurring.
	Param ....:         MinWidth  <FLOAT>  Minimum width.

	Method ...: Draw (struct*;struct*;float;float)
	Info .....:     Draws text using the specified client drawing context.
	Param ....:         ClientDrawingContext  <void>                 An application-defined drawing context.
	Param ....:         Renderer              <IDWriteTextRenderer>  Pointer to the set of callback functions used to draw parts of a text string.
	Param ....:         OriginX               <FLOAT>                The x-coordinate of the layout's left side.
	Param ....:         OriginY               <FLOAT>                The y-coordinate of the layout's top side.

	Method ...: GetClusterMetrics (struct*;uint;uint*)
	Info .....:     Retrieves logical properties and measurements of each glyph cluster.
	Param ....:         ClusterMetrics      <DWRITE_CLUSTER_METRICS>  When this method returns, contains metrics, such as line-break or total advance width, for a glyph cluster.
	Param ....:         MaxClusterCount     <UINT32>                  The maximum size of the "clusterMetrics" array.
	Param ....:         ActualClusterCount  <UINT32>                  When this method returns, contains the actual size of the "clusterMetrics" array that is needed.

	Method ...: GetDrawingEffect (uint;ptr*;uint*;uint*)
	Info .....:     Gets the application-defined drawing effect at the specified text position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text whose drawing effect is to be retrieved.
	Param ....:         DrawingEffect    <IUnknown>           When this method returns, contains an address of a pointer to the current application-defined drawing effect. Usually this effect is a foreground brush that is used in glyph drawing.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  Contains the range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the drawing effect.

	Method ...: GetFontCollection (uint;ptr*;uint*;uint*)
	Info .....:     Gets the font collection associated with the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>                 The position of the text to inspect.
	Param ....:         FontCollection   <IDWriteFontCollection>  Contains an address of a pointer to the current font collection.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>      The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the underline.

	Method ...: GetFontFamilyName (uint;uint*;uint*;uint*)
	Info .....:     Copies the font family name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to examine.
	Param ....:         FontFamilyName   <WCHAR>              When this method returns, contains an array of characters that receives the current font family name. You must allocate storage for this parameter.
	Param ....:         NameSize         <UINT32>             The size of the character array in character count including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font family name.

	Method ...: GetFontFamilyNameLength (uint;uint*;uint*;uint*)
	Info .....:     Get the length of the font family name at the current position.
	Param ....:         CurrentPosition  <UINT32>             The current text position.
	Param ....:         NameLength       <UINT32>             When this method returns, contains the size of the character array containing the font family name, in character count, not including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font family.

	Method ...: GetFontSize (uint;float*;uint*;uint*)
	Info .....:     Gets the font em height of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         FontSize         <FLOAT>              When this method returns, contains the size of the font in ems of the text at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font size.

	Method ...: GetFontStretch (uint;uint*;uint*;uint*)
	Info .....:     Gets the font stretch of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>               The position of the text to inspect.
	Param ....:         FontStretch      <DWRITE_FONT_STRETCH>  When this method returns, contains a value which indicates the type of font stretch (also known as width) being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>    The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font stretch.

	Method ...: GetFontStyle (uint;uint*;uint*;uint*)
	Info .....:     Gets the font style (also known as slope) of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         FontStyle        <DWRITE_FONT_STYLE>  When this method returns, contains a value which indicates the type of font style (also known as slope or incline) being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font style.

	Method ...: GetFontWeight (uint;uint*;uint*;uint*)
	Info .....:     Gets the font weight of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>              The position of the text to inspect.
	Param ....:         FontWeight       <DWRITE_FONT_WEIGHT>  When this method returns, contains a value which indicates the type of font weight being applied at the specified position.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>   The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the font weight.

	Method ...: GetInlineObject (uint;ptr*;uint*;uint*)
	Info .....:     Gets the inline object at the specified position.
	Param ....:         CurrentPosition  <UINT32>               The specified text position.
	Param ....:         InlineObject     <IDWriteInlineObject>  Contains the application-defined inline object.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>    The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the inline object.

	Method ...: GetLineMetrics (struct*;uint;uint*)
	Info .....:     Retrieves the information about each individual text line of the text string.
	Param ....:         LineMetrics      <DWRITE_LINE_METRICS>  When this method returns, contains a pointer to an array of structures containing various calculated length values of individual text lines.
	Param ....:         MaxLineCount     <UINT32>               The maximum size of the "lineMetrics" array.
	Param ....:         ActualLineCount  <UINT32>               When this method returns, contains the actual size of the "lineMetrics" array that is needed.

	Method ...: GetLocaleName (uint;uint*;uint*;uint*)
	Info .....:     Gets the locale name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         LocaleName       <WCHAR>              When this method returns, contains the character array receiving the current locale name.
	Param ....:         NameSize         <UINT32>             Size of the character array, in character count, including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the locale name.

	Method ...: GetLocaleNameLength (uint;uint*;uint*;uint*)
	Info .....:     Gets the length of the locale name of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         NameLength       <UINT32>             Size of the character array, in character count, not including the terminated NULL character.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the locale name.

	Method ...: GetMaxHeight ()
	Info .....:     Gets the layout maximum height.
	Param ....:         This method has no parameters.

	Method ...: GetMaxWidth ()
	Info .....:     Gets the layout maximum width.
	Param ....:         This method has no parameters.

	Method ...: GetMetrics (struct*)
	Info .....:     Retrieves overall metrics for the formatted string.
	Param ....:         TextMetrics  <DWRITE_TEXT_METRICS>  When this method returns, contains the measured distances of text and associated content after being formatted.

	Method ...: GetOverhangMetrics (struct*)
	Info .....:     Returns the overhangs (in DIPs) of the layout and all objects contained in it, including text glyphs and inline objects.
	Param ....:         Overhangs  <DWRITE_OVERHANG_METRICS>  Overshoots of visible extents (in DIPs) outside the layout.

	Method ...: GetStrikethrough (uint;bool*;uint*;uint*)
	Info .....:     Get the strikethrough presence of the text at the specified position.
	Param ....:         CurrentPosition   <UINT32>             The position of the text to inspect.
	Param ....:         HasStrikethrough  <BOOL>               A Boolean flag that indicates whether strikethrough is present at the position indicated by "currentPosition".
	Param ....:         TextRange         <DWRITE_TEXT_RANGE>  Contains the range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to strikethrough.

	Method ...: GetTypography (uint;ptr*;uint*;uint*)
	Info .....:     Gets the typography setting of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The position of the text to inspect.
	Param ....:         Typography       <IDWriteTypography>  When this method returns, contains an address of a pointer to the current typography setting.
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the typography.

	Method ...: GetUnderline (uint;bool*;uint*;uint*)
	Info .....:     Gets the underline presence of the text at the specified position.
	Param ....:         CurrentPosition  <UINT32>             The current text position.
	Param ....:         HasUnderline     <BOOL>               A Boolean flag that indicates whether underline is present at the position indicated by "currentPosition".
	Param ....:         TextRange        <DWRITE_TEXT_RANGE>  The range of text that has the same formatting as the text at the position specified by "currentPosition". This means the run has the exact formatting as the position specified, including but not limited to the underline.

	Method ...: HitTestPoint (float;float;bool*;bool*;struct*)
	Info .....:     The application calls this function passing in a specific pixel location relative to the top-left location of the layout box and obtains the information about the correspondent hit-test metrics of the text string where the hit-test has occurred. When the specified pixel location is outside the text string, the function sets the output value "*isInside" to FALSE.
	Param ....:         PointX          <FLOAT>                    The pixel location X to hit-test, relative to the top-left location of the layout box.
	Param ....:         PointY          <FLOAT>                    The pixel location Y to hit-test, relative to the top-left location of the layout box.
	Param ....:         IsTrailingHit   <BOOL>                     An output flag that indicates whether the hit-test location is at the leading or the trailing side of the character. When the output "*isInside" value is set to FALSE, this value is set according to the output "hitTestMetrics-&gt;textPosition" value to represent the edge closest to the hit-test location.
	Param ....:         IsInside        <BOOL>                     An output flag that indicates whether the hit-test location is inside the text string. When FALSE, the position nearest the text's edge is returned.
	Param ....:         HitTestMetrics  <DWRITE_HIT_TEST_METRICS>  The output geometry fully enclosing the hit-test location. When the output "*isInside" value is set to FALSE, this structure represents the geometry enclosing the edge closest to the hit-test location.

	Method ...: HitTestTextPosition (uint;bool;float*;float*;struct*)
	Info .....:     The application calls this function to get the pixel location relative to the top-left of the layout box given the text position and the logical side of the position. This function is normally used as part of caret positioning of text where the caret is drawn at the location corresponding to the current text editing position. It may also be used as a way to programmatically obtain the geometry of a particular text position in UI automation.
	Param ....:         TextPosition    <UINT32>                   The text position used to get the pixel location.
	Param ....:         IsTrailingHit   <BOOL>                     A Boolean flag that indicates whether the pixel location is of the leading or the trailing side of the specified text position.
	Param ....:         PointX          <FLOAT>                    When this method returns, contains the output pixel location X, relative to the top-left location of the layout box.
	Param ....:         PointY          <FLOAT>                    When this method returns, contains the output pixel location Y, relative to the top-left location of the layout box.
	Param ....:         HitTestMetrics  <DWRITE_HIT_TEST_METRICS>  When this method returns, contains the output geometry fully enclosing the specified text position.

	Method ...: HitTestTextRange (uint;uint;float;float;struct*;uint;uint*)
	Info .....:     The application calls this function to get a set of hit-test metrics corresponding to a range of text positions. One of the main usages is to implement highlight selection of the text string. The function returns E_NOT_SUFFICIENT_BUFFER, which is equivalent to HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER), when the buffer size of hitTestMetrics is too small to hold all the regions calculated by the function. In this situation, the function sets the output value *actualHitTestMetricsCount to the number of geometries calculated. The application is responsible for allocating a new buffer of greater size and calling the function again. A good value to use as an initial value for maxHitTestMetricsCount may be calculated from the following equation: maxHitTestMetricsCount = lineCount * maxBidiReorderingDepth where lineCount is obtained from the value of the output argument *actualLineCount (from the function IDWriteTextLayout::GetLineLengths), and the maxBidiReorderingDepth value from the DWRITE_TEXT_METRICS structure of the output argument *textMetrics (from the function IDWriteFactory:: CreateTextLayout).
	Param ....:         TextPosition               <UINT32>                   The first text position of the specified range.
	Param ....:         TextLength                 <UINT32>                   The number of positions of the specified range.
	Param ....:         OriginX                    <FLOAT>                    The origin pixel location X at the left of the layout box. This offset is added to the hit-test metrics returned.
	Param ....:         OriginY                    <FLOAT>                    The origin pixel location Y at the top of the layout box. This offset is added to the hit-test metrics returned.
	Param ....:         HitTestMetrics             <DWRITE_HIT_TEST_METRICS>  When this method returns, contains a pointer to a buffer of the output geometry fully enclosing the specified position range. The buffer must be at least as large as "maxHitTestMetricsCount".
	Param ....:         MaxHitTestMetricsCount     <UINT32>                   Maximum number of boxes "hitTestMetrics" could hold in its buffer memory.
	Param ....:         ActualHitTestMetricsCount  <UINT32>                   Actual number of geometries "hitTestMetrics" holds in its buffer memory.

	Method ...: SetDrawingEffect (struct*;uint;uint)
	Info .....:     Sets the application-defined drawing effect.
	Param ....:         DrawingEffect  <IUnknown>           Application-defined drawing effects that apply to the range. This data object will be passed back to the application's drawing callbacks for final rendering.
	Param ....:         TextRange      <DWRITE_TEXT_RANGE>  The text range to which this change applies.

	Method ...: SetFontCollection (struct*;struct*)
	Info .....:     Sets the font collection.
	Param ....:         FontCollection  <IDWriteFontCollection>  The font collection to set.
	Param ....:         TextRange       <DWRITE_TEXT_RANGE>      Text range to which this change applies.

	Method ...: SetFontFamilyName (wstr;uint;uint)
	Info .....:     Sets null-terminated font family name for text within a specified text range.
	Param ....:         FontFamilyName  <WCHAR>              The font family name that applies to the entire text string within the range specified by "textRange".
	Param ....:         TextRange       <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetFontSize (float;uint;uint)
	Info .....:     Sets the font size in DIP units for text within a specified text range.
	Param ....:         FontSize   <FLOAT>              The font size in DIP units to be set for text in the range specified by "textRange".
	Param ....:         TextRange  <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetFontStretch (uint;uint;uint)
	Info .....:     Sets the font stretch for text within a specified text range.
	Param ....:         FontStretch  <DWRITE_FONT_STRETCH>  A value which indicates the type of font stretch for text within the range specified by "textRange".
	Param ....:         TextRange    <DWRITE_TEXT_RANGE>    Text range to which this change applies.

	Method ...: SetFontStyle (uint;uint;uint)
	Info .....:     Sets the font style for text within a text range specified by a DWRITE_TEXT_RANGE structure.
	Param ....:         FontStyle  <DWRITE_FONT_STYLE>  The font style to be set for text within a range specified by "textRange".
	Param ....:         TextRange  <DWRITE_TEXT_RANGE>  The text range to which this change applies.

	Method ...: SetInlineObject (struct*;uint;uint)
	Info .....:     Sets the inline object.
	Param ....:         InlineObject  <IDWriteInlineObject>  An application-defined inline object.
	Param ....:         TextRange     <DWRITE_TEXT_RANGE>    Text range to which this change applies.

	Method ...: SetLocaleName (wstr;uint;uint)
	Info .....:     Sets the locale name for text within a specified text range.
	Param ....:         LocaleName  <WCHAR>              A null-terminated locale name string.
	Param ....:         TextRange   <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetMaxHeight (float)
	Info .....:     Sets the layout maximum height.
	Param ....:         MaxHeight  <FLOAT>  A value that indicates the maximum height of the layout box.

	Method ...: SetMaxWidth (float)
	Info .....:     Sets the layout maximum width.
	Param ....:         MaxWidth  <FLOAT>  A value that indicates the maximum width of the layout box.

	Method ...: SetStrikethrough (bool;uint;uint)
	Info .....:     Sets strikethrough for text within a specified text range.
	Param ....:         HasStrikethrough  <BOOL>               A Boolean flag that indicates whether strikethrough takes place in the range specified by "textRange".
	Param ....:         TextRange         <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetTypography (struct*;struct)
	Info .....:     Sets font typography features for text within a specified text range.
	Param ....:         Typography  <IDWriteTypography>  Pointer to font typography settings.
	Param ....:         TextRange   <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetUnderline (bool;uint;uint)
	Info .....:     Sets underlining for text within a specified text range.
	Param ....:         HasUnderline  <BOOL>               A Boolean flag that indicates whether underline takes place within a specified text range.
	Param ....:         TextRange     <DWRITE_TEXT_RANGE>  Text range to which this change applies.

#ce
; _DWrite_TextLayout_DetermineMinWidth
; _DWrite_TextLayout_Draw
; _DWrite_TextLayout_GetClusterMetrics
; _DWrite_TextLayout_GetDrawingEffect
; _DWrite_TextLayout_GetFontCollection
; _DWrite_TextLayout_GetFontFamilyName
; _DWrite_TextLayout_GetFontFamilyNameLength
; _DWrite_TextLayout_GetFontSize
; _DWrite_TextLayout_GetFontStretch
; _DWrite_TextLayout_GetFontStyle
; _DWrite_TextLayout_GetFontWeight
; _DWrite_TextLayout_GetInlineObject
; _DWrite_TextLayout_GetLineMetrics
; _DWrite_TextLayout_GetLocaleName
; _DWrite_TextLayout_GetLocaleNameLength
; _DWrite_TextLayout_GetMaxHeight
; _DWrite_TextLayout_GetMaxWidth
; _DWrite_TextLayout_GetMetrics
; _DWrite_TextLayout_GetOverhangMetrics
; _DWrite_TextLayout_GetStrikethrough
; _DWrite_TextLayout_GetTypography
; _DWrite_TextLayout_GetUnderline
; _DWrite_TextLayout_HitTestPoint
; _DWrite_TextLayout_HitTestTextPosition
; _DWrite_TextLayout_HitTestTextRange
; _DWrite_TextLayout_SetDrawingEffect
; _DWrite_TextLayout_SetFontCollection
; _DWrite_TextLayout_SetFontFamilyName
; _DWrite_TextLayout_SetFontSize
; _DWrite_TextLayout_SetFontStretch
; _DWrite_TextLayout_SetFontStyle
; _DWrite_TextLayout_SetInlineObject
; _DWrite_TextLayout_SetLocaleName
; _DWrite_TextLayout_SetMaxHeight
; _DWrite_TextLayout_SetMaxWidth
; _DWrite_TextLayout_SetStrikethrough
; _DWrite_TextLayout_SetTypography
; _DWrite_TextLayout_SetUnderline





;#######################################################################################################################################################################
;# IDwriteTextLayout1
;#######################################################################################################################################################################
#cs    IDwriteTextLayout1
	Method ...: GetCharacterSpacing (?)
	Info .....:     Gets the spacing between characters.
	Param ....:         CurrentPosition      <UINT32>     The current text position.
	Param ....:         LeadingSpacing       <FLOAT>      The spacing before each character, in reading order.
	Param ....:         TrailingSpacing      <FLOAT>      The spacing after each character, in reading order.
	Param ....:         MinimumAdvanceWidth  <FLOAT>      The minimum advance of each character, to prevent characters from becoming too thin or zero-width. This must be zero or greater.
	Param ....:         TextRange            <textRange>  The position range of the current format.

	Method ...: GetPairKerning (?)
	Info .....:     Gets whether or not pair-kerning is enabled at given position.
	Param ....:         CurrentPosition       <UINT32>     The current text position.
	Param ....:         IsPairKerningEnabled  <BOOL>       The flag that indicates whether text is pair-kerned.
	Param ....:         TextRange             <textRange>  The position range of the current format.

	Method ...: SetCharacterSpacing (?)
	Info .....:     Sets the spacing between characters.
	Param ....:         LeadingSpacing       <FLOAT>              The spacing before each character, in reading order.
	Param ....:         TrailingSpacing      <FLOAT>              The spacing after each character, in reading order.
	Param ....:         MinimumAdvanceWidth  <FLOAT>              The minimum advance of each character, to prevent characters from becoming too thin or zero-width. This must be zero or greater.
	Param ....:         TextRange            <DWRITE_TEXT_RANGE>  Text range to which this change applies.

	Method ...: SetPairKerning (?)
	Info .....:     Enables or disables pair-kerning on a given text range.
	Param ....:         IsPairKerningEnabled  <BOOL>               The flag that indicates whether text is pair-kerned.
	Param ....:         TextRange             <DWRITE_TEXT_RANGE>  The text range to which the change applies.

#ce
; _DWrite_TextLayout1_GetCharacterSpacing
; _DWrite_TextLayout1_GetPairKerning
; _DWrite_TextLayout1_SetCharacterSpacing
; _DWrite_TextLayout1_SetPairKerning





;#######################################################################################################################################################################
;# IDwriteTextLayout2
;#######################################################################################################################################################################
#cs    IDwriteTextLayout2
	Method ...: GetFontFallback (?)
	Info .....:     Get the current font fallback object.
	Param ....:         FontFallback  <IDWriteFontFallback>  The current font fallback object.

	Method ...: GetLastLineWrapping (?)
	Info .....:     Get whether or not the last word on the last line is wrapped.
	Param ....:         This method has no parameters.

	Method ...: GetMetrics (?)
	Info .....:     Retrieves overall metrics for the formatted string.
	Param ....:         TextMetrics  <DWRITE_TEXT_METRICS1>  When this method returns, contains the measured distances of text and associated content after being formatted.

	Method ...: GetOpticalAlignment (?)
	Info .....:     Get how the glyphs align to the edges the margin.
	Param ....:         This method has no parameters.

	Method ...: GetVerticalGlyphOrientation (?)
	Info .....:     Get the preferred orientation of glyphs when using a vertical reading direction.
	Param ....:         This method has no parameters.

	Method ...: SetFontFallback (?)
	Info .....:     Apply a custom font fallback onto layout. If none is specified, the layout uses the system fallback list.
	Param ....:         FontFallback  <IDWriteFontFallback>  Custom font fallback created from IDWriteFontFallbackBuilder::CreateFontFallback or IDWriteFactory2::GetSystemFontFallback.

	Method ...: SetLastLineWrapping (?)
	Info .....:     Set whether or not the last word on the last line is wrapped.
	Param ....:         IsLastLineWrappingEnabled  <BOOL>  Line wrapping option.

	Method ...: SetOpticalAlignment (?)
	Info .....:     Set how the glyphs align to the edges the margin. Default behavior is to align glyphs using their default glyphs metrics, which include side bearings.
	Param ....:         OpticalAlignment  <DWRITE_OPTICAL_ALIGNMENT>  Optical alignment option.

	Method ...: SetVerticalGlyphOrientation (?)
	Info .....:     Set the preferred orientation of glyphs when using a vertical reading direction.
	Param ....:         GlyphOrientation  <DWRITE_VERTICAL_GLYPH_ORIENTATION>  Preferred glyph orientation.

#ce
; _DWrite_TextLayout2_GetFontFallback
; _DWrite_TextLayout2_GetLastLineWrapping
; _DWrite_TextLayout2_GetMetrics
; _DWrite_TextLayout2_GetOpticalAlignment
; _DWrite_TextLayout2_GetVerticalGlyphOrientation
; _DWrite_TextLayout2_SetFontFallback
; _DWrite_TextLayout2_SetLastLineWrapping
; _DWrite_TextLayout2_SetOpticalAlignment
; _DWrite_TextLayout2_SetVerticalGlyphOrientation





;#######################################################################################################################################################################
;# IDwriteTextRenderer
;#######################################################################################################################################################################
#cs    IDwriteTextRenderer
	Method ...: DrawGlyphRun (struct*;float;float;uint;struct*;struct*;struct*)
	Info .....:     IDWriteTextLayout:: Draw calls this function to instruct the client to render a run of glyphs.
	Param ....:         ClientDrawingContext  <void>                          The application-defined drawing context passed to IDWriteTextLayout::Draw.
	Param ....:         BaselineOriginX       <FLOAT>                         The pixel location (X-coordinate) at the baseline origin of the glyph run.
	Param ....:         BaselineOriginY       <FLOAT>                         The pixel location (Y-coordinate) at the baseline origin of the glyph run.
	Param ....:         MeasuringMode         <DWRITE_MEASURING_MODE>          The measuring method for glyphs in the run, used with the other properties to determine the rendering mode.
	Param ....:         GlyphRun              <DWRITE_GLYPH_RUN>              Pointer to the glyph run instance to render.
	Param ....:         GlyphRunDescription   <DWRITE_GLYPH_RUN_DESCRIPTION>  A pointer to the glyph run description instance which contains properties of the characters associated with this run.
	Param ....:         ClientDrawingEffect   <IUnknown>                      Application-defined drawing effects for the glyphs to render. Usually this argument represents effects such as the foreground brush filling the interior of text.

	Method ...: DrawInlineObject (struct*;float;float;struct*;bool;bool;struct*)
	Info .....:     IDWriteTextLayout:: Draw calls this application callback when it needs to draw an inline object.
	Param ....:         ClientDrawingContext  <void>                 The application-defined drawing context passed to IDWriteTextLayout:: Draw.
	Param ....:         OriginX               <FLOAT>                X-coordinate at the top-left corner of the inline object.
	Param ....:         OriginY               <FLOAT>                Y-coordinate at the top-left corner of the inline object.
	Param ....:         InlineObject          <IDWriteInlineObject>  The application-defined inline object set using IDWriteTextFormat:: SetInlineObject.
	Param ....:         IsSideways            <BOOL>                 A Boolean flag that indicates whether the object's baseline runs alongside the baseline axis of the line.
	Param ....:         IsRightToLeft         <BOOL>                 A Boolean flag that indicates whether the object is in a right-to-left context, hinting that the drawing may want to mirror the normal image.
	Param ....:         ClientDrawingEffect   <IUnknown>             Application-defined drawing effects for the glyphs to render. Usually this argument represents effects such as the foreground brush filling the interior of a line.

	Method ...: DrawStrikethrough (struct*;float;float;struct*;struct*)
	Info .....:     IDWriteTextLayout:: Draw calls this function to instruct the client to draw a strikethrough.
	Param ....:         ClientDrawingContext  <void>                  The application-defined drawing context passed to IDWriteTextLayout:: Draw.
	Param ....:         BaselineOriginX       <FLOAT>                 The pixel location (X-coordinate) at the baseline origin of the run where strikethrough applies.
	Param ....:         BaselineOriginY       <FLOAT>                 The pixel location (Y-coordinate) at the baseline origin of the run where strikethrough applies.
	Param ....:         Strikethrough         <DWRITE_STRIKETHROUGH>  Pointer to a structure containing strikethrough logical information.
	Param ....:         ClientDrawingEffect   <IUnknown>              Application-defined effect to apply to the strikethrough. Usually this argument represents effects such as the foreground brush filling the interior of a line.

	Method ...: DrawUnderline (struct*;float;float;struct*;struct*)
	Info .....:     IDWriteTextLayout:: Draw calls this function to instruct the client to draw an underline.
	Param ....:         ClientDrawingContext  <void>              The application-defined drawing context passed to IDWriteTextLayout:: Draw.
	Param ....:         BaselineOriginX       <FLOAT>             The pixel location (X-coordinate) at the baseline origin of the run where underline applies.
	Param ....:         BaselineOriginY       <FLOAT>             The pixel location (Y-coordinate) at the baseline origin of the run where underline applies.
	Param ....:         Underline             <DWRITE_UNDERLINE>  Pointer to a structure containing underline logical information.
	Param ....:         ClientDrawingEffect   <IUnknown>           Application-defined effect to apply to the underline. Usually this argument represents effects such as the foreground brush filling the interior of a line.

#ce
; _DWrite_TextRenderer_DrawGlyphRun
; _DWrite_TextRenderer_DrawInlineObject
; _DWrite_TextRenderer_DrawStrikethrough
; _DWrite_TextRenderer_DrawUnderline





;#######################################################################################################################################################################
;# IDwriteTextRenderer1
;#######################################################################################################################################################################
#cs    IDwriteTextRenderer1
	Method ...: SetCurrentOrientation (?)
	Info .....:     Takes in a DWRITE_GLYPH_ORIENTATION_ANGLE and sets that explicit orientation on the renderer.
	Param ....:         ClientDrawingContext  <void>                            The current drawing context.
	Param ....:         Orientation           <DWRITE_GLYPH_ORIENTATION_ANGLE>  The explicit drawing context to set on the renderer.

#ce
; _DWrite_TextRenderer1_SetCurrentOrientation





;#######################################################################################################################################################################
;# IDwriteTypography
;#######################################################################################################################################################################
#cs    IDwriteTypography
	Method ...: AddFontFeature (struct)
	Info .....:     Adds an OpenType font feature.
	Param ....:         FontFeature  <DWRITE_FONT_FEATURE>  A structure that contains the OpenType name identifier and the execution parameter for the font feature being added.

	Method ...: GetFontFeature ()
	Info .....:     Gets the font feature at the specified index.
	Param ....:         FontFeatureIndex  <UINT32>               The zero-based index of the font feature to retrieve.
	Param ....:         FontFeature       <DWRITE_FONT_FEATURE>  When this method returns, contains the font feature which is at the specified index.

	Method ...: GetFontFeatureCount ()
	Info .....:     Gets the number of OpenType font features for the current font.
	Param ....:         This method has no parameters.

#ce

; #FUNCTION# ====================================================================================================================
; Name ..........: _DWrite_Typography_AddFontFeature
; Description ...:  Adds an OpenType font feature.
; Syntax ........: _DWrite_Typography_AddFontFeature($oITypography, $iNameTag, $iParameter)
; Parameters ....: $oITypography - This IDWriteTypography object
;                  $iNameTag     - A $DWRITE_FONT_FEATURE_TAG enumeraion. The feature OpenType name identifier.
;                  $iParameter   - The execution parameter of the feature.
; Return values .: Success - True
;                  Failure - False
;                              | @error contains a non zero hresult value specifying the error code
;                              | @extended contains an index of the error position within the function
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _DWrite_Typography_AddFontFeature($oITypography, $iNameTag, $iParameter)
	If Not IsObj($oITypography) Then Return SetError($DWRITEERR_NOOBJ, 0, False)

	Local $iHResult = $oITypography.AddFontFeature(_DWRITE_FONT_FEATURE($iNameTag, $iParameter))
	If $iHResult Then Return SetError($iHResult, 1, False)

	Return True
EndFunc





; _DWrite_Typography_GetFontFeature
; _DWrite_Typography_GetFontFeatureCount

























Func _DWRITE_FONT_FEATURE($iNameTag = 0, $iParameter = 0)
	Local $tStruct = DllStructCreate($tagDWRITE_FONT_FEATURE)
	$tStruct.NameTag = $iNameTag
	$tStruct.Parameter = $iParameter
	Return $tStruct
EndFunc

Func _DWRITE_TEXT_RANGE($iStartPosition = 0, $iLength = -1)
	Local $tStruct = DllStructCreate($tagDWRITE_TEXT_RANGE)
	$tStruct.StartPosition = $iStartPosition
	$tStruct.Length = $iLength
	Return $tStruct
EndFunc

Func _DWRITE_GLYPH_RUN($oFontFace, $fFontEmSize, $iGlyphCount, $tGlyphIndices, $tGlyphAdvances = Null, $tGlyphOffsets = Null, $bIsSideways = False, $iBidiLevel = 0)
	Local $tStruct = DllStructCreate($tagDWRITE_GLYPH_RUN)
	If IsObj($oFontFace) Then
		$tStruct.FontFace = $oFontFace()
	Else
		$tStruct.FontFace = $oFontFace
	EndIf

	$tStruct.FontEmSize = $fFontEmSize
	$tStruct.GlyphCount = $iGlyphCount

	If IsDllStruct($tGlyphIndices) Then
		$tStruct.GlyphIndices = DllStructGetPtr($tGlyphIndices)
	Else
		$tStruct.GlyphIndices = $tGlyphIndices
	EndIf

	If IsDllStruct($tGlyphAdvances) Then
		$tStruct.GlyphAdvances = DllStructGetPtr($tGlyphAdvances)
	Else
		$tStruct.GlyphAdvances = $tGlyphAdvances
	EndIf

	If IsDllStruct($tGlyphOffsets) Then
		$tStruct.GlyphOffsets = DllStructGetPtr($tGlyphOffsets)
	Else
		$tStruct.GlyphOffsets = $tGlyphOffsets
	EndIf

	$tStruct.IsSideways = $bIsSideways
	$tStruct.BidiLevel = $iBidiLevel
	Return $tStruct
EndFunc