#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D.au3"
#include <GDIPlus.au3>
#include <ScreenCapture.au3>

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc


_Example()


Func _Example()

	Local $iGui_W = 600
	Local $iGui_H = 400

	Local $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)

	Local $oFactory = _D2D_Factory_Create()
	Local $oRender = _D2D_Factory_CreateHwndRenderTarget($oFactory, $hGui, $iGui_W, $iGui_H)
	Local $oLayer = _D2D_RenderTarget_CreateLayer($oRender)
	Local $oBitmap = _ScreenShot($oRender)

	GUISetState()


	Local $oPath = _D2D_Factory_CreatePathGeometry($oFactory)
	Local $oSink = _D2D_PathGeometry_Open($oPath)
	$oSink.SetFillMode($D2D1_FILL_MODE_WINDING)

	Local $fCX = $iGui_W * 0.5
	Local $fCY = $iGui_H * 0.5
	Local $fR = $iGui_H * 0.48
	Local $fRad2Deg = ATan(1) / 45

	_D2D_SimplifiedGeometrySink_BeginFigure($oSink, $fCX + Cos(270 * $fRad2Deg) * $fR, $fCY + Sin(270 * $fRad2Deg) * $fR)
	Local $aPoints[5][2] = [[4]]
	$aPoints[1][0] = $fCX + Cos(54 * $fRad2Deg) * $fR
	$aPoints[1][1] = $fCY + Sin(54 * $fRad2Deg) * $fR
	$aPoints[2][0] = $fCX + Cos(198 * $fRad2Deg) * $fR
	$aPoints[2][1] = $fCY + Sin(198 * $fRad2Deg) * $fR
	$aPoints[3][0] = $fCX + Cos(342 * $fRad2Deg) * $fR
	$aPoints[3][1] = $fCY + Sin(342 * $fRad2Deg) * $fR
	$aPoints[4][0] = $fCX + Cos(126 * $fRad2Deg) * $fR
	$aPoints[4][1] = $fCY + Sin(126 * $fRad2Deg) * $fR
	_D2D_SimplifiedGeometrySink_AddLines($oSink, $aPoints)
	_D2D_SimplifiedGeometrySink_EndFigure($oSink)

	$oSink.Close()
	$oSink = 0

	Local $tD2D1_LAYER_PARAMETERS = _D2D1_LAYER_PARAMETERS(0, 0, $iGui_W, $iGui_H, $oPath)

	Local $tMatrix, $iTimer = TimerInit()
	While GUIGetMsg() <> -3
		$tMatrix = _D2D_MATRIX3X2_Rotation(TimerDiff($iTimer) / 100, $fCX, $fCY)
		$oRender.SetTransform($tMatrix)


		$oRender.BeginDraw()
		$oRender.Clear(Null)

		$oRender.PushLayer($tD2D1_LAYER_PARAMETERS, $oLayer)
		$oRender.DrawBitmap($oBitmap, Null, 1, $D2D1_BITMAP_INTERPOLATION_MODE_LINEAR, Null)
		$oRender.PopLayer()

		$oRender.EndDraw(0, 0)
	WEnd

	$oPath = 0
	$oBitmap = 0
	$oLayer = 0
	$oRender = 0
	$oFactory = 0

	GUIDelete($hGui)
EndFunc   ;==>_Example





Func _ScreenShot(ByRef Const $oRender, $iLeft = 0, $iTop = 0, $iRight = -1, $iBottom = -1, $bCursor = False)
	_GDIPlus_Startup()

	Local $hBMP_Screen = _ScreenCapture_Capture("", $iLeft, $iTop, $iRight, $iBottom, $bCursor)
	Local $hBitmap_Screen = _GDIPlus_BitmapCreateFromHBITMAP($hBMP_Screen)
	_WinAPI_DeleteObject($hBMP_Screen)

	Local $iBmp_W = _GDIPlus_ImageGetWidth($hBitmap_Screen)
	Local $iBmp_H = _GDIPlus_ImageGetHeight($hBitmap_Screen)
	Local $iBmp_Stride = $iBmp_W * 4
	Local $tPixelData = DllStructCreate("uint PX[" & $iBmp_Stride * $iBmp_H & "];")


	;Local $hBitmap = _GDIPlus_BitmapCreateFromScan0($iBmp_W, $iBmp_H, $GDIP_PXF32ARGB, $iBmp_Stride, $tPixelData)
	Local $hBitmap = _GDIPlus_BitmapCreateFromScan0($iBmp_W, $iBmp_H, $GDIP_PXF32ARGB, $iBmp_Stride, DllStructGetPtr($tPixelData))
	Local $hContext = _GDIPlus_ImageGetGraphicsContext($hBitmap)
	_GDIPlus_GraphicsDrawImageRect($hContext, $hBitmap_Screen, 0, 0, $iBmp_W, $iBmp_H)
	_GDIPlus_BitmapDispose($hBitmap_Screen)
	_GDIPlus_GraphicsDispose($hContext)
	_GDIPlus_BitmapDispose($hBitmap)
	_GDIPlus_Shutdown()

	Local $oBitmap = _D2D_RenderTarget_CreateBitmap($oRender, $iBmp_W, $iBmp_H, $DXGI_FORMAT_B8G8R8A8_UNORM, $D2D1_ALPHA_MODE_PREMULTIPLIED, $tPixelData, $iBmp_Stride)

	Return $oBitmap
EndFunc   ;==>_ScreenShot
