#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D_1.au3"
#include "..\DirectWrite.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



Global $iGui_W = 800
Global $iGui_H = 600
Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)

Global $oD2D_Factory = _D2D_Factory1_Create()

Global $oD2D_DeviceContext, $oD3D11_SwapChain
_D2D_CreateHWNDContext($oD2D_Factory, $hGui, $oD2D_DeviceContext, $oD3D11_SwapChain)
If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error) & @CRLF & "You need Win8 or Win7+PlatformUpdate to run this example")


Global $oD2D_GaussianBlurEffect = _D2D_DeviceContext_CreateEffect($oD2D_DeviceContext, $sIID_D2D1GaussianBlur)
_D2D_Properties_SetValue($oD2D_GaussianBlurEffect, $D2D1_GAUSSIANBLUR_PROP_STANDARD_DEVIATION, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_Float(4))

Global $oD2D_PointSpecularEffect = _D2D_DeviceContext_CreateEffect($oD2D_DeviceContext, $sIID_D2D1PointSpecular)
_D2D_Properties_SetValue($oD2D_PointSpecularEffect, $D2D1_POINTSPECULAR_PROP_LIGHT_POSITION, $D2D1_PROPERTY_TYPE_VECTOR3, _D2D1_VECTOR_3F($iGui_W * 0.2, $iGui_H * 1.3, $iGui_H * 0.1))
_D2D_Properties_SetValue($oD2D_PointSpecularEffect, $D2D1_POINTSPECULAR_PROP_SPECULAR_EXPONENT, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_FLOAT(1))
_D2D_Properties_SetValue($oD2D_PointSpecularEffect, $D2D1_POINTSPECULAR_PROP_SURFACE_SCALE, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_FLOAT(3))
_D2D_Properties_SetValue($oD2D_PointSpecularEffect, $D2D1_POINTSPECULAR_PROP_SPECULAR_CONSTANT, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_FLOAT(1.1))

Global $oD2D_CompositeEffect = _D2D_DeviceContext_CreateEffect($oD2D_DeviceContext, $sIID_D2D1Composite)
_D2D_Properties_SetValue($oD2D_CompositeEffect, $D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT($D2D1_COMPOSITE_MODE_SOURCE_IN))

Global $oD2D_ArithmeticEffect = _D2D_DeviceContext_CreateEffect($oD2D_DeviceContext, $sIID_D2D1ArithmeticComposite)
_D2D_Properties_SetValue($oD2D_ArithmeticEffect, $D2D1_ARITHMETICCOMPOSITE_PROP_COEFFICIENTS, $D2D1_PROPERTY_TYPE_VECTOR4, _D2D1_VECTOR_4F(1, 0, 0, 0))


Global $oD2D_Bitmap = _CreateBitmap($oD2D_Factory, $oD2D_DeviceContext)


_D2D_Effect_SetInput($oD2D_GaussianBlurEffect, $oD2D_Bitmap)

_D2D_Effect_SetInputEffect($oD2D_PointSpecularEffect, $oD2D_GaussianBlurEffect)
_D2D_Effect_SetInput($oD2D_CompositeEffect, $oD2D_Bitmap)
_D2D_Effect_SetInputEffect($oD2D_CompositeEffect, $oD2D_PointSpecularEffect, 1)

_D2D_Effect_SetInput($oD2D_ArithmeticEffect, $oD2D_Bitmap)
_D2D_Effect_SetInputEffect($oD2D_ArithmeticEffect, $oD2D_CompositeEffect, 1)

Global $oD2D_EffectOut = _D2D_Effect_GetOutput($oD2D_ArithmeticEffect)
GUISetState()


$oD2D_DeviceContext.BeginDraw()
$oD2D_DeviceContext.Clear(_D2D1_COLOR_F(0, 0, 0, 1))
$oD2D_DeviceContext.DrawImage($oD2D_EffectOut, Null, Null, $D2D1_INTERPOLATION_MODE_CUBIC, $D2D1_COMPOSITE_MODE_SOURCE_OVER)
$oD2D_DeviceContext.EndDraw(0, 0)
$oD3D11_SwapChain.Present(1, 0)

While GUIGetMsg() <> -3
WEnd
GUIDelete($hGui)





Func _CreateBitmap(ByRef Const $oFactory, ByRef Const $oContext)
	Local $tSize = _D2D_RenderTarget_GetSize($oContext)
	Local $fW = $tSize.Width
	Local $fH = $tSize.Height

	Local $fO = 18

	Local $oPath1 = _D2D_Factory_CreateRoundedRectangleGeometry($oFactory, $fW * 0.1, $fH * 0.1, $fW * 0.9, $fH * 0.9, $fH * 0.1, $fH * 0.1)
	Local $oPath2 = _D2D_Factory_CreateRoundedRectangleGeometry($oFactory, $fW * 0.1 + $fO, $fH * 0.1 + $fO, $fW * 0.9 - $fO, $fH * 0.9 - $fO, $fH * 0.1 - $fO, $fH * 0.1 - $fO)
	Local $oPath_Outer = _D2D_Geometry_CombineWithGeometry($oPath1, $oPath2, $D2D1_COMBINE_MODE_EXCLUDE)



	Local $oDW_Factory = _DWrite_Factory_Create()
	Local $oFontFiles = _DWrite_Factory_CreateFontFileReference($oDW_Factory, @ScriptDir & "\Ethnocentric.ttf")
	Local $oFontFace = _DWrite_Factory_CreateFontFace($oDW_Factory, $oFontFiles)
	$oPath1 = _D2D_Factory_CreateRoundedRectangleGeometry($oFactory, $fW * 0.2, $fH * 0.2, $fW * 0.8, $fH * 0.8, $fH * 0.06, $fH * 0.06)
	$oPath2 = _CreateTextOutlinePath($oFactory, "A", $oFontFace, 96, $fW * 0.25, $fH * 0.25, $fW * 0.5, $fH * 0.5)
	Local $oPath_Inner = _D2D_Geometry_CombineWithGeometry($oPath1, $oPath2, $D2D1_COMBINE_MODE_EXCLUDE)



	Local $oRender = _D2D_RenderTarget_CreateCompatibleRenderTarget($oContext)
	Local $oBitmap = _D2D_BitmapRenderTarget_GetBitmap($oRender)
	Local $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 0, 0.8, 0.2)

	$oRender.BeginDraw()
	$oRender.FillGeometry($oPath_Outer, $oBrush, Null)
	$oRender.FillGeometry($oPath_Inner, $oBrush, Null)
	$oRender.EndDraw(0, 0)

	Return $oBitmap
EndFunc   ;==>_CreateBitmap






Func _CreateTextOutlinePath(ByRef Const $oFactory, $sString, ByRef Const $oFontFace, $fSize, $fX = 0, $fY = 0, $fW = 0, $fH = 0)
	Local $iStrLen = StringLen($sString)

	Local $tGlyphIndices = _DWrite_FontFace_GetGlyphIndices($oFontFace, $sString)

	Local $oPath = _D2D_Factory_CreatePathGeometry($oFactory)
	Local $oSink = _D2D_PathGeometry_Open($oPath)
	_DWrite_FontFace_GetGlyphRunOutline($oFontFace, $oSink, $fSize, $tGlyphIndices, $iStrLen)
	$oSink.Close()
	$oSink = 0

	Local $tBounds = _D2D_Geometry_GetBounds($oPath)

	Local $fScaleX = 1, $fScaleY = 1
	If $fW <> 0 Then $fScaleX = $fW / ($tBounds.Right - $tBounds.Left)
	If $fH <> 0 Then $fScaleY = $fH / ($tBounds.Bottom - $tBounds.Top)

	Local $tMatrix = DllStructCreate($tagD2D1_MATRIX_3X2_F)
	$tMatrix.M11 = $fScaleX
	$tMatrix.M22 = $fScaleY
	$tMatrix.M31 = $fX - $tBounds.Left * $fScaleX
	$tMatrix.M32 = $fY - $tBounds.Top * $fScaleY

	Local $oPathTrans = _D2D_Factory_CreateTransformedGeometry($oFactory, $oPath, $tMatrix)
	$oPath = 0

	Return $oPathTrans
EndFunc   ;==>_CreateTextOutlinePath
