#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D.au3"
#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>
#include <WinAPIMisc.au3>


Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc


HotKeySet("{ESC}", "_Exit")


Global $tWin = _LayeredWindow_Create("", 500, 500, BitOR($WS_EX_TOPMOST, $WS_EX_TOOLWINDOW, $WS_EX_TRANSPARENT))
GUISetState(@SW_SHOWNOACTIVATE, $tWin.Gui)

Global $oD2D_Factory = _D2D_Factory_Create()
Global $oD2D_DCRender = _D2D_Factory_CreateDCRenderTarget($oD2D_Factory)
_D2D_DCRenderTarget_BindDC($oD2D_DCRender, $tWin.CDC, _WinAPI_CreateRect(0, 0, $tWin.W, $tWin.H))

Global $oD2D_Brush = _D2D_RenderTarget_CreateSolidColorBrush($oD2D_DCRender, 0, 0.2, 1)

Global $aStop[3] = [2]
$aStop[1] = _D2D1_GRADIENT_STOP(0, 0, 0.2, 1)
$aStop[2] = _D2D1_GRADIENT_STOP(1, 0, 0.2, 1, 0)
Global $oD2D_StopCollection = _D2D_RenderTarget_CreateGradientStopCollection($oD2D_DCRender, $aStop)
Global $oD2D_BrushRadial = _D2D_RenderTarget_CreateRadialGradientBrush($oD2D_DCRender, $oD2D_StopCollection, 600, 10, 200, 200)
$oD2D_StopCollection = 0

Global $oD2D_Geometry = _D2D_Geometry_CombineWithGeometry(_D2D_Factory_CreateEllipseGeometry($oD2D_Factory, 250, 250, 240, 240), _
		_D2D_Factory_CreateEllipseGeometry($oD2D_Factory, 250, 250, 180, 180), $D2D1_COMBINE_MODE_EXCLUDE)



Global $iTimer = TimerInit()
While 1
	_Draw(TimerDiff($iTimer) / 250)
	_LayeredWindow_Draw($tWin)
WEnd




Func _Draw($fProgress)
	Local $fX = Cos($fProgress) * 210
	Local $fY = Sin($fProgress) * 210
	Local $fX1 = Cos($fProgress + 0.9) * 210
	Local $fY1 = Sin($fProgress + 0.9) * 210

	$oD2D_DCRender.BeginDraw()
	$oD2D_DCRender.Clear(_D2D1_COLOR_F(0, 0, 0, 0))

	$oD2D_BrushRadial.SetCenter(_D2D1_POINT_2F(250 + $fX, 250 + $fY))
	$oD2D_BrushRadial.SetGradientOriginOffset(_D2D1_POINT_2F($fX1 - $fX, $fY1 - $fY))

	$oD2D_DCRender.FillGeometry($oD2D_Geometry, $oD2D_BrushRadial, Null)
	$oD2D_DCRender.FillEllipse(_D2D1_ELLIPSE(250 + $fX1, 250 + $fY1, 40, 40), $oD2D_Brush)

	$oD2D_DCRender.EndDraw(0, 0)
EndFunc   ;==>_Draw


Func _LayeredWindow_Create($sTitle, $iWidth, $iHeight, $iExStyle = 0, $iX = Default, $iY = Default, $iAlpha = 0xFF)
	Local $tWin = DllStructCreate("int X; int Y; int W; int H; hwnd Gui; hwnd DC; handle BMP; hwnd CDC; hwnd OBJ; int Src[2]; byte Blend[4];")
	$tWin.Blend(3) = $iAlpha
	$tWin.Blend(4) = 1

	$tWin.Gui = GUICreate($sTitle, $iWidth, $iHeight, $iX, $iY, $WS_POPUP, BitOR($WS_EX_LAYERED, $iExStyle))

	$tWin.DC = _WinAPI_GetDC($tWin.Gui)
	$tWin.CDC = _WinAPI_CreateCompatibleDC($tWin.DC)
	$tWin.BMP = _WinAPI_CreateCompatibleBitmap($tWin.DC, $iWidth, $iHeight)
	$tWin.OBJ = _WinAPI_SelectObject($tWin.CDC, $tWin.BMP)

	DllCall("user32.dll", "bool", "GetWindowRect", "hwnd", $tWin.Gui, "struct*", $tWin)
	$tWin.W -= $tWin.X
	$tWin.H -= $tWin.Y

	Return $tWin
EndFunc   ;==>_LayeredWindow_Create


Func _LayeredWindow_Dispose(ByRef $tWin)
	If Not IsDllStruct($tWin) Then Return

	_WinAPI_SelectObject($tWin.CDC, $tWin.OBJ)
	_WinAPI_DeleteObject($tWin.BMP)
	_WinAPI_DeleteDC($tWin.CDC)
	_WinAPI_ReleaseDC($tWin.Gui, $tWin.DC)

	GUIDelete($tWin.Gui)
	$tWin = 0
EndFunc   ;==>_LayeredWindow_Dispose


Func _LayeredWindow_Draw(ByRef $tWin)
	If Not IsDllStruct($tWin) Then Return
	_WinAPI_UpdateLayeredWindow($tWin.Gui, $tWin.DC, DllStructGetPtr($tWin, "X"), DllStructGetPtr($tWin, "W"), $tWin.CDC, DllStructGetPtr($tWin, "Src"), 0, DllStructGetPtr($tWin, "Blend"), 2)
EndFunc   ;==>_LayeredWindow_Draw


Func _Exit()
	$oD2D_BrushRadial = 0
	$oD2D_DCRender = 0
	$oD2D_Factory = 0

	_LayeredWindow_Dispose($tWin)
	Exit
EndFunc   ;==>_Exit
