#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D_1.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



Global $iGui_W = 120
Global $iGui_H = 80
Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)
GUISetState()



Global $oD2D_Factory = _D2D_Factory1_Create()
Global $oD2D_Context, $oDXGI_SwapChain
_D2D_CreateHWNDContext($oD2D_Factory, $hGui, $oD2D_Context, $oDXGI_SwapChain)
If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error) & @CRLF & "You need Win8 or Win7+PlatformUpdate to run this example")

Global $oD2D_Brush = _D2D_RenderTarget_CreateSolidColorBrush($oD2D_Context, 1, 1, 0)

Global $oD2D_GetBitmap = _D2D_DeviceContext_CreateBitmap($oD2D_Context, $iGui_W, $iGui_H, BitOR($D2D1_BITMAP_OPTIONS_CPU_READ, $D2D1_BITMAP_OPTIONS_CANNOT_DRAW))
Global $oD2D_SetBitmap = _D2D_DeviceContext_GetTarget($oD2D_Context)


;Draw some stuff ($oD2D_SetBitmap)
$oD2D_Context.BeginDraw()
_D2D_RenderTarget_Clear($oD2D_Context, 0, 0, 1)
_D2D_RenderTarget_DrawLine($oD2D_Context, 10, 10, $iGui_W - 10, $iGui_H - 10, $oD2D_Brush, 10)
$oD2D_Context.EndDraw(0, 0)
$oDXGI_SwapChain.Present(1, 0)


Sleep(1000)


Global $tData = _GetPixelData()

Global $iStride = $tData.Stride / 4 ; Bytes -> UInt
Global $iOff, $iColor, $iRed, $iGreen, $iBlue, $iAlpha
For $y = 0 To $iGui_H - 1
	$iOff = $y * $iStride
	For $x = 1 To $iGui_W
		$iColor = $tData.Color(($iOff + $x))

		$iAlpha = BitAND(BitShift($iColor, 24), 0xFF)
		$iRed = BitAND(BitShift($iColor, 16), 0xFF)
		$iGreen = BitAND(BitShift($iColor, 8), 0xFF)
		$iBlue = BitAND($iColor, 0xFF)

		;Swap Green and Blue
		$tData.Color(($iOff + $x)) = BitOR(BitShift($iAlpha, -24), BitShift($iRed, -16), BitShift($iBlue, -8), $iGreen)
	Next
Next

_SetPixelData($tData)



While GUIGetMsg() <> -3
WEnd



Func _GetPixelData()
	Local $iHResult = $oD2D_GetBitmap.CopyFromRenderTarget(Null, $oD2D_Context, Null)
	;Or: Local $iHResult = $oD2D_GetBitmap.CopyFromBitmap(Null, $oD2D_SetBitmap, Null)
	ConsoleWrite("! CopyFromRenderTarged: " & _D2D_ErrorMessage($iHResult) & @CRLF)

	Local $tD2D1_MAPPED_RECT = DllStructCreate($tagD2D1_MAPPED_RECT)
	$iHResult = $oD2D_GetBitmap.Map($D2D1_MAP_OPTIONS_READ, $tD2D1_MAPPED_RECT)
	ConsoleWrite("! Map to Memory: " & _D2D_ErrorMessage($iHResult) & @CRLF)

	Local $iStride = $tD2D1_MAPPED_RECT.Pitch / 4 ; Bytes to UInt

	Local $tData = DllStructCreate("uint Stride; uint Color[" & $iStride * $iGui_H & "];")
	$tData.Stride = $tD2D1_MAPPED_RECT.Pitch

	DllCall("kernel32.dll", "none", "RtlMoveMemory", "struct*", DllStructGetPtr($tData, "Color"), "struct*", $tD2D1_MAPPED_RECT.Bits, "ulong_ptr", DllStructGetSize($tData) - 4)

	$oD2D_GetBitmap.UnMap()

	Return $tData
EndFunc   ;==>_GetPixelData



Func _SetPixelData(ByRef $tData)
	Local $iHResult = $oD2D_SetBitmap.CopyFromMemory(Null, DllStructGetPtr($tData, "Color"), $tData.Stride)
	ConsoleWrite("! CopyFromMemory: " & _D2D_ErrorMessage($iHResult) & @CRLF)

	$oDXGI_SwapChain.Present(1, 0)
EndFunc   ;==>_SetPixelData
