#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\WIC.au3"
#include "..\Direct2D_1.au3"
#include <GUIConstantsEx.au3>


Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



_Example()



Func _Example()

	Opt("GUIOnEventMode", 1)

	Local $iGui_W = 1000
	Local $iGui_H = 600
	Local $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)
	GUISetOnEvent($GUI_EVENT_CLOSE, "_Exit")
	GUISetState()

	Local $oD2D_Factory = _D2D_Factory1_Create()
	Local $oD2D_DeviceContext, $oD2D_SwapChain
	_D2D_CreateHWNDContext($oD2D_Factory, $hGui, $oD2D_DeviceContext, $oD2D_SwapChain)
	If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error) & @CRLF & "You need Win8 or Win7+PlatformUpdate to run this example")


	Local $oWIC_ImagingFactory = _WIC_ImagingFactory_Create()
	Local $oWIC_BitmapDecoder = _WIC_ImagingFactory_CreateDecoderFromFilename($oWIC_ImagingFactory, @ScriptDir & "\mandrill.jpg")
	If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error))

	Local $oWIC_Bitmap = _WIC_BitmapDecoder_GetFrame($oWIC_BitmapDecoder)
	Local $oWIC_FormatConverter = _WIC_ImagingFactory_CreateFormatConverter($oWIC_ImagingFactory)
	$oWIC_FormatConverter.Initialize($oWIC_Bitmap, _WinAPI_GUIDFromString($sGUID_WICPixelFormat32bppPBGRA), 0, 0, 0, 1)

	Local $iImgW, $iImgH
	$oWIC_FormatConverter.GetSize($iImgW, $iImgH)

	Local $oD2D_Bitmap = _D2D_RenderTarget_CreateBitmapFromWicBitmap($oD2D_DeviceContext, $oWIC_FormatConverter)


	Local $tMatrix_Perspective = _D2D_Matrix4x4_PerspectiveProjection(1000)
	Local $tMatrix_Translation = _D2D_Matrix4x4_Translation(-$iImgW * 0.5, -$iImgH * 0.5, -400)
	Local $tMatrix_Center = _D2D_Matrix4x4_Translation($iGui_W * 0.5, $iGui_H * 0.5, 0)

	Local $tMatrix
	Local $iTimer = TimerInit()
	While 1
		$oD2D_DeviceContext.BeginDraw()
		$oD2D_DeviceContext.Clear(_D2D1_Color_F(0, 0.03, 0.1, 1))

		$tMatrix = _D2D_MATRIX4X4_SetProduct($tMatrix_Translation, _D2D_Matrix4x4_RotationY(TimerDiff($iTimer) / 8))
		$tMatrix = _D2D_MATRIX4X4_SetProduct($tMatrix, $tMatrix_Perspective)
		$tMatrix = _D2D_MATRIX4X4_SetProduct($tMatrix, $tMatrix_Center)

		_D2D_DeviceContext_DrawBitmap($oD2D_DeviceContext, $oD2D_Bitmap, 0, 0, 1, $D2D1_INTERPOLATION_MODE_DEFINITION_MULTI_SAMPLE_LINEAR, $tMatrix)

		$oD2D_DeviceContext.EndDraw(0, 0)
		$oD2D_SwapChain.Present(1, 0)
	WEnd

EndFunc   ;==>_Example



Func _Exit()
	Exit
EndFunc   ;==>_Exit
