#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc


_Example()


Func _Example()
	Local $iGui_W = 400
	Local $iGui_H = 300
	Local $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)
	GUISetState()

	;while there is no DXGI or DX10 UDF...:
	Local $__g_hD3D10_1DLL = DllOpen("d3d10_1.dll")

	Local Const $D3D10_1_SDK_VERSION = 0x20
	Local Const $D3D10_DRIVER_TYPE_HARDWARE = 0
	Local Const $D3D10_CREATE_DEVICE_BGRA_SUPPORT = 0x20
	Local Const $D3D10_FEATURE_LEVEL_10_1 = 0xA100


	Local Const $DXGI_USAGE_RENDER_TARGET_OUTPUT = 0x00000020

	Local Const $tagDXGI_RATIONAL = "struct; uint Numerator; uint Denominator; endstruct;"
	Local Const $tagDXGI_MODE_DESC = "struct; uint Width; uint Height; " & $tagDXGI_RATIONAL & " uint Format; uint ScanlineOrdering; uint Scaling; endstruct;"
	Local Const $tagDXGI_SAMPLE_DESC = "struct; uint SampleDescCount; uint SampleDescQuality; endstruct;"
	Local Const $tagDXGI_SWAP_CHAIN_DESC = "struct; " & $tagDXGI_MODE_DESC & " " & $tagDXGI_SAMPLE_DESC & " uint BufferUsage; uint BufferCount; hwnd OutputWindow; bool Windowed; uint SwapEffect; uint Flags; endstruct;"

	Local Const $sIID_IDXGISurface = "{cafcb56c-6ac3-4889-bf47-9e23bbd260ec}"
	Local Const $sIID_IDXGISwapChain = "{310d36a0-d2e7-4c0a-aa04-6a9d23b8886a}"

	Local Const $tagIDXGISwapChain = "SetPrivateData hresult(wstr;uint;struct*);" & _
			"SetPrivateDataInterface hresult(wstr;struct*);" & _
			"GetPrivateData hresult(wstr;uint*;struct*);" & _
			"GetParent hresult(struct*;ptr*);" & _
			"GetDevice hresult(struct*;ptr*);" & _
			"Present hresult(uint;uint);" & _
			"GetBuffer hresult(uint;struct*;ptr*);" & _
			"SetFullscreenState hresult(bool;struct*);" & _
			"GetFullscreenState hresult(bool*;ptr*);" & _
			"GetDesc hresult(struct*);" & _
			"ResizeBuffers hresult(uint;uint;uint;uint;uint);" & _
			"ResizeTarget hresult(struct*);" & _
			"GetContainingOutput hresult(ptr*);" & _
			"GetFrameStatistics hresult(struct*);" & _
			"GetLastPresentCount hresult(uint*);"



	Local $tDXGI_SWAP_CHAIN_DESC = DllStructCreate($tagDXGI_SWAP_CHAIN_DESC)
	$tDXGI_SWAP_CHAIN_DESC.Width = $iGui_W
	$tDXGI_SWAP_CHAIN_DESC.Height = $iGui_H
	$tDXGI_SWAP_CHAIN_DESC.BufferUsage = $DXGI_USAGE_RENDER_TARGET_OUTPUT
	$tDXGI_SWAP_CHAIN_DESC.Format = $DXGI_FORMAT_B8G8R8A8_UNORM
	$tDXGI_SWAP_CHAIN_DESC.SampleDescCount = 1
	$tDXGI_SWAP_CHAIN_DESC.BufferCount = 1
	$tDXGI_SWAP_CHAIN_DESC.Numerator = 60
	$tDXGI_SWAP_CHAIN_DESC.Denominator = 1
	$tDXGI_SWAP_CHAIN_DESC.OutputWindow = $hGui
	$tDXGI_SWAP_CHAIN_DESC.Windowed = True



	Local $aResult = DllCall($__g_hD3D10_1DLL, "uint", "D3D10CreateDeviceAndSwapChain1", "struct*", Null, "uint", $D3D10_DRIVER_TYPE_HARDWARE, "ptr", Null, "uint", $D3D10_CREATE_DEVICE_BGRA_SUPPORT, "uint", $D3D10_FEATURE_LEVEL_10_1, "uint", $D3D10_1_SDK_VERSION, "struct*", $tDXGI_SWAP_CHAIN_DESC, "ptr*", 0, "ptr*", 0)
	If @error Then Exit
	If $aResult[0] Then Exit ConsoleWrite("! CreateDeviceAndSwapChain: " & _D2D_ErrorMessage($aResult[0]) & @CRLF)
	Local $pDX10_Device1 = $aResult[9]

	Local $oDXGI_SwapChain = __D2D_ObjCreate($aResult[8], $sIID_IDXGISwapChain, $tagIDXGISwapChain)

	Local $pDXGI_Surface
	Local $iHresult = $oDXGI_SwapChain.GetBuffer(0, _WinAPI_GUIDFromString($sIID_IDXGISurface), $pDXGI_Surface)
	If $iHresult Then Exit ConsoleWrite("! SwapChain.GetBuffer: " & _D2D_ErrorMessage($iHresult) & @CRLF)



	Local $oD2D_Factory = _D2D_Factory_Create()
	Local $oD2D_Render = _D2D_Factory_CreateDxgiSurfaceRenderTarget($oD2D_Factory, $pDXGI_Surface, $D2D1_RENDER_TARGET_TYPE_DEFAULT, $DXGI_FORMAT_B8G8R8A8_UNORM, $D2D1_ALPHA_MODE_PREMULTIPLIED)

	Local $oD2D_Brush = _D2D_RenderTarget_CreateSolidColorBrush($oD2D_Render, 1, 1, 0)
	Local $oD2D_Stroke = _D2D_Factory_CreateStrokeStyle($oD2D_Factory, 0, 0, $D2D1_CAP_STYLE_ROUND, $D2D1_CAP_STYLE_TRIANGLE)



	$oD2D_Render.BeginDraw()
	_D2D_RenderTarget_DrawLine($oD2D_Render, 100, 100, 300, 100, $oD2D_Brush, 60, $oD2D_Stroke)
	$oD2D_Render.DrawLine(_D2D1_POINT_2F(100, 200), _D2D1_POINT_2F(300, 200), $oD2D_Brush, 60, $oD2D_Stroke)
	$oD2D_Render.EndDraw(0, 0)
	$oDXGI_SwapChain.Present(1, 0)


	While GUIGetMsg() <> -3
	WEnd

	$oD2D_Render = 0
	__D2D_ObjDispose($pDXGI_Surface)

	$oDXGI_SwapChain = 0
	__D2D_ObjDispose($pDX10_Device1)

	GUIDelete($hGui)
EndFunc   ;==>_Example
