#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\Direct2D.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc


_Example()



Func _Example()

	Local $hGui = GUICreate("Direct2D", 600, 400, 10, 10)

	Local $oFactory = _D2D_Factory_Create()
	Local $oRender = _D2D_Factory_CreateHwndRenderTarget($oFactory, $hGui)

	Local $oGeometry_Rect_A = _D2D_Factory_CreateRectangleGeometry($oFactory, 50, 50, 400, 250)
	Local $oGeometry_Rect_B = _D2D_Factory_CreateRectangleGeometry($oFactory, 200, 150, 550, 350)

	GUISetState()

	Local $oGeometry_Union = _D2D_Geometry_CombineWithGeometry($oGeometry_Rect_A, $oGeometry_Rect_B, $D2D1_COMBINE_MODE_UNION)
	Local $oGeometry_Intersec = _D2D_Geometry_CombineWithGeometry($oGeometry_Rect_A, $oGeometry_Rect_B, $D2D1_COMBINE_MODE_INTERSECT)
	Local $oGeometry_XOr = _D2D_Geometry_CombineWithGeometry($oGeometry_Rect_A, $oGeometry_Rect_B, $D2D1_COMBINE_MODE_XOR)
	Local $oGeometry_Exclude_A = _D2D_Geometry_CombineWithGeometry($oGeometry_Rect_A, $oGeometry_Rect_B, $D2D1_COMBINE_MODE_EXCLUDE)
	Local $oGeometry_Exclude_B = _D2D_Geometry_CombineWithGeometry($oGeometry_Rect_B, $oGeometry_Rect_A, $D2D1_COMBINE_MODE_EXCLUDE)



	Local $oBrush_A = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 1, 0, 0, 0.7)
	Local $oBrush_B = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 0, 0, 1, 0.7)
	Local $oBrush_C1 = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 1, 0, 1, 0.5)
	Local $oBrush_C2 = _D2D_RenderTarget_CreateSolidColorBrush($oRender)

	Local $tColor_BG = _D2D1_COLOR_F(1, 1, 1)


	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "RectangleGeometry A and RectangleGeometry B")


	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.FillGeometry($oGeometry_Union, $oBrush_C1, Null)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.DrawGeometry($oGeometry_Union, $oBrush_C2, 6, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "CombineMode: UNION")

	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.FillGeometry($oGeometry_Intersec, $oBrush_C1, Null)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.DrawGeometry($oGeometry_Intersec, $oBrush_C2, 6, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "CombineMode: INTERSECT")

	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.FillGeometry($oGeometry_XOr, $oBrush_C1, Null)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.DrawGeometry($oGeometry_XOr, $oBrush_C2, 6, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "CombineMode: XOR")

	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.FillGeometry($oGeometry_Exclude_A, $oBrush_C1, Null)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.DrawGeometry($oGeometry_Exclude_A, $oBrush_C2, 6, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "CombineMode: EXCLUDE A - B")

	$oRender.BeginDraw()
	$oRender.Clear($tColor_BG)
	$oRender.FillGeometry($oGeometry_Exclude_B, $oBrush_C1, Null)
	$oRender.DrawGeometry($oGeometry_Rect_A, $oBrush_A, 4, 0)
	$oRender.DrawGeometry($oGeometry_Rect_B, $oBrush_B, 4, 0)
	$oRender.DrawGeometry($oGeometry_Exclude_B, $oBrush_C2, 6, 0)
	$oRender.EndDraw(0, 0)
	MsgBox(0, "", "CombineMode: EXCLUDE B - A")

	$oGeometry_Union = 0
	$oGeometry_Intersec = 0
	$oGeometry_XOr = 0
	$oGeometry_Exclude_A = 0
	$oGeometry_Exclude_B = 0
	$oGeometry_Rect_A = 0
	$oGeometry_Rect_B = 0

	$oBrush_A = 0
	$oBrush_B = 0
	$oBrush_C1 = 0
	$oBrush_C2 = 0
	$oRender = 0
	$oFactory = 0

	GUIDelete($hGui)
EndFunc   ;==>_Example
