#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y
#include "..\WIC.au3"
#include "..\Direct2D.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc


Global $iGui_W = 600
Global $iGui_H = 400

Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)
Global $oFactory = _D2D_Factory_Create()
Global $oRender = _D2D_Factory_CreateHwndRenderTarget($oFactory, $hGui, $iGui_W, $iGui_H, 0, $D2D1_RENDER_TARGET_TYPE_SOFTWARE);SoftwareRendering!
Global $oBuffer = _D2D_RenderTarget_CreateCompatibleRenderTarget($oRender)
Global $oBitmap = _D2D_BitmapRenderTarget_GetBitmap($oBuffer)
GUISetState()



Global $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 1, 1, 0)
$oBuffer.BeginDraw()
$oBuffer.Clear(_D2D1_COLOR_F(0, 0.2, 0.2))
$oBrush.SetColor(_D2D1_COLOR_F(0.2, 0, 0.2))
$oBuffer.FillEllipse(_D2D1_ELLIPSE(300, 200, 200, 100), $oBrush)
$oBrush.SetColor(_D2D1_COLOR_F(0.7, 0, 0.7))
$oBuffer.DrawEllipse(_D2D1_ELLIPSE(300, 200, 200, 100), $oBrush, 8, 0)
$oBuffer.EndDraw(0, 0)



$oRender.BeginDraw()
$oRender.DrawBitmap($oBitmap, Null, 1, 0, Null)
$oRender.EndDraw(0, 0)


_SaveBitmapToFile($oFactory, $oBitmap, @MyDocumentsDir & "\Save.jpg")

Global $iHResult = @error, $iExt = @extended
If $iHResult Then
	ConsoleWrite("! Save: " & _D2D_ErrorMessage($iHResult) & " " & $iExt & @CRLF)
Else
	ConsoleWrite("+ Save OK" & @CRLF)
	ShellExecute(@MyDocumentsDir & "\Save.jpg")
EndIf



While GUIGetMsg() <> -3
WEnd


$oBrush = 0
$oBitmap = 0
$oBuffer = 0
$oRender = 0
$oFactory = 0

GUIDelete($hGui)






Func _SaveBitmapToFile(ByRef Const $oFactory, $oBitmap, $sFile, $sGUID_ContainerFormat = "", $fJpgQuality = 1)
	If Not IsObj($oFactory) Then Return SetError($D2DERR_NOOBJ, 0, False)
	If Not IsObj($oBitmap) Then Return SetError($D2DERR_NOOBJ, 1, False)

	Local $iHResult = 0

	Local $tSize = DllStructCreate($tagD2D1_SIZE_U)
	$oBitmap.GetPixelSize($tSize)

	Local $oWIC_Factory = _WIC_ImagingFactory_Create()
	Local $oWIC_Bitmap = _WIC_ImagingFactory_CreateBitmap($oWIC_Factory, $tSize.Width, $tSize.Height)
	If @error Then Return SetError(@error, 2, False)

	Local $oRender = _D2D_Factory_CreateWicBitmapRenderTarget($oFactory, $oWIC_Bitmap)
	If @error Then Return SetError(@error, 3, False)


	$oRender.BeginDraw()
	$oRender.DrawBitmap($oBitmap, Null, 1, 0, Null)
	$iHResult = $oRender.EndDraw(0, 0)
	If $iHResult Then Return SetError($iHResult, 4, False)


	Local $oStream = _WIC_ImagingFactory_CreateStream($oWIC_Factory)
	Local $tFormat = _WinAPI_GUIDFromString($sGUID_WICPixelFormat32bppPBGRA)

	$iHResult = $oStream.InitializeFromFilename($sFile, $GENERIC_WRITE)
	If $iHResult Then Return SetError($iHResult, 5, False)

	Local $sExt = StringUpper(StringRegExpReplace($sFile, "(?m).*\.(\w+)$", "$1"))
	If Not $sGUID_ContainerFormat Then
		Switch $sExt
			Case "PNG"
				$sGUID_ContainerFormat = $sGUID_ContainerFormatPng
			Case "ICO"
				$sGUID_ContainerFormat = $sGUID_ContainerFormatIco
			Case "JPG", "JPE" ;?
				$sGUID_ContainerFormat = $sGUID_ContainerFormatJpeg
			Case "TIF", "TIFF"
				$sGUID_ContainerFormat = $sGUID_ContainerFormatTiff
			Case "GIF"
				$sGUID_ContainerFormat = $sGUID_ContainerFormatGif
			Case "WMP" ;?
				$sGUID_ContainerFormat = $sGUID_ContainerFormatWmp
			Case Else
				$sGUID_ContainerFormat = $sGUID_ContainerFormatBmp
		EndSwitch
	EndIf

	Local $oEncoder = _WIC_ImagingFactory_CreateEncoder($oWIC_Factory, $sGUID_ContainerFormat)
	If @error Then Return SetError(@error, 6, False)

	$iHResult = $oEncoder.Initialize($oStream, $WICBitmapEncoderNoCache)
	If $iHResult Then Return SetError($iHResult, 7, False)

	Local $oProp
	Local $oFrame = _WIC_BitmapEncoder_CreateNewFrame($oEncoder, $oProp)

	If $sExt = "JPG" Then
		Local $tagPROPBAG2 = "struct; uint dwType; ushort vt; short cfType; uint dwHint; ptr pstrName; byte clsid[16]; endstruct;"
		Local $tPropBag = DllStructCreate($tagPROPBAG2)
		$tPropBag.vt = 4 ;VT_R4 = float

		Local $tVariant = DllStructCreate("ushort Type; ushort[3]; float Val;")
		$tVariant.Type = 4
		$tVariant.Val = $fJpgQuality

		Local $tString = DllStructCreate("wchar[32];")
		DllStructSetData($tString, 1, "ImageQuality")
		DllStructSetData($tPropBag, "pstrName", DllStructGetPtr($tString))

		$iHResult = $oProp.Write(1, $tPropBag, $tVariant)
		If $iHResult Then Return SetError($iHResult, 1000, False)
	EndIf


	$iHResult = $oFrame.Initialize($oProp)
	If $iHResult Then Return SetError($iHResult, 8, False)

	$iHResult = $oFrame.SetSize($tSize.Width, $tSize.Height)
	If $iHResult Then Return SetError($iHResult, 9, False)

	$iHResult = $oFrame.SetPixelFormat($tFormat)
	If $iHResult Then Return SetError($iHResult, 10, False)

	$iHResult = $oFrame.WriteSource($oWIC_Bitmap, Null)
	If $iHResult Then Return SetError($iHResult, 11, False)

	$iHResult = $oFrame.Commit()
	If $iHResult Then Return SetError($iHResult, 12, False)

	$iHResult = $oEncoder.Commit()
	If $iHResult Then Return SetError($iHResult, 13, False)


	$oFrame = 0
	$oEncoder = 0
	$oStream = 0
	$oRender = 0
	$oWIC_Bitmap = 0
	$oWIC_Factory = 0

	Return True
EndFunc   ;==>_SaveBitmapToFile