#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y

#include "..\WIC.au3"
#include "..\Direct2D.au3"

Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



;Load AutoIt-Logo
Global $sRegPath = "HKLM\SOFTWARE\AutoIt v3\AutoIt"
If StringInStr("X64IA64", @OSArch) Then $sRegPath = StringReplace($sRegPath, "SOFTWARE", "SOFTWARE\Wow6432Node") ;get AutoIt install dir
Global $sFile = RegRead($sRegPath, "InstallDir") & "\Examples\GUI\logo4.gif"
If Not FileExists($sFile) Then Exit MsgBox(BitOR($MB_SYSTEMMODAL, $MB_ICONHAND), "", $sFile & " not found!", 30)

Global $oWIC_ImagingFactory = _WIC_ImagingFactory_Create()
Global $oWIC_BitmapDecoder = _WIC_ImagingFactory_CreateDecoderFromFilename($oWIC_ImagingFactory, $sFile)
If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error))


;Convert to 32bppPBGRA
Global $oWIC_Frame = _WIC_BitmapDecoder_GetFrame($oWIC_BitmapDecoder)
Global $oWIC_FormatConverter = _WIC_ImagingFactory_CreateFormatConverter($oWIC_ImagingFactory)
$oWIC_FormatConverter.Initialize($oWIC_Frame, _WinAPI_GUIDFromString($sGUID_WICPixelFormat32bppPBGRA), 0, 0, 0, 1)
Global $oWIC_Bitmap = _WIC_ImagingFactory_CreateBitmapFromSource($oWIC_ImagingFactory, $oWIC_FormatConverter)
$oWIC_Frame = 0
$oWIC_BitmapDecoder = 0
$oWIC_FormatConverter = 0




;Lock Bitmap
Global $oWIC_Bitmap_Lock = _WIC_Bitmap_Lock($oWIC_Bitmap, 0, 0, 169, 68, BitOR($WICBitmapLockRead, $WICBitmapLockWrite))

Global $iLockW, $iLockH, $iStride
$oWIC_Bitmap_Lock.GetSize($iLockW, $iLockH)
$oWIC_Bitmap_Lock.GetStride($iStride)

Global $pData, $iData
$oWIC_Bitmap_Lock.GetDataPointer($iData, $pData)

Global $tPixelFormat = DllStructCreate($tagGUID)
$oWIC_Bitmap_Lock.GetPixelFormat($tPixelFormat)
ConsoleWrite("> PixelFormat: " & _WinAPI_StringFromGUID($tPixelFormat) & " = " & $sGUID_WICPixelFormat32bppPBGRA & @CRLF)

$iStride /= 4 ;(Stride in Bytes to Stride in UInts)

Global $tData = DllStructCreate("uint C[" & $iStride * $iLockH & "]", $pData)
Global $iColor, $iR, $iG, $iB, $iA, $iOff

;Swap Blue and Green channel
For $y = 0 To $iLockH - 1
	$iOff = $y * $iStride
	For $x = 1 To $iLockW
		$iColor = $tData.C(($iOff + $x))
		$iA = BitAND(BitShift($iColor, 24), 0xFF)
		$iR = BitAND(BitShift($iColor, 16), 0xFF)
		$iB = BitAND(BitShift($iColor, 8), 0xFF) ;Swap Green and Blue
		$iG = BitAND($iColor, 0xFF) ;Swap Green and Blue
		$tData.C(($iOff + $x)) = BitOR(BitShift($iA, -24), BitShift($iR, -16), BitShift($iG, -8), $iB)
	Next
Next
$oWIC_Bitmap_Lock = 0
$oWIC_ImagingFactory = 0







;Create GUI and show Bitmap
Global $iImgW, $iImgH
$oWIC_Bitmap.GetSize($iImgW, $iImgH)

Global $fScale = @DesktopWidth / $iImgW
Global $fScaleY = @DesktopHeight / $iImgH
If $fScaleY < $fScale Then $fScale = $fScaleY

Global $iGui_W = $iImgW * $fScale * 0.5
Global $iGui_H = $iImgH * $fScale * 0.5

Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)

Global $oD2D_Factory = _D2D_Factory_Create()
Global $oD2D_HwndRenderTarget = _D2D_Factory_CreateHwndRenderTarget($oD2D_Factory, $hGui, $iGui_W, $iGui_H)
Global $oD2D_Bitmap = _D2D_RenderTarget_CreateBitmapFromWicBitmap($oD2D_HwndRenderTarget, $oWIC_Bitmap)
$oWIC_Bitmap = 0


GUISetState()



$oD2D_HwndRenderTarget.BeginDraw()
$oD2D_HwndRenderTarget.DrawBitmap($oD2D_Bitmap, _D2D1_RECT_F(0,0,$iGui_W, $iGui_H), 1, 1, Null)
$oD2D_HwndRenderTarget.EndDraw(0, 0)


While GUIGetMsg() <> -3
WEnd

$oD2D_Bitmap = 0
$oD2D_HwndRenderTarget = 0
$oD2D_Factory = 0

GUIDelete($hGui)
