#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y

#include "..\Direct2D.au3"
#include "..\DirectWrite.au3"
#include <GUIConstantsEx.au3>


Opt("MustDeclareVars", 1)
Opt("GUIOnEventMode", 1)


Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



Global $iGui_W = 800
Global $iGui_H = 500

Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)
GUISetOnEvent($GUI_EVENT_CLOSE, "_Exit")
Global $oFactory = _D2D_Factory_Create()
Global $oRender = _D2D_Factory_CreateHwndRenderTarget($oFactory, $hGui, $iGui_W, $iGui_H)
$oRender.SetAntialiasMode($D2D1_ANTIALIAS_MODE_ALIASED)
Global $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRender, 0, 0.4, 1)

Global $oDW_Factory = _DWrite_Factory_Create()
Global $oFontFiles = _DWrite_Factory_CreateFontFileReference($oDW_Factory, @WindowsDir & "\Fonts\Gabriola.ttf")
Global $oFontFace = _DWrite_Factory_CreateFontFace($oDW_Factory, $oFontFiles)
$oFontFiles = 0

Global $oPath = _CreateTextOutlinePath($oFactory, "AutoIt", $oFontFace, 300, $iGui_W * 0.1, $iGui_H * 0.3, $iGui_W * 0.8, $iGui_H * 0.4)

GUISetState()


Global $tTriangles = _Tessellate($oPath)
Global $aRnd[$tTriangles.Cnt][2]

For $i = 0 To $tTriangles.Cnt - 1
	$aRnd[$i][0] = Random(-100, 100)
	$aRnd[$i][1] = Random(-40, 40)
Next


Global $oMesh, $fStep = 0, $fT

While 1
	$fStep += 0.03
	$fT = Sin($fStep) * 0.5 + 0.48
	If $fT < 0 Then $fT = 0
	$oMesh = _CalcMesh($oRender, $tTriangles, $aRnd, $fT)

	$oRender.BeginDraw()
	$oRender.Clear(_D2D1_COLOR_F(0, 0, 0.3))
	$oRender.FillMesh($oMesh, $oBrush)
	$oRender.EndDraw(0, 0)
WEnd







Func _CalcMesh(ByRef Const $oRender, ByRef Const $tTriangles, ByRef Const $aRnd, $fT)
	Local $tTri = DllStructCreate("float[" & $tTriangles.Cnt * 6 & "];")

	Local $fOX, $fOY, $iIdx
	For $i = 0 To $tTriangles.Cnt - 1
		$iIdx = $i * 6
		$fOX = $aRnd[$i][0] * $fT
		$fOY = $aRnd[$i][1] * $fT

		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 1) + $fOX, $iIdx + 1)
		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 2) + $fOY, $iIdx + 2)
		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 3) + $fOX, $iIdx + 3)
		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 4) + $fOY, $iIdx + 4)
		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 5) + $fOX, $iIdx + 5)
		DllStructSetData($tTri, 1, DllStructGetData($tTriangles, 1, $iIdx + 6) + $fOY, $iIdx + 6)
	Next

	Local $oMesh = _D2D_RenderTarget_CreateMesh($oRender)
	Local $oTess = _D2D_Mesh_Open($oMesh)
	$oTess.AddTriangles($tTri, $tTriangles.Cnt)
	$oTess.Close()

	Return $oMesh
EndFunc   ;==>_CalcMesh





Func _Tessellate(ByRef Const $oPath, $iMaxTriangles = 10000)
	Local $hRelease

	Local $tTriangles = DllStructCreate("float Pnts[" & ($iMaxTriangles + 1) * 6 & "]; uint Cnt;")

	Local $tTessEx = _CreateTessellationSink($hRelease, $tTriangles, $iMaxTriangles)
	Local $oTessEx = _CreateTessellationSinkOBJ($tTessEx)

	$oPath.Tessellate(Null, 0.1, $oTessEx)
	$oTessEx.Close()
	$oTessEx = 0
	DllCallbackFree($hRelease)

	Return $tTriangles
EndFunc   ;==>_Tess







Func _QueryInterface($pSink, $IID, $pOBJ)
	#forceref $pSink, $IID, $pOBJ
	If Not $pSink Then Return

	Local $tObj = DllStructCreate("ptr P;", $pOBJ)
	$tObj.P = $pSink

	_AddRef($pSink)
	Return $S_OK
EndFunc   ;==>_QueryInterface


Func _AddRef($pSink)
	If Not $pSink Then Return

	Local $tSink = DllStructCreate("ptr lpVtbl; int iRefCnt;", $pSink)
	$tSink.iRefCnt += 1

	Return $tSink.iRefCnt
EndFunc   ;==>_AddRef


Func _Release($pSink)
	If Not $pSink Then Return

	Local $tSink = DllStructCreate("ptr lpVtbl; int iRefCnt;", $pSink)
	$tSink.iRefCnt -= 1

	If $tSink.iRefCnt = 0 Then
		$tSink = DllStructCreate("ptr lpVtbl; int iRefCnt; ptr pQueryInterface; ptr pAddRef; ptr pRelease; ptr pAddTriangles; ptr pClose; hwnd hQueryInterface; hwnd hAddRef; hwnd hRelease; hwnd hAddTriangles; hwnd hClose;", $pSink)

		DllCallbackFree($tSink.hQueryInterface)
		DllCallbackFree($tSink.hAddRef)
		;DllCallbackFree($tSink.hRelease)
		DllCallbackFree($tSink.hAddTriangles)
		DllCallbackFree($tSink.hClose)

		Return 0
	EndIf

	Return $tSink.iRefCnt
EndFunc   ;==>_Release




Func _AddTriangles($pSink, $pPnts, $iPnt)
	If Not $pSink Then Return

	Local $tSink = DllStructCreate("ptr lpVtbl; int iRefCnt; ptr pQueryInterface; ptr pAddRef; ptr pRelease; ptr pAddTriangles; ptr pClose; hwnd hQueryInterface; hwnd hAddRef; hwnd hRelease; hwnd hAddTriangles; hwnd hClose; int iMax; ptr pTriangles;", $pSink)
	Local $tTriangles = DllStructCreate("float Pnts[" & ($tSink.iMax + 1) * 6 & "]; uint Cnt;", $tSink.pTriangles)

	If ($tTriangles.Cnt + $iPnt) < $tSink.iMax Then
		DllCall("kernel32.dll", "none", "RtlMoveMemory", "struct*", DllStructGetPtr($tTriangles, "Pnts") + 24 * $tTriangles.Cnt, "struct*", $pPnts, "ulong_ptr", $iPnt * 24)
		$tTriangles.Cnt += $iPnt
	EndIf
EndFunc   ;==>_AddTriangles


Func _Close($pSink)
	If Not $pSink Then Return
EndFunc   ;==>_Close





Func _CreateTessellationSinkOBJ($tSink)
	If Not IsDllStruct($tSink) Then Return SetError(1, 1, False)

	Local $oSink = ObjCreateInterface(DllStructGetPtr($tSink), $sIID_ID2D1TessellationSink, $tagID2D1TessellationSink)
	If Not IsObj($oSink) Then Return SetError(1, 2, False)

	Return $oSink
EndFunc   ;==>_CreateTessellationSinkOBJ



Func _CreateTessellationSink(ByRef $hRelease, $tTriangles, $iMaxTriangles)

	Local $tSink = DllStructCreate("ptr lpVtbl; int iRefCnt; ptr pQueryInterface; ptr pAddRef; ptr pRelease; ptr pAddTriangles; ptr pClose; hwnd hQueryInterface; hwnd hAddRef; hwnd hRelease; hwnd hAddTriangles; hwnd hClose; int iMax; ptr pTriangles;")
	$tSink.lpVtbl = DllStructGetPtr($tSink, "pQueryInterface")

	$tSink.hQueryInterface = DllCallbackRegister("_QueryInterface", "none", "ptr;ptr;ptr*")
	$tSink.pQueryInterface = DllCallbackGetPtr($tSink.hQueryInterface)

	$tSink.hAddRef = DllCallbackRegister("_AddRef", "uint", "ptr")
	$tSink.pAddRef = DllCallbackGetPtr($tSink.hAddRef)

	$tSink.hRelease = DllCallbackRegister("_Release", "uint", "ptr")
	$tSink.pRelease = DllCallbackGetPtr($tSink.hRelease)

	$tSink.hAddTriangles = DllCallbackRegister("_AddTriangles", "none", "ptr;ptr;uint")
	$tSink.pAddTriangles = DllCallbackGetPtr($tSink.hAddTriangles)

	$tSink.hClose = DllCallbackRegister("_Close", "uint", "ptr")
	$tSink.pClose = DllCallbackGetPtr($tSink.hClose)

	$tSink.iRefCnt = 1;?!

	$tSink.iMax = $iMaxTriangles
	$tSink.pTriangles = DllStructGetPtr($tTriangles)

	$hRelease = $tSink.hRelease
	Return $tSink
EndFunc   ;==>_CreateTessellationSink


























Func _CreateTextOutlinePath(ByRef Const $oFactory, $sString, ByRef Const $oFontFace, $fSize, $fX = 0, $fY = 0, $fW = 0, $fH = 0)
	Local $iStrLen = StringLen($sString)

	Local $tGlyphIndices = _DWrite_FontFace_GetGlyphIndices($oFontFace, $sString)

	Local $oPath = _D2D_Factory_CreatePathGeometry($oFactory)
	Local $oSink = _D2D_PathGeometry_Open($oPath)
	_DWrite_FontFace_GetGlyphRunOutline($oFontFace, $oSink, $fSize, $tGlyphIndices, $iStrLen)
	$oSink.Close()
	$oSink = 0

	Local $tBounds = _D2D_Geometry_GetBounds($oPath)

	Local $fScaleX = 1, $fScaleY = 1
	If $fW <> 0 Then $fScaleX = $fW / ($tBounds.Right - $tBounds.Left)
	If $fH <> 0 Then $fScaleY = $fH / ($tBounds.Bottom - $tBounds.Top)

	Local $tMatrix = DllStructCreate($tagD2D1_MATRIX_3X2_F)
	$tMatrix.M11 = $fScaleX
	$tMatrix.M22 = $fScaleY
	$tMatrix.M31 = $fX - $tBounds.Left * $fScaleX
	$tMatrix.M32 = $fY - $tBounds.Top * $fScaleY

	Local $oPathTrans = _D2D_Factory_CreateTransformedGeometry($oFactory, $oPath, $tMatrix)
	$oPath = 0

	Return $oPathTrans
EndFunc   ;==>_CreateTextOutlinePath



Func _Exit()
	Exit
EndFunc   ;==>_Exit
