#AutoIt3Wrapper_AU3Check_Parameters=-d -w 1 -w 2 -w 3 -w 4 -w 5 -w 6
;#AutoIt3Wrapper_UseX64 = y

#include "..\Direct2D_1.au3"
#include "..\DirectWrite.au3"

Opt("MustDeclareVars", 1)
Global $oError = ObjEvent("AutoIt.Error", "_ErrFunc")
Func _ErrFunc()
	ConsoleWrite("COM Error, ScriptLine(" & $oError.ScriptLine & ") : Number 0x" & Hex($oError.Number, 8) & " - " & $oError.WinDescription & @CRLF)
EndFunc   ;==>_ErrFunc



Global $iGui_W = 800
Global $iGui_H = 500

Global $hGui = GUICreate("Direct2D", $iGui_W, $iGui_H)

Global $oFactory = _D2D_Factory1_Create()
Global $oContext, $oSwapChain
_D2D_CreateHWNDContext($oFactory, $hGui, $oContext, $oSwapChain)
If @error Then Exit MsgBox(0, "Error", _D2D_ErrorMessage(@error) & @CRLF & "You need Win8 or Win7+PlatformUpdate to run this example")

Global $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oContext, 1, 1, 1)

Global $oDW_Factory = _DWrite_Factory_Create()
Global $oFontFiles = _DWrite_Factory_CreateFontFileReference($oDW_Factory, @ScriptDir & "\Ethnocentric.ttf")
Global $oFontFace = _DWrite_Factory_CreateFontFace($oDW_Factory, $oFontFiles)
$oFontFiles = 0

Global $oPath = _CreateTextOutlinePath($oFactory, "A", $oFontFace, 96, $iGui_W * 0.2, $iGui_H * 0.2, $iGui_W * 0.6, $iGui_H * 0.6)

GUISetState()

$oContext.BeginDraw()
$oContext.Clear(_D2D1_COLOR_F(0, 0, 0.1, 1))
$oContext.FillGeometry($oPath, $oBrush, 0)
_DrawGeometryGlow($oContext, $oPath, 40, 1, 0.3, 1, 0.7, 0.8)
_DrawGeometryGlow($oContext, $oPath, 18, 2, 0, 1, 0.3, 1)
$oContext.EndDraw(0, 0)
$oSwapChain.Present(1, 0)

While GUIGetMsg() <> -3
WEnd

$oDW_Factory = 0
$oFontFace = 0

$oPath = 0

$oBrush = 0
$oContext = 0
$oSwapChain = 0
$oFactory = 0

GUIDelete($hGui)





Func _DrawGeometryGlow(ByRef Const $oContext, ByRef Const $oGeometry, $fGlow = 12, $iMode = 3, $fRed = 1, $fGreen = 1, $fBlue = 1, $fAlpha = 1)
	Local $oRender = _D2D_RenderTarget_CreateCompatibleRenderTarget($oContext)
	Local $oBitmap = _D2D_BitmapRenderTarget_GetBitmap($oRender)
	Local $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRender, $fRed, $fGreen, $fBlue, 1)

	$oRender.BeginDraw()
	$oRender.FillGeometry($oGeometry, $oBrush, 0)
	$oRender.EndDraw(0, 0)

	Local $oFX_Blur = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1GaussianBlur)
	$oFX_Blur.SetValue($D2D1_GAUSSIANBLUR_PROP_STANDARD_DEVIATION, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_FLOAT($fGlow), _D2D1_SIZEOF("float"))
	$oFX_Blur.SetInput(0, $oBitmap, True)


	Local $oFX_Composite = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1Composite)
	Switch $iMode
		Case 1
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(4), _D2D1_SIZEOF("uint"))
		Case 2
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(5), _D2D1_SIZEOF("uint"))
		Case Else
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(8), _D2D1_SIZEOF("uint"))
	EndSwitch
	$oFX_Composite.SetInput(0, $oBitmap, True)
	_D2D_Effect_SetInputEffect($oFX_Composite, $oFX_Blur, 1)


	Local $oFX_Arithmetic = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1ArithmeticComposite)
	_D2D_Properties_SetValue($oFX_Arithmetic, $D2D1_ARITHMETICCOMPOSITE_PROP_COEFFICIENTS, $D2D1_PROPERTY_TYPE_VECTOR4, _D2D1_VECTOR_4F(0, 0, $fAlpha, 0))

	$oFX_Arithmetic.SetInput(0, $oBitmap, True)
	_D2D_Effect_SetInputEffect($oFX_Arithmetic, $oFX_Composite, 1)

	Local $pOutPut
	$oFX_Arithmetic.GetOutput($pOutPut)

	$oContext.DrawImage($pOutPut, Null, Null, $D2D1_INTERPOLATION_MODE_CUBIC, $D2D1_COMPOSITE_MODE_SOURCE_OVER)
EndFunc   ;==>_DrawGeometryGlow





Func _DrawGeometryGlow_Stroke(ByRef Const $oContext, ByRef Const $oGeometry, $fGlow = 12, $fStrokeW = 10, $iMode = 2, $fRed = 1, $fGreen = 1, $fBlue = 1, $fAlpha = 1)
	Local $oRender = _D2D_RenderTarget_CreateCompatibleRenderTarget($oContext)
	Local $oBitmap = _D2D_BitmapRenderTarget_GetBitmap($oRender)
	Local $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRender, $fRed, $fGreen, $fBlue, 1)

	$oRender.BeginDraw()
	$oRender.DrawGeometry($oGeometry, $oBrush, $fStrokeW, 0)
	$oRender.EndDraw(0, 0)

	Local $oFX_Blur = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1GaussianBlur)
	$oFX_Blur.SetValue($D2D1_GAUSSIANBLUR_PROP_STANDARD_DEVIATION, $D2D1_PROPERTY_TYPE_FLOAT, _D2D1_FLOAT($fGlow), _D2D1_SIZEOF("float"))
	$oFX_Blur.SetInput(0, $oBitmap, True)


	Local $oFX_Composite = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1Composite)
	Switch $iMode
		Case 1
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(4), _D2D1_SIZEOF("uint"))
		Case 2
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(5), _D2D1_SIZEOF("uint"))
		Case Else
			$oFX_Composite.SetValue($D2D1_COMPOSITE_PROP_MODE, $D2D1_PROPERTY_TYPE_ENUM, _D2D1_UINT(8), _D2D1_SIZEOF("uint"))
	EndSwitch
	$oFX_Composite.SetInput(0, $oBitmap, True)
	_D2D_Effect_SetInputEffect($oFX_Composite, $oFX_Blur, 1)


	Local $oFX_Arithmetic = _D2D_DeviceContext_CreateEffect($oContext, $sIID_D2D1ArithmeticComposite)
	$oFX_Arithmetic.SetValue($D2D1_ARITHMETICCOMPOSITE_PROP_COEFFICIENTS, $D2D1_PROPERTY_TYPE_VECTOR4, _D2D1_VECTOR_4F(0, 0, $fAlpha, 0), _D2D1_SIZEOF($tagD2D1_VECTOR_4F))
	$oFX_Arithmetic.SetInput(0, $oBitmap, True)
	_D2D_Effect_SetInputEffect($oFX_Arithmetic, $oFX_Composite, 1)

	Local $pOutPut
	$oFX_Arithmetic.GetOutput($pOutPut)

	$oContext.DrawImage($pOutPut, Null, Null, $D2D1_INTERPOLATION_MODE_CUBIC, $D2D1_COMPOSITE_MODE_SOURCE_OVER)
EndFunc   ;==>_DrawGeometryGlow_Stroke











Func _CreateTextOutlinePath(ByRef Const $oFactory, $sString, ByRef Const $oFontFace, $fSize, $fX = 0, $fY = 0, $fW = 0, $fH = 0)
	Local $iStrLen = StringLen($sString)

	Local $tGlyphIndices = _DWrite_FontFace_GetGlyphIndices($oFontFace, $sString)

	Local $oPath = _D2D_Factory_CreatePathGeometry($oFactory)
	Local $oSink = _D2D_PathGeometry_Open($oPath)
	_DWrite_FontFace_GetGlyphRunOutline($oFontFace, $oSink, $fSize, $tGlyphIndices, $iStrLen)
	$oSink.Close()
	$oSink = 0

	Local $tBounds = _D2D_Geometry_GetBounds($oPath)

	Local $fScaleX = 1, $fScaleY = 1
	If $fW <> 0 Then $fScaleX = $fW / ($tBounds.Right - $tBounds.Left)
	If $fH <> 0 Then $fScaleY = $fH / ($tBounds.Bottom - $tBounds.Top)

	Local $tMatrix = DllStructCreate($tagD2D1_MATRIX_3X2_F)
	$tMatrix.M11 = $fScaleX
	$tMatrix.M22 = $fScaleY
	$tMatrix.M31 = $fX - $tBounds.Left * $fScaleX
	$tMatrix.M32 = $fY - $tBounds.Top * $fScaleY

	Local $oPathTrans = _D2D_Factory_CreateTransformedGeometry($oFactory, $oPath, $tMatrix)
	$oPath = 0

	Return $oPathTrans
EndFunc   ;==>_CreateTextOutlinePath
