#include-once
#include "Direct2D\Direct2D.au3"
#include "Direct2D\DirectWrite.au3"
#include "Direct2D\WIC.au3"
#include <GDIPlus.au3>

; Version: 13.06.12
; Version: 30.03.16

; #INDEX# =======================================================================================================================
; Title .........: EasyD2D
; AutoIt Version : 3.3.12.0
; Language ......: English
; Description ...: Wrapper for the Direct2D-UDF from trancexx and Eukalyptus
; Author ........: Mars
; Modified ......:
; Dll ...........: dwrite.dll
; ===============================================================================================================================

Global $oD2DFactory = 0 ; Factory Objekt
Global $oD2DWriteFactory = 0 ; WriteFactory
Global $oD2DRenderTarget = 0 ; Aktuelles RenderTarget
Global $iD2DFrameRate = 0 ; Aktuelle Framerate
Global $iD2DFrameCounter = 0 ; Aktueller Frame
Global $iD2DFrameTimer = TimerInit() ; Frame Timer
Global $nD2DThreadLoad = 0 ; Aktuelle Auslastung in %
Global $nD2DThreadLoadCounter ; Auslastungszhler

; ##### Hier entstehen neue Funktionen #####

Func _ED2D_BitmapCreateFromMemory($iW, $iH, $pPixelData = Null, $iAlphaMode = 0)
	Return _D2D_RenderTarget_CreateBitmap($oD2DRenderTarget, $iW, $iH, $iW*4, $pPixelData, 0, $iAlphaMode)
EndFunc

Func _ED2D_TextFormatCreate($sFont = 'Arial', $fSize = 12, $iWeight = 400, $iAlign = -1, $iStyle = 0, $iStretch = 5)
	Local $oFormat = _DWrite_Factory_CreateTextFormat($oD2DWriteFactory, $sFont, $fSize, $iWeight, $iStyle, $iStretch)
	; Align:
	; 0 -> xLeft
	; 1 -> xRight
	; 2 -> xMid
	If $iAlign > -1 Then $oFormat.SetTextAlignment($iAlign)
	Return $oFormat
EndFunc

Func _ED2D_RenderTargetDrawString($oRenderTarget, $sString, $oFormat, $oBrush, $tRect)
	$oRenderTarget.DrawText($sString, StringLen($sString), $oFormat, $tRect, $oBrush, 0, 0)
EndFunc

; ##### Hier entstehen neue Funktionen #####

; ---------------------------------------------------------------
; Description    : Draws a bitmap on a d2d-Rendertarget
; Return Success : none
; Return Failure : none
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawBitmap($oRenderTarget, $iX_Dst, $iY_Dst, $iW_Dst, $iH_Dst, $oBitmap, $iX_Src = -1, $iY_Src = -1, $iW_Src = -1, $iH_Src = -1, $fAlpha = 1, $InterpolationMode = 1)
	$oRenderTarget.DrawBitmap($oBitmap, Rect4F($iX_Dst, $iY_Dst, $iW_Dst, $iH_Dst), $fAlpha, $InterpolationMode, Rect4F($iX_Src, $iY_Src, $iW_Src, $iH_Src))
EndFunc

; ---------------------------------------------------------------
; Description    : Creates a D2D-Bitmap from file
; Return Success : D2D-BitmapObject
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_BitmapCreateFromFile($sFileName, $oRenderTarget = $oD2DRenderTarget)
	Local $oWICFactory = _WIC_ImagingFactory_Create()
	Local $oDecoder = _WIC_ImagingFactory_CreateDecoderFromFilename($oWICFactory, $sFileName)
	Local $oBitmapWIC = _WIC_BitmapDecoder_GetFrame($oDecoder)
	Local $oFormatConverter = _WIC_ImagingFactory_CreateFormatConverter($oWICFactory)
	Local $hr = $oFormatConverter.Initialize($oBitmapWIC, _WinAPI_GUIDFromString($sGUID_WICPixelFormat32bppPBGRA), $WICBitmapDitherTypeNone, Null, 0, $WICBitmapPaletteTypeMedianCut)
	Local $oBitmapD2D = _D2D_RenderTarget_CreateBitmapFromWicBitmap($oRenderTarget, $oFormatConverter, 0, 1)
	If Not IsObj($oBitmapD2D) Then Return SetError(1, 0, False)
	Return $oBitmapD2D
EndFunc

; ---------------------------------------------------------------
; Description    : Disposes a D2D-BitmapObject
; Return Success : None - Should always succeed
; Return Failure : None - Should always succeed
; ---------------------------------------------------------------
Func _ED2D_BitmapDispose(ByRef $oBitmap)
	$oBitmap = 0
EndFunc   ;==>_D2D_BitmapDispose

; ---------------------------------------------------------------
; Description    : Creates a solid Brush
; Return Success : D2D-BrushObject
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_BrushCreateSolid($oRenderTarget, $iARGB = 0xFF000000)
	If Not __ED2D_CheckValidColor($iARGB) Then Return SetError(0x06, 0, False)
	If Not IsObj($oRenderTarget) Then Return SetError(0x07, 0, False)
	Local $iA = BitAND(BitShift($iARGB, 24), 0xFF) / 0xFF
	Local $iR = BitAND(BitShift($iARGB, 16), 0xFF) / 0xFF
	Local $iG = BitAND(BitShift($iARGB, 8), 0xFF) / 0xFF
	Local $iB = BitAND($iARGB, 0xFF) / 0xFF
	Local $oBrush = _D2D_RenderTarget_CreateSolidColorBrush($oRenderTarget, $iR, $iG, $iB, $iA)
	If Not IsObj($oBrush) Then Return SetError(0x08, 0, False)
	Return $oBrush
EndFunc   ;==>_D2D_BrushCreateSolid

; ---------------------------------------------------------------
; Description    : Sets Color of a D2D-BrushObject
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_BrushSetSolidColor(ByRef $oBrush, $iARGB)
	If Not IsObj($oBrush) Then Return SetError(0x11, 0, False)
	If Not __ED2D_CheckValidColor($iARGB) Then Return SetError(0x06, 0, False)
    $oBrush.SetColor(_ED2D_StructColorCreate($iARGB))
	Return True
EndFunc

; ---------------------------------------------------------------
; Description    : Disposes a D2D-BrushObject
; Return Success : None - Should always succeed
; Return Failure : None - Should always succeed
; ---------------------------------------------------------------
Func _ED2D_BrushDispose(ByRef $oBrush)
	$oBrush = 0
EndFunc   ;==>_D2D_BrushDispose

; ---------------------------------------------------------------
; Description    : Returns actual framerate
; Return Success : Integer
; Return Failure : None - Should always succeed
; ---------------------------------------------------------------
Func _ED2D_GetFrameRate()
	Return $iD2DFrameRate
EndFunc   ;==>_D2D_GetFrameRate

; ---------------------------------------------------------------
; Description    : Returns load of AutoItScript in %
; Return Success : Integer (0 -> 100)
; Return Failure : None - Should always succeed
; ---------------------------------------------------------------
Func _ED2D_GetLoad()
	Return Round($nD2DThreadLoad * 100, 0)
EndFunc   ;==>_D2D_GetLoad

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.BeginDraw
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetBeginDraw(ByRef Const $oRenderTarget)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	$oRenderTarget.BeginDraw()
	Return True
EndFunc   ;==>_D2D_RenderTargetBeginDraw

; ---------------------------------------------------------------
; Description    : Clears content of D2D-RenderTarget with some color
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetClear(ByRef Const $oRenderTarget, $iARGB = 0xFF000000)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	If Not __ED2D_CheckValidColor($iARGB) Then Return SetError(0x06, 0, False)
	$oRenderTarget.Clear(_ED2D_StructColorCreate($iARGB))
	Return True
EndFunc   ;==>_D2D_RenderTargetClear

; ---------------------------------------------------------------
; Description    : Creates a D2D-RenderTarget linked to a HWND
; Return Success : D2D-RenderTarget
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetCreateFromHWND(ByRef Const $hGUI, $iW = 0, $iH = 0, $bSetMain = False)
	If Not IsHWnd($hGUI) Then Return SetError(0x09, 0, False)
	Local $oRenderTarget = _D2D_Factory_CreateHwndRenderTarget($oD2DFactory, $hGUI, $iW, $iH, 0, 2)
	If Not IsObj($oRenderTarget) Then Return SetError(0x08, 0, False)
	If $bSetMain Then _ED2D_SetMainRenderTarget($oRenderTarget)
	Return $oRenderTarget
EndFunc   ;==>_D2D_RenderTargetCreateFromHWND

; ---------------------------------------------------------------
; Description    : Creates a D2D-RenderTarget linked to another D2D-RenderTarget
; Return Success : D2D-RenderTarget
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetCreateFromRenderTarget(ByRef Const $oRenderTarget)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	Local $oCompatibleRenderTarget = _D2D_RenderTarget_CreateCompatibleRenderTarget($oRenderTarget, 0, 0, 0, 0)
	If Not IsObj($oCompatibleRenderTarget) Then Return SetError(0x08, 0, False)
	Return $oCompatibleRenderTarget
EndFunc   ;==>_D2D_RenderTargetCreateFromRenderTarget

; ---------------------------------------------------------------
; Description    : Disposes a D2D-RenderTarget
; Return Success : None - Should always succeed
; Return Failure : None - Should always succeed
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDispose(ByRef $oRenderTarget)
	$oRenderTarget = 0
EndFunc   ;==>_D2D_RenderTargetDispose

; ---------------------------------------------------------------
; Description    : Display a D2D-Bitmap on the main D2D-RenderTarget
; Return Success : I do not know how to check for errors here
; Return Failure : I do not know how to check for errors here
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawBitmapOnHWND(ByRef Const $oBitmap, $fAlpha = 1)
	Local Static $iTimer = TimerInit()
	Local $iTimerDiff = TimerDiff($iTimer)
	Local $nFrameDiff = (1000 / $iD2DFrameRate - $iTimerDiff)
	If $nFrameDiff < 0 Then $nFrameDiff = 0
	$oD2DRenderTarget.BeginDraw()
	$oD2DRenderTarget.DrawBitmap($oBitmap, Null, $fAlpha, 0, Null)
	$oD2DRenderTarget.EndDraw(0, 0)
	$iTimer = TimerInit()
	$iD2DFrameCounter += 1
	$nD2DThreadLoadCounter += $iTimerDiff / ($iTimerDiff + $nFrameDiff)
	If TimerDiff($iD2DFrameTimer) > 1000 Then
		$iD2DFrameTimer = TimerInit()
		$iD2DFrameRate = $iD2DFrameCounter
		$nD2DThreadLoad = $nD2DThreadLoadCounter / $iD2DFrameCounter
		$iD2DFrameCounter = 0
		$nD2DThreadLoadCounter = 0
	EndIf
EndFunc   ;==>_D2D_RenderTargetDrawBitmapOnHWND

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.EndDraw
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetEndDraw(ByRef Const $oRenderTarget)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	$oRenderTarget.EndDraw(0, 0)
	Return True
EndFunc   ;==>_D2D_RenderTargetEndDraw

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.FillEllipse
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetFillEllipse(ByRef Const $oRenderTarget, $iX, $iY, $iW, $iH, ByRef Const $oBrush)
	Local $tEllipse = _ED2D_StructEllipseCreate($iX, $iY, $iW, $iH)
    $oRenderTarget.FillEllipse(DllStructGetPtr($tEllipse), $oBrush)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.FillEllipse
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetFillEllipseEx(ByRef Const $oRenderTarget, $pEllipse, ByRef Const $oBrush)
    $oRenderTarget.FillEllipse($pEllipse, $oBrush)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.FillRectangle
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetFillRect(ByRef Const $oRenderTarget, $iX, $iY, $iW, $iH, ByRef Const $oBrush)
	Local $tRect = Rect4F($iX, $iY, $iW, $iH)
	$oRenderTarget.FillRectangle(DllStructGetPtr($tRect), $oBrush)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.FillRectangle
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetFillRectEx(ByRef Const $oRenderTarget, $pRect, ByRef Const $oBrush)
	$oRenderTarget.FillRectangle($pRect, $oBrush)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.DrawEllipse
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawEllipse(ByRef Const $oRenderTarget, $iX, $iY, $iW, $iH, ByRef Const $oBrush, $iPenWidth = 1, $Unbekannt = 0)
	Local $tEllipse = _ED2D_StructEllipseCreate($iX, $iY, $iW, $iH)
    $oRenderTarget.DrawEllipse(DllStructGetPtr($tEllipse), $oBrush, $iPenWidth, $Unbekannt)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.DrawEllipse
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawEllipseEx(ByRef Const $oRenderTarget, $pEllipse, ByRef Const $oBrush, $iPenWidth = 1, $Unbekannt = 0)
    $oRenderTarget.DrawEllipse($pEllipse, $oBrush, $iPenWidth, $Unbekannt)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.DrawRectangle
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawRect(ByRef Const $oRenderTarget, $iX, $iY, $iW, $iH, ByRef Const $oBrush, $iPenWidth = 1, $Unbekannt = 0)
	Local $tRect = Rect4F($iX, $iY, $iW, $iH)
	$oRenderTarget.DrawRectangle(DllStructGetPtr($tRect), $oBrush, $iPenWidth, $Unbekannt)
EndFunc

; ---------------------------------------------------------------
; Description    : Wrapper for RenderTarget.DrawRectangle
; Return Success : None
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_RenderTargetDrawRectEx(ByRef Const $oRenderTarget, $pRect, ByRef Const $oBrush, $iPenWidth = 1, $Unbekannt = 0)
	$oRenderTarget.DrawRectangle($pRect, $oBrush, $iPenWidth, $Unbekannt)
EndFunc

; ---------------------------------------------------------------
; Description    : Returns a D2D-BitmapObject linked to the D2D-RenderTarget
; Return Success : D2D-BitmapObject
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_RenderTargetGetBitmap(ByRef Const $oRenderTarget)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	Local $oBitmap = _D2D_BitmapRenderTarget_GetBitmap($oRenderTarget)
	If Not IsObj($oBitmap) Then Return SetError(0x08, 0, False)
	Return $oBitmap
EndFunc   ;==>_D2D_RenderTargetGetBitmap

; ---------------------------------------------------------------
; Description    : Sets Main D2D-RenderTarget
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_SetMainRenderTarget(ByRef Const $oRenderTarget)
	If Not IsObj($oRenderTarget) Then Return SetError(0x10, 0, False)
	$oD2DRenderTarget = $oRenderTarget
	Return True
EndFunc   ;==>_D2D_SetMainRenderTarget

; ---------------------------------------------------------------
; Description    : Releases D2D-FactoryObject
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_Shutdown()
	If IsObj($oD2DFactory) Then
		$oD2DFactory = 0
		Return True
	EndIf
	If $oD2DFactory = 0 Then Return SetError(0x04, 0, False)
	Return SetError(0x05, 0, False)
EndFunc   ;==>_D2D_Shutdown

; ---------------------------------------------------------------
; Description    : Creates D2D-FactoryObject
; Return Success : True
; Return Failure : False -> @error is set
; ---------------------------------------------------------------
Func _ED2D_Startup()
	If IsObj($oD2DFactory) And IsObj($oD2DWriteFactory) Then  Return SetError(0x02, 0, False)
	If $oD2DFactory = 0 Then $oD2DFactory = _D2D_Factory_Create()
	If $oD2DWriteFactory = 0 Then $oD2DWriteFactory = _DWrite_Factory_Create()
	If IsObj($oD2DFactory) And IsObj($oD2DWriteFactory) Then  Return True
	Return SetError(0x01, 0, False)
EndFunc   ;==>_D2D_Startup

; ---------------------------------------------------------------
; Description    : Creates a float[4] DllStruct containing RGBA colors
; Return Success : tARGB-Struct
; Return Failure : If ErrorCheck is enabled: False -> @error is set, else: None
; ---------------------------------------------------------------
Func _ED2D_StructColorCreate($iARGB, $bCheckError = False)
	If $bCheckError And Not __ED2D_CheckValidColor($iARGB) Then Return SetError(0x06, 0, False)
	Local $tColor = DllStructCreate('float R; float G; float B; float A')
	$tColor.A = BitAND(BitShift($iARGB, 24), 0xFF) / 0xFF
	$tColor.R = BitAND(BitShift($iARGB, 16), 0xFF) / 0xFF
	$tColor.G = BitAND(BitShift($iARGB, 8), 0xFF) / 0xFF
	$tColor.B = BitAND($iARGB, 0xFF) / 0xFF
	Return $tColor
EndFunc   ;==>_D2D_StructColorCreate

; ---------------------------------------------------------------
; Description    : Creates a float[4] DllStruct containing the
;                  upper left and lower right point of a rectangle
; Return Success : tRect-Struct
; Return Failure : None
; ---------------------------------------------------------------
Func Rect4F($iX = -1, $iY = -1, $iW = -1, $iH = -1)
	If $iX = -1 And $iY = -1 And $iW = -1 And $iH = -1 Then Return Null
	If IsDllStruct($iX) Then ; Cast: Rect4I -> Rect4F
		$iY = $iX.Y1
		$iW = $iX.X2 - $iX.X1
		$iH = $iX.Y2 - $iY
		$iX = $iX.X1
	EndIf
	Local $tRect = DllStructCreate('float X1; float Y1; float X2; float Y2')
	$tRect.X1 = $iX
    $tRect.Y1 = $iY
    $tRect.X2 = $iW + $iX
    $tRect.Y2 = $iH + $iY
	Return $tRect
EndFunc

; ---------------------------------------------------------------
; Description    : Creates a int[4] DllStruct containing the
;                  upper left and lower right point of a rectangle
; Return Success : tRect-Struct
; Return Failure : None
; ---------------------------------------------------------------
Func Rect4I($iX = -1, $iY = -1, $iW = -1, $iH = -1)
	If $iX = -1 And $iY = -1 And $iW = -1 And $iH = -1 Then Return Null
	If IsDllStruct($iX) Then ; Cast: Rect4F -> Rect4I
		$iY = $iX.Y1
		$iW = $iX.X2 - $iX.X1
		$iH = $iX.Y2 - $iY
		$iX = $iX.X1
	EndIf
	Local $tRect = DllStructCreate('int X1; int Y1; int X2; int Y2')
	$tRect.X1 = $iX
    $tRect.Y1 = $iY
    $tRect.X2 = $iW + $iX
    $tRect.Y2 = $iH + $iY
	Return $tRect
EndFunc

Func Point2I($iX = -1, $iY = -1)
	If $iX = -1 And $iY = -1 Then Return Null
	Local $tPoint = DllStructCreate('int X; int Y')
	$tPoint.X = $iX
	$tPoint.Y = $iY
	Return $tPoint
EndFunc

; ---------------------------------------------------------------
; Description    : Creates a float[4] DllStruct containing ellipse coords
; Return Success : tEllipse-Struct
; Return Failure : None
; ---------------------------------------------------------------
Func _ED2D_StructEllipseCreate($iX = 0, $iY = 0, $iW = 0, $iH = 0)
    Local $tEllipse = DllStructCreate('float cX; float cY; float rX; float rY')
    $tEllipse.rX = $iW/2
    $tEllipse.rY = $iH/2
	$tEllipse.cX = $iX + $tEllipse.rX
    $tEllipse.cY = $iY + $tEllipse.rY
    Return $tEllipse
EndFunc

; ---------------------------------------------------------------
; Description    : String representation of D2D-ErrorCodes
; Return Success : String
; Return Failure : 'Invalid errorcode'
; ---------------------------------------------------------------
Func _ED2D_ErrorCode($iError)
	Local Static $aError[] = [ _
			'No error', _ ; ------------------------ 0x00
			'Initialisation failed', _ ; ----------- 0x01
			'Factory already exists', _ ; ---------- 0x02
			'$oD2DFactory contains no object', _ ; - 0x03
			'Factory does not exist', _ ; ---------- 0x04
			'$oD2DFactory contains data', _ ; ------ 0x05
			'Invalid color', _ ; ------------------- 0x06
			'No main D2D-RenderTarget set', _ ; ---- 0x07
			'Object creation failed', _ ; ---------- 0x08
			'Invalid GUI-Handle', _ ; -------------- 0x09
			'Invalid D2D-RenderTarget', _ ; -------- 0x10
			'Invalid D2D-BrushObject' _ ; ---------- 0x11
			], $iUBound = UBound($aError)
	If IsInt($iError) And $iError >= 0 And $iError < $iUBound Then Return $aError[$iError]
	Return 'Invalid errorcode'
EndFunc   ;==>_D2D_ErrorCode

; ##################
; INTERNAL FUNCTIONS
; ##################

Func __ED2D_CheckValidColor(ByRef Const $iARGB)
	Return (StringIsInt($iARGB) Or ((StringLeft($iARGB, 2) = '0x') And StringIsXDigit(StringTrimLeft($iARGB, 2)) And StringLen($iARGB) = 10))
	; [Heute, 19:49] James: wie wre es mit StringIsInt($col) Or StringRexExp($col, "(?i)^0x[0-9A-F]+$")
	; [Heute, 19:50] James: Oder {8} statt +, wenn es genau 32 bit sein sollen.
EndFunc   ;==>__D2D_CheckValidColor
